<?php

// Load necessary classes and functions
require_once __DIR__ . '/simplify/autoload.php';
require_once __DIR__ . '/simplify/functions.php';
require_once __DIR__ . '/simplify/config.php';

// Load modules and register hooks
$modules = ModuleLoader::loadModules();
ModuleLoader::requiresModules(["SimpleBlog"]);

// Initialize the SimpleBlog class and other dependencies
$blog = new SimpleBlog();
$entriesPerPage = ModuleLoader::getSetting('SimpleBlog', 'postsPerPage') ?? 10;
$pagination = new Pagination($entriesPerPage);

// Fetch alias from GET parameter
$alias = getAlias();

// Get post by alias
if ($alias) {
    $blog->getEntryByAlias($alias);
}

// Fetch meta tags for SEO (Search Engine Optimization)
$metaTags = $blog->getMetaTags();

// Include header file for the page layout
include_once __DIR__ . '/header.php';

// Get messages for the current page (if any)
$messages = Message::get();

// Render page based on the type of request
if ($alias) {
    // Show single post if alias is provided
    renderSinglePostPage($blog, $messages, $alias);
} else {
    // Show all posts or posts filtered by hashtag
    renderPostListPage($blog, $messages);
}
?>

<!-- JavaScript for the Smileys module -->
<script>
    // Wait for the DOM to be fully loaded
    document.addEventListener('DOMContentLoaded', function () {
        // Attach click event to each smiley image
        document.querySelectorAll('.smiley').forEach(function (smiley) {
            smiley.addEventListener('click', function () {
                var smileyCode = smiley.getAttribute('data-smiley');

                // Get all textareas with the "smiley-support" class
                document.querySelectorAll('.smiley-support').forEach(function (textarea) {
                    // Get the current cursor position
                    var cursorPos = textarea.selectionStart;

                    // Get the text before and after the cursor
                    var textBefore = textarea.value.substring(0, cursorPos);
                    var textAfter = textarea.value.substring(cursorPos);

                    // Remove any extra spaces before or after the smiley
                    textBefore = textBefore.replace(/\s+$/, ''); // Remove trailing spaces before the smiley
                    textAfter = textAfter.replace(/^\s+/, ''); // Remove leading spaces after the smiley

                    // Insert the smiley code at the current cursor position, with a single space before it
                    textarea.value = textBefore + ' ' + smileyCode + ' ' + textAfter;

                    // Move the cursor to the right after the inserted smiley code
                    textarea.selectionStart = textarea.selectionEnd = cursorPos + smileyCode.length + 2; // +2 for space before and after the smiley
                });
            });
        });
    });
</script>

<?php
// Include footer file for the page layout
include_once __DIR__ . '/footer.php';

/**
 * Helper function to get alias from URL
 */
function getAlias()
{
    return isset($_GET['alias']) ? sanitizeInput(trim($_GET['alias'])) : null;
}

/**
 * Renders the page for a single post, including the post, comments, and comment form.
 */
function renderSinglePostPage($blog, $messages, $alias)
{
    // Get entry data based on alias
    $entryData = $blog->getEntryByAlias($alias);

    if (!$entryData) {
        die($blog::ERROR_MESSAGES['entry_not_found']);
    }

    // Handle comment form submission
    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
        // Clear any previous messages before processing the new form submission
        Message::clear();

        // Validate and sanitize input
        $name = isset($_POST['name']) ? trim($_POST['name']) : '';
        $email = isset($_POST['email']) ? trim($_POST['email']) : '';
        $website = isset($_POST['website']) ? trim($_POST['website']) : '';
        $commentText = isset($_POST['comment']) ? trim($_POST['comment']) : '';
        $captcha = isset($_POST['captcha']) ? trim($_POST['captcha']) : '';
        $hashedAnswer = isset($_POST['captcha_hashed_answer']) ? trim($_POST['captcha_hashed_answer']) : '';

        // Attempt to add the comment
        $blog->addComment($entryData['id'], $name, $email, $website, $commentText, $captcha, $hashedAnswer);

        // Redirect to avoid old messages persisting across requests
        safeRedirect(getScriptUrl());
    }

    // Get comments for the specific post
    $comments = $blog->getCommentsByPostId($entryData['id']);
    $entryData['messages'] = $messages;
    $entryData['comments'] = $comments;

    $templateSetting = ModuleLoader::getSetting('SimpleBlog', 'oneTemplate') ?? false;

    // Conditionally render the post template based on 'oneTemplate' setting
    if ($templateSetting) {
        renderPostTemplate($blog, $entryData); // Use post.php template
    } else {
        renderViewPostTemplate($blog, $entryData); // Use view_post.php template
    }

    renderAddCommentFormTemplate($blog, $entryData);
    renderCommentsTemplate($blog, $entryData);
}

/**
 * Renders the list of blog posts with pagination, optional hashtag filtering, and messages.
 */
function renderPostListPage($blog, $messages)
{
    $hashtag = isset($_GET['hashtag']) ? sanitizeInput(trim($_GET['hashtag'])) : null;

    if ($hashtag && !preg_match('/^[a-zA-Z0-9]+$/', $hashtag)) {
        die('Invalid hashtag.');
    }

    // Fetch entries, either by hashtag or all entries
    $entries = $hashtag ? $blog->getEntriesByHashtag($hashtag) : $blog->getAllEntries(SimpleBlog::POST_STATUS_PUBLISHED);
    $totalItems = count($entries);

    // Setup pagination
    global $pagination;
    $pagination->setTotalItems($totalItems);
    $currentPage = $pagination->getCurrentPage();
    $entries = array_slice($entries, ($currentPage - 1) * $pagination->getLimit(), $pagination->getLimit());

    // Prepare data for rendering
    $entryData['messages'] = $messages;

    // Display the posts or a message if none are found
    if (empty($entries)) {
        echo "<p>No blog entries found.</p>";
    } else {
        foreach ($entries as $entry) {
            $entryData = $entry;
            renderPostTemplate($blog, $entryData);
        }
    }

    // Render pagination links
    echo $hashtag
        ? $pagination->render(getScriptUrl(), ['hashtag' => urlencode($hashtag), 'customized' => true])
        : $pagination->render(getScriptUrl(), ['customized' => true]);
}

/**
 * Renders a single blog post using the post.php template.
 */
function renderPostTemplate($blog, $entryData)
{
    $entryTemplatePath = __DIR__ . DIRECTORY_SEPARATOR . basename(MODULES_DIR) . '/SimpleBlog/templates/post.php';
    echo $blog->loadAndParseTemplate($entryTemplatePath, $entryData, [$blog, 'parsePost']);
}

/**
 * Renders a single blog post using the view_post.php template.
 */
function renderViewPostTemplate($blog, $entryData)
{
    $entryTemplatePath = __DIR__ . DIRECTORY_SEPARATOR . basename(MODULES_DIR) . '/SimpleBlog/templates/view_post.php';
    echo $blog->loadAndParseTemplate($entryTemplatePath, $entryData, [$blog, 'parsePost']);
}

/**
 * Renders the comments section of a blog post.
 */
function renderCommentsTemplate($blog, $entryData)
{
    $commentsTemplatePath = __DIR__ . DIRECTORY_SEPARATOR . basename(MODULES_DIR) . '/SimpleBlog/templates/comments.php';
    echo $blog->loadAndParseTemplate($commentsTemplatePath, $entryData, [$blog, 'parseComments']);
}

/**
 * Renders the add comment form for a blog post.
 */
function renderAddCommentFormTemplate($blog, $entryData)
{
    $addCommentFormTemplatePath = __DIR__ . DIRECTORY_SEPARATOR . basename(MODULES_DIR) . '/SimpleBlog/templates/add_comment_form.php';
    echo $blog->loadAndParseTemplate($addCommentFormTemplatePath, $entryData, [$blog, 'parseCommentForm']);
}
