<?php
// Protects the module files from being accessed directly
if (!defined('ALLOW_DIRECT_ACCESS')) {
    die("Access denied.");
}

/**
 * Captcha class provides methods to generate and validate CAPTCHA questions.
 * It supports math and logic questions to protect against bots and spam.
 */
class Captcha
{
    // Current version
    const VERSION = "1.0.1";

    // Shows the current version of the module
    public function getVersion()
    {
        return self::VERSION;
    }

    const MATH_QUESTION_TYPE = 'math';
    const LOGIC_QUESTION_TYPE = 'logic';

    /**
     * List of available question types (math and logic).
     *
     * @var array
     */
    private $questionTypes = [
        self::MATH_QUESTION_TYPE,
        self::LOGIC_QUESTION_TYPE
    ];

    /**
     * Initializes the Captcha module with default settings.
     * Sets default settings for the CAPTCHA module including icon, label, description, and captcha options.
     */
    public function __construct()
    {
        ModuleLoader::setDefaultSettings(
            'Captcha',
            'moduleIcon',
            '', // No value here, it's just for the icon
            '', // No label here
            '', // No description
            'string', // Just a placeholder for type
            'fa-shield'
        );

        ModuleLoader::setDefaultSettings(
            'Captcha',
            'captcha',
            true,
            'Simple Captcha',
            'A CAPTCHA system that generates random questions to protect against bots and spam.',
            'bool',
            'fa-shield'
        );

        ModuleLoader::setDefaultSettings(
            'Captcha',
            'include_math_questions',
            true,
            'Include Math Questions',
            'Enable or disable the use of math questions in the CAPTCHA system.',
            'bool',
            'fa-calculator'
        );
    }

    /**
     * Generates a random CAPTCHA question (either math or logic) and returns the question along with a hashed answer.
     *
     * @return array Associative array containing 'question' and 'hashedAnswer'.
     */
    public function getQuestion()
    {
        $availableQuestionTypes = $this->getAvailableQuestionTypes();
        $questionType = $this->getRandomQuestionType($availableQuestionTypes);

        $questionAnswerPair = ($questionType === self::MATH_QUESTION_TYPE)
            ? $this->generateMathQuestion()
            : $this->generateLogicQuestion();

        $hashedAnswer = $this->hashAnswer($questionAnswerPair['answer']);

        return [
            'question' => $questionAnswerPair['question'],
            'hashedAnswer' => $hashedAnswer,
        ];
    }

    /**
     * Validates the user's CAPTCHA answer by comparing the hashed version of the user's input with the stored correct answer.
     *
     * @param string $userAnswer The user's input answer to the CAPTCHA question.
     * @param string $hashedAnswerFromForm The correct hashed answer from the form.
     * @return bool True if the user's answer is correct, false otherwise.
     */
    public function validateCaptcha($userAnswer, $hashedAnswerFromForm)
    {
        $userAnswer = strtolower(trim($userAnswer));
        $hashedUserAnswer = $this->hashAnswer($userAnswer);

        return hash_equals($hashedUserAnswer, $hashedAnswerFromForm);
    }

    /**
     * Hashes the provided answer using SHA-256 for consistent validation.
     *
     * @param string $answer The plain text answer to hash.
     * @return string The hashed answer (SHA-256).
     */
    private function hashAnswer($answer)
    {
        return hash('sha256', $answer);
    }

    /**
     * Returns the list of available question types based on configuration settings.
     * 
     * @return array List of available question types ('math' and/or 'logic').
     */
    private function getAvailableQuestionTypes()
    {
        $includeMathQuestions = ModuleLoader::getSetting('Captcha', 'include_math_questions') ?? true;
        $availableQuestionTypes = [self::LOGIC_QUESTION_TYPE];

        if ($includeMathQuestions) {
            $availableQuestionTypes[] = self::MATH_QUESTION_TYPE;
        }

        return $availableQuestionTypes;
    }

    /**
     * Selects a random question type from the available types.
     *
     * @param array $availableQuestionTypes List of available question types.
     * @return string Randomly selected question type ('math' or 'logic').
     */
    private function getRandomQuestionType(array $availableQuestionTypes)
    {
        return $availableQuestionTypes[array_rand($availableQuestionTypes)];
    }

    /**
     * Generates a simple math question (e.g., "What is 3 + 4?") and returns the question and its answer.
     *
     * @return array Associative array containing 'question' and 'answer' for a math question.
     */
    private function generateMathQuestion()
    {
        $num1 = rand(1, 5);
        $num2 = rand(1, 5);
        $answer = $num1 + $num2;

        return [
            'question' => "What is {$num1} + {$num2}?",
            'answer' => (string)$answer
        ];
    }

    /**
     * Generates a random logic question (e.g., "How many legs does a dog have?") and returns the question and its answer.
     *
     * @return array Associative array containing 'question' and 'answer' for a logic question.
     */
    private function generateLogicQuestion()
    {
        $questions = [
            ['question' => 'How many legs does a dog have?', 'answer' => '4'],
            ['question' => 'What is the opposite of hot?', 'answer' => 'cold'],
            ['question' => 'What color is the sky on a clear day?', 'answer' => 'blue'],
            ['question' => 'How many days are in a week?', 'answer' => '7'],
            ['question' => 'What is the opposite of up?', 'answer' => 'down'],
            ['question' => 'Which animal says "meow"?', 'answer' => 'cat'],
            ['question' => 'What is the color of grass?', 'answer' => 'green'],
            ['question' => 'How many months are in a year?', 'answer' => '12'],
            ['question' => 'What is the color of an orange?', 'answer' => 'orange'],
            ['question' => 'Which animal barks?', 'answer' => 'dog'],
            ['question' => 'What is the opposite of left?', 'answer' => 'right'],
        ];

        $randomQuestion = $questions[array_rand($questions)];
        return [
            'question' => $randomQuestion['question'],
            'answer' => strtolower($randomQuestion['answer'])
        ];
    }
}
