<?php
// Protects the module files from being accessed directly
if (!defined('ALLOW_DIRECT_ACCESS')) {
    die("Access denied.");
}

/**
 * Class Pages
 * 
 * This class provides methods to manage pages including creating, editing,
 * deleting, and retrieving pages from a JSON file, with message handling internally.
 */
class Pages
{
    // Current version
    const VERSION = "1.0.1";

    // Shows the current version of the module
    public function getVersion()
    {
        return self::VERSION;
    }
    
    // SimpleLogin class
    private SimpleLogin $auth;

    // The directory where uploaded files are stored
    public string $uploadDir;

    private const PAGES_DATA_FILE = '/pages/database.json';

    public const PAGE_STATUS_DRAFT = 0;
    public const PAGE_STATUS_PUBLISHED = 1;
    public const PAGE_STATUS_DELETED = 2;

    // Error and success messages stored as constants for easy translation
    public const ERROR_MESSAGES = [
        'page_not_found' => 'Failed to find the page.',
        'delete_failed' => 'Failed to delete the page.',
        'update_failed' => 'Failed to update the page.',
        'no_permission_to_edit' => 'You do not have permission to edit this page.',
        'no_permission_to_delete' => 'You do not have permission to delete this page.',
        'no_permission_to_restore' => 'You do not have permission to restore this page.',
        'restore_failed' => 'Failed to restore the page.',
    ];

    public const SUCCESS_MESSAGES = [
        'page_created' => 'Page created successfully.',
        'page_deleted' => 'Page deleted successfully.',
        'page_updated' => 'Page updated successfully.',
        'page_restored' => 'Page restored successfully.',
        'recycle_bin_emptied' => 'The recycle bin has been emptied.',
    ];

    public function __construct()
    {
        // Initialize the login class
        $this->auth = new SimpleLogin();

        // Ensure required directories and files exist
        $this->ensureRequiredFilesAndDirectories();
    }

    /**
     * Ensures that required data files and directories exist; creates them if they do not.
     */
    private function ensureRequiredFilesAndDirectories()
    {
        // Ensure required directories exist
        ensureRequiredDirectories([
            dirname(CONTENT_DIR . self::PAGES_DATA_FILE),
        ]);

        // Ensure the data file exists; creates it if it does not
        if (!file_exists(CONTENT_DIR . self::PAGES_DATA_FILE)) {
            file_put_contents(CONTENT_DIR . self::PAGES_DATA_FILE, json_encode([], JSON_PRETTY_PRINT));
        }
    }

    /**
     * Checks if the current user is authorized to access or modify a page.
     *
     * @param int $pageUserId The user ID of the page owner.
     * @return bool True if the user is authorized, false otherwise.
     */
    public function isAuthorizedUser(int $pageUserId): bool
    {
        $currentUserId = $_SESSION['user_id'] ?? null;

        // Use SimpleLogin's isAdmin function to check if the user is an admin
        return $this->auth->isAdmin() || $currentUserId === $pageUserId;
    }

    /**
     * Get all pages, optionally filtered by status.
     *
     * @param mixed $status The status or array of statuses to filter by (0: draft, 1: published, 2: deleted). Defaults to null (no filter).
     * @return array The list of pages.
     */
    public function getAllPages($status = null): array
    {
        // Retrieve all pages from the JSON file
        $pages = json_decode(file_get_contents(CONTENT_DIR . self::PAGES_DATA_FILE), true) ?: [];

        // If status is provided and it's an array, filter the pages by the array of statuses
        if (is_array($status)) {
            $pages = array_filter($pages, function ($page) use ($status) {
                return in_array($page['status'], $status);
            });
        }
        // If a single status is provided, filter by that status
        elseif ($status !== null) {
            $pages = array_filter($pages, function ($page) use ($status) {
                return $page['status'] === $status;
            });
        }

        // Sort the pages by created_at in descending order
        usort($pages, function ($a, $b) {
            return strtotime($b['created_at']) <=> strtotime($a['created_at']);
        });

        return $pages;
    }

    /**
     * Get all pages the current user is authorized to access, optionally filtered by status.
     *
     * @param mixed $status The status or array of statuses to filter by (0: draft, 1: published, 2: deleted). Defaults to null (no filter).
     * @return array The list of pages the current user is authorized to access.
     */
    public function getAllPagesForCurrentUser($status = null): array
    {
        // Retrieve all pages, optionally filtered by status
        $allPages = $this->getAllPages($status);

        // If status is an array, filter by multiple statuses
        if (is_array($status)) {
            $authorizedPages = array_filter($allPages, function ($page) use ($status) {
                return $this->isAuthorizedUser($page['user_id']) && in_array($page['status'], $status);
            });
        } else {
            // If a single status is provided, filter by that
            $authorizedPages = array_filter($allPages, function ($page) use ($status) {
                return $this->isAuthorizedUser($page['user_id']) && ($status === null || $page['status'] === $status);
            });
        }

        return $authorizedPages;
    }

    /**
     * Retrieve a specific page by its ID.
     *
     * @param int $id The ID of the page to retrieve.
     * @return array|null The page if found, or null if not accessible.
     */
    public function getPageById(int $id): ?array
    {
        $pages = $this->getAllPages();
        foreach ($pages as $page) {
            if ($page['id'] === $id) {
                // Check if the user is logged in and either the page is published or the user is authorized
                if ($this->auth->isLoggedIn() && $this->isAuthorizedUser($page['user_id'])) {
                    return $page;
                } elseif ($page['status'] === self::PAGE_STATUS_PUBLISHED) {
                    return $page;
                } else {
                    return null;
                }
            }
        }

        Message::set('error', self::ERROR_MESSAGES['page_not_found']);
        return null;
    }

    /**
     * Retrieve a specific page by its alias.
     *
     * @param string $alias The alias of the page to retrieve.
     * @return array|null The page if found, or null if not accessible.
     */
    public function getPageByAlias(string $alias): ?array
    {
        $pages = $this->getAllPages();
        foreach ($pages as $page) {
            if ($page['alias'] === $alias) {
                // Check if the user is logged in and either the page is published or the user is authorized
                if ($this->auth->isLoggedIn() && $this->isAuthorizedUser($page['user_id'])) {
                    return $page;
                } elseif ($page['status'] === self::PAGE_STATUS_PUBLISHED) {
                    return $page;
                } else {
                    return null;
                }
            }
        }

        Message::set('error', self::ERROR_MESSAGES['page_not_found']);
        return null;
    }

    /**
     * Creates a page.
     *
     * @return bool Returns true on success, false on failure.
     */
    public function createPage(array $data)
    {
        // Extract variables from $data
        $title = $data['title'] ?? '';
        $text = $data['text'] ?? '';
        $userId = $data['userId'] ?? 0;
        $status = (int) $data['status'] ?? self::PAGE_STATUS_PUBLISHED;

        // Get all existing pages
        $pages = $this->getAllPages();
        $originalAlias = AliasGenerator::create($title);
        $alias = $originalAlias;

        // Handle duplicate aliases by appending an incrementing number
        $existingAliases = array_column($pages, 'alias');
        $counter = 1;
        while (in_array($alias, $existingAliases)) {
            $alias = $originalAlias . '-' . $counter;
            $counter++;
        }

        // Prepare page data
        $page = [
            'id' => $this->getNextId(),
            'title' => sanitizeInput($title),
            'text' => $text,
            'alias' => $alias,
            'created_at' => date('Y-m-d H:i:s'),
            'user_id' => sanitizeInput($userId),
            'status' => $status,
        ];

        // Add and save page
        $pages[] = $page;
        $this->savePage($pages);
        Message::set('success', self::SUCCESS_MESSAGES['page_created']);

        return $page['id']; // Return the ID of the newly created page
    }

    /**
     * Saves the Pages to the JSON file.
     *
     * @param array $pages The pages to save.
     */
    private function savePage(array $pages): void
    {
        file_put_contents(CONTENT_DIR . self::PAGES_DATA_FILE, json_encode($pages, JSON_PRETTY_PRINT));
    }

    /**
     * Gets a unique ID for a new page.
     *
     * @return int A unique ID.
     */
    private function getNextId(): int
    {
        $pages = $this->getAllPages();
        $existingIds = array_column($pages, 'id');

        do {
            $id = random_int(1000, 100000); // Generate a random ID
        } while (in_array($id, $existingIds, true)); // Repeat until a unique ID is found

        return $id;
    }

    /**
     * Edits an existing page.
     *
     * @return bool Returns true if the page was successfully updated, false otherwise.
     * 
     * @throws Message::set() If there are any errors or success messages during the process.
     */
    public function editPage(int $id, array $data): bool
    {
        $pages = $this->getAllPages();

        foreach ($pages as $key => $page) {
            if ($page['id'] === $id) {
                // Check if the current user is authorized to edit this page
                if (!$this->isAuthorizedUser($page['user_id'])) {
                    Message::set('error', self::ERROR_MESSAGES['no_permission_to_edit']);
                    return false;
                }

                // Update title and check for duplicates
                $originalAlias = AliasGenerator::create($data['title']);
                $alias = $originalAlias;
                $existingAliases = array_column($pages, 'alias');
                unset($existingAliases[$key]); // Exclude the current page alias from the check
                $counter = 1;

                while (in_array($alias, $existingAliases)) {
                    $alias = $originalAlias . '-' . $counter;
                    $counter++;
                }

                $pages[$key]['title'] = sanitizeInput($data['title']);
                $pages[$key]['alias'] = $alias;

                // Update text
                $pages[$key]['text'] = $data['text'];

                // Update status
                $pages[$key]['status'] = (int) ($data['status'] ?? SimpleBlog::PAGE_STATUS_DRAFT);

                // Save the updated page
                $this->savePage($pages);
                Message::set('success', self::SUCCESS_MESSAGES['page_updated']);
                return true;
            }
        }

        Message::set('error', self::ERROR_MESSAGES['update_failed']);
        return false;
    }

    /**
     * Soft delete a page by setting its status to 'deleted', appending '(Deleted)' to its title and alias.
     *
     * @param int $id The ID of the page to delete.
     * @return bool True on success, false on failure.
     */
    public function softDeletePage(int $id): bool
    {
        $pages = $this->getAllPages();

        foreach ($pages as $key => $page) {
            if ($page['id'] === $id) {
                // Check if the current user is authorized to delete this page
                if (!$this->isAuthorizedUser($page['user_id'])) {
                    Message::set('error', self::ERROR_MESSAGES['no_permission_to_delete']);
                    return false;
                }

                // Set the status to deleted
                $pages[$key]['status'] = self::PAGE_STATUS_DELETED;

                // Append '(Deleted)' to the title to mark it as deleted
                $pages[$key]['title'] = $page['title'] . ' (Deleted)';

                // Preserve the existing alias and append "-deleted"
                if (str_ends_with($page['alias'], '-deleted')) {
                    // Avoid double-appending "-deleted" if already present
                    $pages[$key]['alias'] = $page['alias'];
                } else {
                    $pages[$key]['alias'] = $page['alias'] . '-deleted';
                }

                // Save the updated entries
                $this->savePage($pages);
                Message::set('success', self::SUCCESS_MESSAGES['page_deleted']);
                return true;
            }
        }

        Message::set('error', self::ERROR_MESSAGES['delete_failed']);
        return false;
    }

    /**
     * Deletes a page if the current user is authorized to delete it.
     *
     * @param int $id The ID of the page to delete.
     * @return bool True if the page was deleted, false otherwise.
     */
    public function deletePage(int $id): bool
    {
        $pages = $this->getAllPages();
        foreach ($pages as $key => $page) {
            if ($page['id'] === $id) {
                if ($this->isAuthorizedUser($page['user_id'])) {
                    // Remove the page from the pages array
                    unset($pages[$key]);

                    // Save the updated data
                    $this->savePage($pages);
                    Message::set('success', self::SUCCESS_MESSAGES['page_deleted']);
                    return true;
                } else {
                    Message::set('error', self::ERROR_MESSAGES['no_permission_to_delete']);
                    return false;
                }
            }
        }

        Message::set('error', self::ERROR_MESSAGES['delete_failed']);
        return false;
    }

    /**
     * Clear the recycle bin by permanently deleting all soft-deleted pages.
     *
     * @return void
     */
    public function clearRecycleBin(): void
    {
        $pages = $this->getAllPages();

        // Loop through pages and delete all soft-deleted pages
        foreach ($pages as $page) {
            if ($page['status'] === self::PAGE_STATUS_DELETED) {
                // Check if the current user is authorized to delete this page
                if ($this->isAuthorizedUser($page['user_id'])) {
                    // Call deletePage to permanently delete the page
                    $this->deletePage($page['id']);
                }
            }
        }

        Message::set('success', self::SUCCESS_MESSAGES['recycle_bin_emptied']);
    }

    /**
     * Restore a page by setting its status back to draft, 
     * removing '(Deleted)' from the title, and replacing it with '(Draft)', 
     * and updating the alias to match the new title.
     *
     * @param int $id The ID of the page to restore.
     * @return bool True on success, false on failure.
     */
    public function restorePage(int $id): bool
    {
        $pages = $this->getAllPages();

        foreach ($pages as $key => $page) {
            if ($page['id'] === $id) {
                // Check if the current user is authorized to restore this page
                if (!$this->isAuthorizedUser($page['user_id'])) {
                    Message::set('error', self::ERROR_MESSAGES['no_permission_to_restore']);
                    return false;
                }

                // Set the status back to draft
                $pages[$key]['status'] = self::PAGE_STATUS_DRAFT;

                // Remove '(Deleted)' and replace it with '(Draft)' in the title
                $pages[$key]['title'] = str_replace(' (Deleted)', ' (Draft)', $page['title']);

                // Handle alias restoration
                $originalAlias = $page['alias'];
                if (str_ends_with($originalAlias, '-deleted')) {
                    // Remove '-deleted' suffix from alias
                    $originalAlias = substr($originalAlias, 0, -strlen('-deleted'));
                }

                // Ensure alias is unique with logical suffix handling
                $existingAliases = array_column($pages, 'alias');
                unset($existingAliases[$key]); // Exclude the current page alias from the check

                $baseAlias = $originalAlias;
                $suffix = 0;

                foreach ($existingAliases as $existingAlias) {
                    if (preg_match('/^' . preg_quote($baseAlias, '/') . '-(\d+)$/', $existingAlias, $matches)) {
                        $suffix = max($suffix, (int) $matches[1]);
                    }
                }

                // Increment suffix for logical uniqueness
                $pages[$key]['alias'] = $suffix > 0 ? $baseAlias . '-' . ($suffix + 1) : $baseAlias;

                // Save the updated entries
                $this->savePage($pages);
                Message::set('success', self::SUCCESS_MESSAGES['page_restored']);
                return true;
            }
        }

        Message::set('error', self::ERROR_MESSAGES['restore_failed']);
        return false;
    }

    /**
     * Get the username by user ID.
     *
     * @param int $userId The user ID.
     * @return string|null The username or null if not found.
     */
    public function getUsernameById(int $userId): ?string
    {
        return SimpleLogin::getUsernameById($userId);
    }
}
