<?php
// Protects the module files from being accessed directly
if (!defined('ALLOW_DIRECT_ACCESS')) {
    die("Access denied.");
}

/**
 * Settings Class
 * 
 * This class provides methods to manage settings for each module.
 * It allows retrieving, updating, and generating HTML forms for module settings.
 * 
 */
class Settings
{
    // Current version
    const VERSION = "1.0.4";

    // Shows the current version of the module
    public function getVersion()
    {
        return self::VERSION;
    }
    
    /**
     * Constructor for the Settings class.
     */
    public function __construct()
    {
        // Load and initialize all modules to ensure their settings are available
        self::initializeModules();
    }

    /**
     * Loads and initializes all modules by scanning the module directory, including
     * each module's main PHP file, and instantiating its class. Excludes specified 
     * modules from initialization. Cleans up settings to ensure they reflect only 
     * active modules.
     *
     * @return void
     */
    public static function initializeModules()
    {
        $moduleDir = __DIR__ . '/../'; 
        $excludedModules = ['Settings', 'Updater']; // Modules to skip
    
        // Ensure the directory exists
        if (is_dir($moduleDir)) {
            foreach (scandir($moduleDir) as $moduleFolder) {
                // Skip special directories (.) and (..) and excluded modules
                if ($moduleFolder === '.' || $moduleFolder === '..' || in_array($moduleFolder, $excludedModules)) {
                    continue;
                }
    
                // Build the full path to the module's PHP file
                $modulePath = realpath("$moduleDir/$moduleFolder/$moduleFolder.php");
    
                // Skip if the path is invalid or not within the allowed directory
                if (!$modulePath || strpos($modulePath, realpath($moduleDir)) !== 0) {
                    // Log the invalid path
                    error_log("Warning: Skipping $moduleFolder: Invalid path or outside allowed directory.");
                    continue;
                }
    
                // Ensure the module file exists within the allowed directory
                if (file_exists($modulePath)) {
                    include_once $modulePath;
    
                    // Proceed with further module initialization
                    if (class_exists($moduleFolder)) {
                        $moduleConfigPath = realpath("$moduleDir/$moduleFolder/module_config.json");
                        $params = [];
    
                        // Load configuration if available
                        if ($moduleConfigPath && file_exists($moduleConfigPath)) {
                            $config = json_decode(file_get_contents($moduleConfigPath), true);
                            if (isset($config['parameters']) && is_array($config['parameters'])) {
                                $params = $config['parameters'];
                            }
                        }
    
                        // Use Reflection to dynamically instantiate the module class
                        $refClass = new ReflectionClass($moduleFolder);
                        $constructor = $refClass->getConstructor();
    
                        if ($constructor && $constructor->getNumberOfRequiredParameters() > 0) {
                            // If the module requires parameters but no config file exists, skip it
                            if (empty($params)) {
                                error_log("Warning: Skipping $moduleFolder: requires parameters but no module_config.json found.");
                                continue;
                            }
    
                            // Ensure we have enough parameters for the constructor
                            $requiredParams = $constructor->getNumberOfRequiredParameters();
                            if (count($params) >= $requiredParams) {
                                $refClass->newInstanceArgs($params);
                            } else {
                                error_log("Warning: Skipping $moduleFolder: insufficient parameters in module_config.json.");
                                continue;
                            }
                        } else {
                            new $moduleFolder();
                        }
                    }
                } else {
                    error_log("Warning: Skipping $moduleFolder: Module file does not exist.");
                }
            }
        }
    
        // Clean up settings after modules are loaded
        ModuleLoader::cleanUpSettings();
    }    

    /**
     * Retrieves all settings for the modules from settings.json.
     *
     * @return array The settings for all modules.
     */
    public function getAllSettings(): array
    {
        ModuleLoader::loadSettings(); // Ensure settings are loaded
        return ModuleLoader::getAllSettings(); // Fetch all settings from the ModuleLoader
    }

    /**
     * Updates a setting for a specific module.
     *
     * @param string $module The module name.
     * @param string $key The setting key.
     * @param mixed $value The new value for the setting.
     * @return void
     */
    public function setSetting(string $module, string $key, $value): void
    {
        // Call the original setSetting in ModuleLoader to handle the actual update
        ModuleLoader::setSetting($module, $key, $value);
    }

    /**
     * Generates HTML forms for editing module settings.
     * 
     * @return string HTML form markup for the specified module or all modules.
     *                Returns an error message if no settings are available for any module or if the specified module does not exist.
     */
    public function generateSettingsForms(?string $moduleName = null): string
    {
        $settings = $this->getAllSettings();
    
        if (empty($settings)) {
            Message::set('error', 'No settings available to edit.');
            return '';
        }
    
        $formsHtml = '';
    
        if ($moduleName) {
            if (!isset($settings[$moduleName])) {
                Message::set('error', "Error: The module '$moduleName' does not exist or has no settings.");
                return '';
            }
            $settings = [$moduleName => $settings[$moduleName]];
        }
    
        foreach ($settings as $module => $moduleSettings) {
            $formsHtml .= '<h2>' . htmlspecialchars($module) . '</h2>';
            $formsHtml .= '<form method="POST" class="form">';
            $formsHtml .= '<input type="hidden" name="module" value="' . htmlspecialchars($module) . '">' . "\n";
    
            $sortedSettings = [
                'bool' => [],
                'int' => [],
                'string' => [],
                'color' => [],
            ];
    
            // Prioritize accentColor at the top of the form if it exists
            if (isset($moduleSettings['simplifyAccentColor'])) {
                $accentColorSetting = $moduleSettings['simplifyAccentColor'];
                unset($moduleSettings['simplifyAccentColor']);
            }
    
            // Now continue with sorting other settings as usual
            foreach ($moduleSettings as $key => $setting) {
                $type = $setting['type'] ?? 'string';
                if ($type === 'bool') {
                    $sortedSettings['bool'][$key] = $setting;
                } elseif ($type === 'int') {
                    $sortedSettings['int'][$key] = $setting;
                } elseif ($type === 'color') {
                    $sortedSettings['color'][$key] = $setting;
                } else {
                    $sortedSettings['string'][$key] = $setting;
                }
            }
    
            // If accentColor is set, add it as the first field in the form
            if (isset($accentColorSetting)) {
                $formsHtml .= '<div class="color-field" style="text-align: center;">';
                $formsHtml .= '<label for="simplifyAccentColor"><i class="fas fa-palette"></i> Accent Color</label>';
                $formsHtml .= '<input style="width: 100px; height: 80px; display: block; margin: 0 auto;" type="color" name="' . htmlspecialchars($module) . '[simplifyAccentColor]" value="' . htmlspecialchars($accentColorSetting['value']) . '">';
                $formsHtml .= '<small>' . $accentColorSetting['description'] . '</small>';
                $formsHtml .= '</div>';
            }
    
            // Continue rendering other settings
            foreach ($sortedSettings as $type => $settingsByType) {
                foreach ($settingsByType as $key => $setting) {
                    $value = $setting['value'] ?? '';
                    $label = $setting['label'] ?? $key;
                    $description = $setting['description'] ?? '';
                    $icon = $setting['icon'] ?? '';
    
                    if ($key === 'moduleIcon' && empty($setting['label']) && empty($setting['value']) && empty($setting['description'])) {
                        continue;
                    }
    
                    $labelHtml = $icon ? '<i class="fas ' . htmlspecialchars($icon) . '"></i> ' . htmlspecialchars($label) : htmlspecialchars($label);
    
                    switch ($type) {
                        case 'bool':
                            // Hidden field to send false if the checkbox is unchecked
                            $formsHtml .= '<input type="hidden" name="' . htmlspecialchars($module) . '[' . htmlspecialchars($key) . ']" value="0">';
                            $formsHtml .= '<div class="checkbox">';
                            $formsHtml .= '<label for="' . htmlspecialchars($key) . '">' . $labelHtml . '</label>';
                            $formsHtml .= '<input type="checkbox" name="' . htmlspecialchars($module) . '[' . htmlspecialchars($key) . ']" value="1" ' . ($value == 1 ? 'checked' : '') . '>';
                            if ($description) {
                                $formsHtml .= '<small>' . $description . '</small>';
                            }
                            $formsHtml .= '</div>';
                            break;
    
                        case 'int':
                            $formsHtml .= '<div class="number-field">';
                            $formsHtml .= '<label for="' . htmlspecialchars($key) . '">' . $labelHtml . '</label>';
                            $formsHtml .= '<input type="number" name="' . htmlspecialchars($module) . '[' . htmlspecialchars($key) . ']" value="' . htmlspecialchars($value) . '">';
                            if ($description) {
                                $formsHtml .= '<small>' . $description . '</small>';
                            }
                            $formsHtml .= '</div>';
                            break;
    
                        case 'color':
                            $formsHtml .= '<div class="color-field">';
                            $formsHtml .= '<label for="' . htmlspecialchars($key) . '">' . $labelHtml . '</label>';
                            $formsHtml .= '<input type="color" name="' . htmlspecialchars($module) . '[' . htmlspecialchars($key) . ']" value="' . htmlspecialchars($value) . '">';
                            if ($description) {
                                $formsHtml .= '<small>' . $description . '</small>';
                            }
                            $formsHtml .= '</div>';
                            break;
    
                        case 'string':
                        default:
                            $formsHtml .= '<div class="text-field">';
                            $formsHtml .= '<label for="' . htmlspecialchars($key) . '">' . $labelHtml . '</label>';
                            $formsHtml .= '<input type="text" name="' . htmlspecialchars($module) . '[' . htmlspecialchars($key) . ']" value="' . htmlspecialchars($value) . '">';
                            if ($description) {
                                $formsHtml .= '<small>' . $description . '</small>';
                            }
                            $formsHtml .= '</div>';
                            break;
                    }
                }
            }
    
            $formsHtml .= '<button type="submit" name="saveSettings">Save Settings</button>';
            $formsHtml .= '</form>';
        }
    
        return $formsHtml;
    }    

    /**
     * Handle the form submission, save settings and display appropriate messages.
     *
     * This method processes the form data, saves the new settings, 
     * and displays a success or error message based on the result.
     *
     * @return void
     */
    public function handleFormSubmission()
    {
        try {
            // Get the module from POST data to determine which module's settings are being updated
            $currentModule = $_POST['module'] ?? null;

            if (!$currentModule) {
                Message::set('error', 'No module specified for saving settings.');
                return;
            }

            // Loop through the submitted form data
            $postData = $_POST;

            // Ensure settings are loaded
            $settings = $this->getAllSettings();

            // Check if the current module exists in the settings
            if (!isset($settings[$currentModule])) {
                Message::set('error', "Error: The module '$currentModule' does not exist.");
                return;
            }

            // Get module-specific settings
            $moduleSettings = $settings[$currentModule];

            // Loop through the module's settings to update values
            foreach ($moduleSettings as $key => $setting) {
                // Check if the setting is in the POST data or is a checkbox missing from POST data (unchecked)
                $newValue = $postData[$currentModule][$key] ?? null;

                // Handle boolean checkbox input: if not present, set as false (unchecked)
                if ($setting['type'] === 'bool') {
                    $newValue = $newValue !== null ? (bool) $newValue : false;
                }

                // Update the setting if new value exists or was set to false for checkboxes
                $this->setSetting($currentModule, $key, $newValue);
            }

            // Success message after saving
            Message::set('success', 'Settings have been successfully updated.');
        } catch (Exception $e) {
            Message::set('error', 'An error occurred while saving the settings. Please try again.');
        }
    }

}
