<?php
// Protects the module files from being accessed directly
if (!defined('ALLOW_DIRECT_ACCESS')) {
    die("Access denied.");
}

/**
 * Class SimpleBlog
 * 
 * This class provides methods to manage blog entries including creating, editing,
 * deleting, and retrieving entries from a JSON file, with message handling internally.
 * It also manages comments for each blog entry.
 */
class SimpleBlog
{
    // Current version
    const VERSION = "1.0.3";

    // Shows the current version of the module
    public function getVersion()
    {
        return self::VERSION;
    }
    
    // SimpleLogin class
    private SimpleLogin $auth;

    // Store meta tags globally
    private $metaTags = [];

    // The directory where uploaded files are stored
    public string $uploadDir;

    private const BLOG_DATA_FILE = '/blog/blog_entries.json';
    private const BLOG_COMMENTS_DATA_FILE = '/blog/blog_comments.php';
    private const IMAGE_UPLOAD_DIR = '/blog/images/';
    private const BLOG_CSS_FILE = '/css/blog.css';

    public const POST_STATUS_DRAFT = 0;
    public const POST_STATUS_PUBLISHED = 1;
    public const POST_STATUS_DELETED = 2;

    // Error and success messages stored as constants for easy translation
    public const ERROR_MESSAGES = [
        'entry_not_found' => 'Failed to find the blog entry.',
        'delete_failed' => 'Failed to delete the blog entry. Entry not found.',
        'update_failed' => 'Failed to update the blog entry. Entry not found.',
        'comment_failed' => 'Failed to add comment. Entry not found.',
        'comment_failed_captcha' => 'Failed to add comment. Incorrect CAPTCHA.',
        'comment_not_found' => 'Failed to find the comment.',
        'metadata_not_found' => 'Metadata file not found.',
        'no_metadata_found' => 'No metadata found.',
        'file_delete_failed' => 'Failed to delete the file from the server.',
        'file_not_found' => 'The file you are trying to delete does not exist.',
        'no_permission_to_edit' => 'You do not have permission to edit this entry.',
        'no_permission_to_delete' => 'You do not have permission to delete this entry.',
        'no_permission_to_restore' => 'You do not have permission to restore this entry.',
        'restore_failed' => 'Failed to restore the blog entry.',
    ];

    public const SUCCESS_MESSAGES = [
        'entry_created' => 'Blog entry created successfully.',
        'entry_deleted' => 'Blog entry deleted successfully.',
        'entry_updated' => 'Blog entry updated successfully.',
        'comment_added' => 'Comment added successfully.',
        'comment_deleted' => 'Comment deleted successfully.',
        'comment_updated' => 'Comment updated successfully.',
        'image_deleted' => 'Image deleted successfully from the server and metadata.',
        'images_deleted' => 'All images deleted successfully from the server and metadata.',
        'entry_restored' => 'Blog entry restored successfully.',
        'recycle_bin_emptied' => 'The recycle bin has been emptied.',
    ];

    public function __construct()
    {
        // Initialize the login class
        $this->auth = new SimpleLogin();

        // The upload directory
        $this->uploadDir = CONTENT_DIR . self::IMAGE_UPLOAD_DIR;

        // Ensure required directories and files exist
        $this->ensureRequiredFilesAndDirectories();

        // After installing the SimpleBlog module, set default settings.

        // Setting for date format
        ModuleLoader::setDefaultSettings(
            'SimpleBlog',
            'dateFormat',
            'd.m.Y', // Default date format
            'Date Format',
            'The format in which dates will be displayed when using <code>[date]</code>.',
            'string', // The value is a string (text input)
            'fa-calendar-alt' // FontAwesome icon for the setting
        );

        // Setting for comment date format
        ModuleLoader::setDefaultSettings(
            'SimpleBlog',
            'commentDateFormat',
            'd.m.Y', // Default comment date format
            'Comment Date Format',
            'The format in which dates will be displayed when using <code>[comment_date]</code>.',
            'string', // The value is a string (text input)
            'fa-comments' // FontAwesome icon for the setting
        );

        // Setting for posts per page (number of posts to display on each page)
        ModuleLoader::setDefaultSettings(
            'SimpleBlog',
            'postsPerPage',
            10, // Default to 10 posts per page
            'Posts Per Page',
            'Number of posts to display on each page.',
            'int', // The value is an integer (number input)
            'fa-list-alt' // FontAwesome icon for the setting
        );

        // Setting for using one template for both index and post pages
        ModuleLoader::setDefaultSettings(
            'SimpleBlog',
            'oneTemplate',
            false,
            'Single Template',
            'Use a single template for the index.php and post view pages, or use separate templates for each.',
            'bool', // The value is a boolean (checkbox input)
            'fa-puzzle-piece' // FontAwesome icon for the setting
        );

        // Setting for using the user's profile picture as a fallback post image
        ModuleLoader::setDefaultSettings(
            'SimpleBlog',
            'profilePictureAsPostImage',
            true,
            'Use Profile Picture as Fallback Post Image',
            'Enable this to display the profile picture as a fallback image when no post image is uploaded.',
            'bool', // The value is a boolean (checkbox input)
            'fa-user' // FontAwesome icon for the setting
        );

        // Unique Module Icon for SimpleBlog (this will be used in the dashboard)
        ModuleLoader::setDefaultSettings(
            'SimpleBlog',
            'moduleIcon',
            '', // No value here, it's just for the icon
            '', // No label here
            '', // No description
            'string', // Just a placeholder for type
            'fa-newspaper' // FontAwesome icon for the module
        );
    }

    // Static method to register assets
    public static function registerAssets()
    {
        AssetsManager::addCss('/SimpleBlog' . self::BLOG_CSS_FILE);
    }

    /**
     * Ensures that required data files and directories exist; creates them if they do not.
     */
    private function ensureRequiredFilesAndDirectories()
    {
        // Ensure required directories exist
        ensureRequiredDirectories([
            dirname(CONTENT_DIR . self::BLOG_DATA_FILE),
            dirname(CONTENT_DIR . self::BLOG_COMMENTS_DATA_FILE),
            $this->uploadDir
        ]);

        // Ensure the blog data file exists; creates it if it does not
        if (!file_exists(CONTENT_DIR . self::BLOG_DATA_FILE)) {
            file_put_contents(CONTENT_DIR . self::BLOG_DATA_FILE, json_encode([], JSON_PRETTY_PRINT));
        }

        /**
         * Ensure the comments data file exists; creates it if it does not.
         * 
         * Using PHP files instead of JSON for storing comments to prevent 
         * sensitive data (e.g., emails) from being publicly accessible.
         * 
         */
        if (!file_exists(CONTENT_DIR . self::BLOG_COMMENTS_DATA_FILE)) {
            $phpContent = "<?php\nreturn [];\n";
            file_put_contents(CONTENT_DIR . self::BLOG_COMMENTS_DATA_FILE, $phpContent);
        }

        // Ensure the image upload directory exists
        if (!file_exists($this->uploadDir)) {
            mkdir($this->uploadDir, 0755, true);
        }
    }

    /**
     * Checks if the current user is authorized to access or modify an entry.
     *
     * @param int $entryUserId The user ID of the blog entry owner.
     * @return bool True if the user is authorized, false otherwise.
     */
    public function isAuthorizedUser(int $entryUserId): bool
    {
        $currentUserId = $_SESSION['user_id'] ?? null;

        // Use SimpleLogin's isAdmin function to check if the user is an admin
        return $this->auth->isAdmin() || $currentUserId === $entryUserId;
    }

    /**
     * Get all blog entries, optionally filtered by status.
     *
     * @param mixed $status The status or array of statuses to filter by (0: draft, 1: published, 2: deleted). Defaults to null (no filter).
     * @return array The list of blog entries.
     */
    public function getAllEntries($status = null): array
    {
        // Retrieve all entries from the JSON file
        $entries = json_decode(file_get_contents(CONTENT_DIR . self::BLOG_DATA_FILE), true) ?: [];

        // If status is provided and it's an array, filter the entries by the array of statuses
        if (is_array($status)) {
            $entries = array_filter($entries, function ($entry) use ($status) {
                return in_array($entry['status'], $status);
            });
        }
        // If a single status is provided, filter by that status
        elseif ($status !== null) {
            $entries = array_filter($entries, function ($entry) use ($status) {
                return $entry['status'] === $status;
            });
        }

        // Sort the entries by created_at in descending order
        usort($entries, function ($a, $b) {
            return strtotime($b['created_at']) <=> strtotime($a['created_at']);
        });

        return $entries;
    }

    /**
     * Get all blog entries the current user is authorized to access, optionally filtered by status.
     *
     * @param mixed $status The status or array of statuses to filter by (0: draft, 1: published, 2: deleted). Defaults to null (no filter).
     * @return array The list of blog entries the current user is authorized to access.
     */
    public function getAllEntriesForCurrentUser($status = null): array
    {
        // Retrieve all entries, optionally filtered by status
        $allEntries = $this->getAllEntries($status);

        // If status is an array, filter by multiple statuses
        if (is_array($status)) {
            $authorizedEntries = array_filter($allEntries, function ($entry) use ($status) {
                return $this->isAuthorizedUser($entry['user_id']) && in_array($entry['status'], $status);
            });
        } else {
            // If a single status is provided, filter by that
            $authorizedEntries = array_filter($allEntries, function ($entry) use ($status) {
                return $this->isAuthorizedUser($entry['user_id']) && ($status === null || $entry['status'] === $status);
            });
        }

        return $authorizedEntries;
    }

    /**
     * Retrieve a specific blog entry by its ID.
     *
     * @param int $id The ID of the blog entry to retrieve.
     * @return array|null The blog entry if found, or null if not accessible.
     */
    public function getEntryById(int $id): ?array
    {
        $entries = $this->getAllEntries();
        foreach ($entries as $entry) {
            if ($entry['id'] === $id) {
                // Check if the user is logged in and either the entry is published or the user is authorized
                if ($this->auth->isLoggedIn() && $this->isAuthorizedUser($entry['user_id'])) {
                    return $entry;
                } elseif ($entry['status'] === self::POST_STATUS_PUBLISHED) {
                    return $entry;
                } else {
                    return null;
                }
            }
        }

        Message::set('error', self::ERROR_MESSAGES['entry_not_found']);
        return null;
    }

    /**
     * Retrieve a specific blog entry by its alias.
     *
     * @param string $alias The alias of the blog entry to retrieve.
     * @return array|null The blog entry if found, or null if not accessible.
     */
    public function getEntryByAlias(string $alias): ?array
    {
        $entries = $this->getAllEntries();
        foreach ($entries as $entry) {
            if ($entry['alias'] === $alias) {
                // Check if the user is logged in and either the entry is published or the user is authorized
                if ($this->auth->isLoggedIn() && $this->isAuthorizedUser($entry['user_id'])) {
                    $this->addMetaTags($entry);
                    return $entry;
                } elseif ($entry['status'] === self::POST_STATUS_PUBLISHED) {
                    $this->addMetaTags($entry);
                    return $entry;
                } else {
                    return null;
                }
            }
        }

        Message::set('error', self::ERROR_MESSAGES['entry_not_found']);
        return null;
    }

    /**
     * Add meta tags for SEO.
     *
     * @param array $entry The blog entry.
     * @return void
     */
    public function addMetaTags(array $entry): void
    {
        // Set recommended lengths for meta tags
        $descriptionMaxLength = 160; // Recommended length for meta description
        $ogTitleMaxLength = 60; // Recommended length for Open Graph title
        $twitterTitleMaxLength = 70; // Recommended length for Twitter title

        // Strip HTML tags from the entry text
        $cleanText = isset($entry['text']) ? strip_tags($entry['text']) : '';
        $cleanTitle = isset($entry['title']) ? strip_tags(getSiteTitle($entry['title'])) : '';
        $cleanUrl = getScriptUrl();

        // Handle the image URL
        $imageUrl = isset($entry['image']) ? $this->getPostImage($entry['image']) : '';

        // Shorten the meta description if it exceeds the max length
        $description = strip_tags(shortenString($cleanText, $descriptionMaxLength));

        // Shorten the Open Graph title if it exceeds the max length
        $ogTitle = strip_tags(shortenString($cleanTitle, $ogTitleMaxLength));

        // Shorten the Twitter title if it exceeds the max length
        $twitterTitle = strip_tags(shortenString($cleanTitle, $twitterTitleMaxLength));

        // Build the metaTags array with values
        $this->metaTags = [
            'title' => $cleanTitle,
            'description' => $description,
            'keywords' => implode(', ', $entry['hashtags'] ?? ['blog', 'simple', 'post']),
            'og:title' => $ogTitle,
            'og:description' => $description,
            'og:image' => $imageUrl,
            'og:url' => $cleanUrl,
            'twitter:title' => $twitterTitle,
            'twitter:description' => $description,
            'twitter:image' => $imageUrl,
        ];
    }

    /**
     * Get meta tags for SEO.
     *
     * @return array The meta tags.
     */
    public function getMetaTags(): array
    {
        return $this->metaTags;
    }

    /**
     * Renders the meta tags for SEO, including standard, Open Graph, and Twitter Card tags.
     *
     * @param array $metaTags The meta tags to render.
     * @return void
     */
    function renderMetaTags(array $metaTags): void
    {
        $indent = '    '; // 4 spaces per indentation level
        $isFirst = true; // Flag to track if it's the first meta tag

        // Loop through the meta tags array and create the meta elements
        foreach ($metaTags as $name => $content) {
            // Apply indentation for the first tag only after the first iteration
            if ($isFirst) {
                echo '<meta name="' . sanitizeInput($name) . '" content="' . sanitizeInput($content) . '">' . PHP_EOL;
                $isFirst = false;  // Set the flag to false after the first tag
            } else {
                // Apply proper indentation for all subsequent tags
                echo $indent . '<meta name="' . sanitizeInput($name) . '" content="' . sanitizeInput($content) . '">' . PHP_EOL;
            }
        }
    }

    /**
     * Retrieve blog entries that contain a specific hashtag.
     *
     * @param string $hashtag The hashtag to filter entries by.
     * @return array The filtered list of blog entries.
     */
    public function getEntriesByHashtag(string $hashtag): array
    {
        $entries = $this->getAllEntries();
        $filteredEntries = array_filter($entries, function ($entry) use ($hashtag) {
            return in_array($hashtag, $entry['hashtags']);
        });

        return array_values($filteredEntries); // Re-index the array
    }

    /**
     * Creates a new blog entry.
     *
     * @return bool Returns true on success, false on failure.
     */
    public function createEntry(array $data)
    {
        // Extract variables from $data
        $title = $data['title'] ?? '';
        $text = $data['text'] ?? '';
        $userId = $data['userId'] ?? 0;
        $hashtags = array_filter(array_map('trim', explode(',', $data['hashtags'] ?? '')));
        $file = $data['image'] ?? null;
        $isImageRequired = $data['options']['is_image_required'] ?? false;
        $status = (int) $data['status'] ?? self::POST_STATUS_PUBLISHED;

        $entries = $this->getAllEntries();
        $originalAlias = AliasGenerator::create($title);
        $alias = $originalAlias;

        // Handle duplicate titles by appending an incrementing number
        $existingAliases = array_column($entries, 'alias');
        $counter = 1;
        while (in_array($alias, $existingAliases)) {
            $alias = $originalAlias . '-' . $counter;
            $counter++;
        }

        $imagePath = null;

        // Handle image upload if a file is provided
        if ($file) {
            $uploader = new SimpleUpload($this->uploadDir, ['image/jpeg', 'image/webp', 'image/png', 'image/gif']);
            $imagePath = $uploader->upload('image', [], $isImageRequired);

            if (strpos($imagePath, 'Invalid file') !== false || strpos($imagePath, 'Failed to move uploaded file') !== false) {
                Message::set('error', $imagePath);
                return false; // Return false on upload error
            }
        }

        $imageName = $uploader->getUploadedFilesAsString();

        // Sanitize hashtags: Remove all non-alphanumeric characters
        $hashtags = array_map(function ($hashtag) {
            // Remove everything that is not a letter or a number
            $hashtag = preg_replace('/[^a-zA-Z0-9]/', '', $hashtag); // Keep only alphanumeric characters
            return $hashtag;
        }, $hashtags);

        // Ensure no empty hashtags are included
        $hashtags = array_filter($hashtags);

        // Prepare entry data with the status field
        $entry = [
            'id' => $this->getNextId(),
            'title' => sanitizeInput($title),
            'text' => $text,
            'alias' => $alias,
            'created_at' => date('Y-m-d H:i:s'),
            'user_id' => sanitizeInput($userId),
            'hashtags' => $hashtags,  // The sanitized hashtags
            'image' => $imageName,
            'status' => $status,
        ];

        // Add and save entry
        $entries[] = $entry;
        $this->saveEntry($entries);
        Message::set('success', self::SUCCESS_MESSAGES['entry_created']);

        return $entry['id']; // Return the ID of the newly created entry
    }

    /**
     * Saves the blog entries to the JSON file.
     *
     * @param array $entries The entries to save.
     */
    private function saveEntry(array $entries): void
    {
        file_put_contents(CONTENT_DIR . self::BLOG_DATA_FILE, json_encode($entries, JSON_PRETTY_PRINT));
    }

    /**
     * Gets a unique ID for a new blog entry.
     *
     * @return int A unique ID.
     */
    private function getNextId(): int
    {
        $entries = $this->getAllEntries();
        $existingIds = array_column($entries, 'id');

        do {
            $id = random_int(1000, 100000); // Generate a random ID
        } while (in_array($id, $existingIds, true)); // Repeat until a unique ID is found

        return $id;
    }

    /**
     * Edits an existing blog entry.
     * 
     * This method updates the details of an existing blog entry identified by the provided ID. It includes updating the title, alias, content (text), hashtags, status, and image. It also ensures that the user has the necessary permissions to edit the entry.
     *
     * @return bool Returns true if the entry was successfully updated, false otherwise.
     * 
     * @throws Message::set() If there are any errors or success messages during the process.
     */
    public function editEntry(int $id, array $data): bool
    {
        $entries = $this->getAllEntries();

        foreach ($entries as $key => $entry) {
            if ($entry['id'] === $id) {
                // Check if the current user is authorized to edit this entry
                if (!$this->isAuthorizedUser($entry['user_id'])) {
                    Message::set('error', self::ERROR_MESSAGES['no_permission_to_edit']);
                    return false;
                }

                // Update title and check for duplicates
                $originalAlias = AliasGenerator::create($data['title']);
                $alias = $originalAlias;
                $existingAliases = array_column($entries, 'alias');
                unset($existingAliases[$key]); // Exclude the current entry's alias from the check
                $counter = 1;

                while (in_array($alias, $existingAliases)) {
                    $alias = $originalAlias . '-' . $counter;
                    $counter++;
                }

                $entries[$key]['title'] = sanitizeInput($data['title']);
                $entries[$key]['alias'] = $alias;

                // Update text
                $entries[$key]['text'] = $data['text'];

                // Sanitize hashtags: Remove everything that is not a letter or a number
                $hashtags = array_filter(array_map('trim', explode(',', $data['hashtags'] ?? '')));
                $hashtags = array_map(function ($hashtag) {
                    return preg_replace('/[^a-zA-Z0-9]/', '', $hashtag); // Keep only alphanumeric characters
                }, $hashtags);
                // Ensure no empty hashtags are included
                $entries[$key]['hashtags'] = $hashtags;

                // Update status
                $entries[$key]['status'] = (int) ($data['status'] ?? SimpleBlog::POST_STATUS_DRAFT);

                // Handle image upload
                if (isset($data['image']) && $data['image']['error'] === UPLOAD_ERR_OK) {
                    $uploader = new SimpleUpload($this->uploadDir, ['image/jpeg', 'image/webp', 'image/png', 'image/gif']);
                    $imagePath = $uploader->upload('image');

                    // Check for upload errors
                    if (strpos($imagePath, 'Invalid file') !== false || strpos($imagePath, 'Failed to move uploaded file') !== false) {
                        Message::set('error', $imagePath);
                        return false;
                    }

                    // Delete the old image file if a new one is uploaded
                    if (!empty($entry['image'])) {
                        $this->deleteImage($entry['image']);
                    }

                    // Get the new image name as a string
                    $imageName = $uploader->getUploadedFilesAsString();

                    // Update with the new image path
                    $entries[$key]['image'] = $imageName;
                } else {
                    // If no new image uploaded, retain the existing image
                    $entries[$key]['image'] = $entry['image'];
                }

                // Save the updated entry
                $this->saveEntry($entries);
                Message::set('success', self::SUCCESS_MESSAGES['entry_updated']);
                return true;
            }
        }

        Message::set('error', self::ERROR_MESSAGES['update_failed']);
        return false;
    }

    /**
     * Soft delete a blog entry by setting its status to 'deleted', appending '(Deleted)' to its title and alias.
     *
     * @param int $id The ID of the blog entry to delete.
     * @return bool True on success, false on failure.
     */
    public function softDeleteEntry(int $id): bool
    {
        $entries = $this->getAllEntries();

        foreach ($entries as $key => $entry) {
            if ($entry['id'] === $id) {
                // Check if the current user is authorized to delete this entry
                if (!$this->isAuthorizedUser($entry['user_id'])) {
                    Message::set('error', self::ERROR_MESSAGES['no_permission_to_delete']);
                    return false;
                }

                // Set the status to deleted
                $entries[$key]['status'] = self::POST_STATUS_DELETED;

                // Append '(Deleted)' to the title to mark it as deleted
                $entries[$key]['title'] = $entry['title'] . ' (Deleted)';

                // Preserve the existing alias and append "-deleted"
                if (str_ends_with($entry['alias'], '-deleted')) {
                    // Avoid double-appending "-deleted" if already present
                    $entries[$key]['alias'] = $entry['alias'];
                } else {
                    $entries[$key]['alias'] = $entry['alias'] . '-deleted';
                }

                // Save the updated entries
                $this->saveEntry($entries);
                Message::set('success', self::SUCCESS_MESSAGES['entry_deleted']);
                return true;
            }
        }

        Message::set('error', self::ERROR_MESSAGES['delete_failed']);
        return false;
    }

    /**
     * Deletes a blog entry if the current user is authorized to delete it.
     *
     * @param int $id The ID of the blog entry to delete.
     * @return bool True if the entry was deleted, false otherwise.
     */
    public function deleteEntry(int $id): bool
    {
        $entries = $this->getAllEntries();
        foreach ($entries as $key => $entry) {
            if ($entry['id'] === $id) {
                if ($this->isAuthorizedUser($entry['user_id'])) {
                    // Delete all comments associated with this post
                    $this->deleteCommentsByPostId($id);

                    // Delete the image associated with the entry if it exists
                    if (!empty($entry['image'])) {
                        $this->deleteImage($entry['image']); // Delete the old image
                    }

                    // Remove the entry from the entries array
                    unset($entries[$key]);

                    // Save the updated data
                    $this->saveEntry($entries);
                    Message::set('success', self::SUCCESS_MESSAGES['entry_deleted']);
                    return true;
                } else {
                    Message::set('error', self::ERROR_MESSAGES['no_permission_to_delete']);
                    return false;
                }
            }
        }

        Message::set('error', self::ERROR_MESSAGES['delete_failed']);
        return false;
    }

    /**
     * Clear the recycle bin by permanently deleting all soft-deleted entries.
     *
     * @return void
     */
    public function clearRecycleBin(): void
    {
        $entries = $this->getAllEntries();

        // Loop through entries and delete all soft-deleted entries
        foreach ($entries as $entry) {
            if ($entry['status'] === self::POST_STATUS_DELETED) {
                // Check if the current user is authorized to delete this entry
                if ($this->isAuthorizedUser($entry['user_id'])) {
                    // Call deleteEntry to permanently delete the entry
                    $this->deleteEntry($entry['id']);
                }
            }
        }

        Message::set('success', self::SUCCESS_MESSAGES['recycle_bin_emptied']);
    }

    /**
     * Restore a blog entry by setting its status back to draft,
     * removing '(Deleted)' from the title, and replacing it with '(Draft)',
     * and updating the alias while preserving its original structure.
     *
     * @param int $id The ID of the blog entry to restore.
     * @return bool True on success, false on failure.
     */
    public function restoreEntry(int $id): bool
    {
        $entries = $this->getAllEntries();

        foreach ($entries as $key => $entry) {
            if ($entry['id'] === $id) {
                // Check if the current user is authorized to restore this entry
                if (!$this->isAuthorizedUser($entry['user_id'])) {
                    Message::set('error', self::ERROR_MESSAGES['no_permission_to_restore']);
                    return false;
                }

                // Set the status back to draft
                $entries[$key]['status'] = self::POST_STATUS_DRAFT;

                // Remove '(Deleted)' and replace it with '(Draft)' in the title
                $entries[$key]['title'] = str_replace(' (Deleted)', ' (Draft)', $entry['title']);

                // Handle alias restoration
                $originalAlias = $entry['alias'];
                if (str_ends_with($originalAlias, '-deleted')) {
                    // Remove '-deleted' suffix from alias
                    $originalAlias = substr($originalAlias, 0, -strlen('-deleted'));
                }

                // Ensure alias is unique with logical suffix handling
                $existingAliases = array_column($entries, 'alias');
                unset($existingAliases[$key]); // Exclude the current entry's alias from the check

                $baseAlias = $originalAlias;
                $suffix = 0;

                foreach ($existingAliases as $existingAlias) {
                    if (preg_match('/^' . preg_quote($baseAlias, '/') . '-(\d+)$/', $existingAlias, $matches)) {
                        $suffix = max($suffix, (int) $matches[1]);
                    }
                }

                // Increment suffix for logical uniqueness
                $entries[$key]['alias'] = $suffix > 0 ? $baseAlias . '-' . ($suffix + 1) : $baseAlias;

                // Save the updated entries
                $this->saveEntry($entries);
                Message::set('success', self::SUCCESS_MESSAGES['entry_restored']);
                return true;
            }
        }

        Message::set('error', self::ERROR_MESSAGES['restore_failed']);
        return false;
    }

    /**
     * Deletes an image file from the server and its metadata from the database.
     *
     * @param string $imageName The name of the image file to be deleted.
     * @return bool True if deletion was successful, false otherwise.
     */
    public function deleteImage(string $imageName): bool
    {
        $filePath = $this->uploadDir . $imageName;

        // Check if the file exists before attempting to delete
        if (file_exists($filePath)) {
            if (unlink($filePath)) {
                // If file deletion is successful, remove the associated metadata
                $this->removeMetadataEntry($imageName);
                return true; // Return true on success
            } else {
                Message::set('error', self::ERROR_MESSAGES['file_delete_failed']);
                return false; // Return false if unlink fails
            }
        } else {
            Message::set('error', self::ERROR_MESSAGES['file_not_found']);
            return false; // Return false if file not found
        }
    }

    /**
     * Clear the image for a specific blog entry by its ID.
     *
     * @param int $id The ID of the blog entry to clear the image for.
     * @return bool True on success, false on failure.
     */
    public function clearPostImage(int $id): bool
    {
        $entries = $this->getAllEntries();

        foreach ($entries as $key => $entry) {
            if ($entry['id'] === $id) {
                // Delete the existing image file
                if (!empty($entry['image'])) {
                    $this->deleteImage($entry['image']); // This deletes the file from the server
                }

                // Clear the image reference in the entry
                $entries[$key]['image'] = ''; // Set image to an empty string

                // Save the updated entries
                $this->saveEntry($entries);
                return true; // Successfully cleared the image
            }
        }

        return false; // Entry not found or image could not be cleared
    }

    /**
     * Helper function to remove the metadata entry associated with a deleted image.
     *
     * @param string $imageName The name of the image file whose metadata is to be removed.
     * @return bool True if metadata removal was successful, false otherwise.
     */
    private function removeMetadataEntry(string $imageName): bool
    {
        $databaseFilePath = $this->uploadDir . 'database.json';

        // Check if the metadata file exists
        if (!file_exists($databaseFilePath)) {
            Message::set('error', self::ERROR_MESSAGES['metadata_not_found']);
            return false; // Return false if metadata not found
        }

        // Read and decode the JSON file
        $data = file_get_contents($databaseFilePath);
        $metadata = json_decode($data, true);

        // If no metadata is found, return false
        if (!is_array($metadata)) {
            Message::set('error', self::ERROR_MESSAGES['no_metadata_found']);
            return false; // Return false if there is no valid metadata
        }

        // Filter out the metadata entry corresponding to the image name
        $updatedMetadata = array_filter($metadata, fn($entry) => $entry['imageName'] !== $imageName);

        // Write the updated metadata back to the database.json
        if (file_put_contents($databaseFilePath, json_encode(array_values($updatedMetadata), JSON_PRETTY_PRINT)) === false) {
            Message::set('error', self::ERROR_MESSAGES['metadata_update_failed']);
            return false; // Return false if writing fails
        }

        return true; // Return true if everything was successful
    }

    /**
     * Retrieve blog entries that match the search query in title, text, or hashtags.
     *
     * @param string $query The search query.
     * @return array The list of matching blog entries.
     */
    public function searchEntries(string $query): array
    {
        // Sanitize the input query
        $query = sanitizeInput($query);

        // Limit query length to prevent performance issues
        if (strlen($query) > 100) {
            Message::set('error', 'Search query is too long.');
            return [];
        }

        $entries = $this->getAllEntries();
        $filteredEntries = [];

        foreach ($entries as $entry) {
            // Check if the query is found in the title, text, or hashtags (case-insensitive)
            if (
                stripos($entry['title'], $query) !== false ||
                stripos($entry['text'], $query) !== false ||
                array_filter($entry['hashtags'], function ($hashtag) use ($query) {
                    return stripos($hashtag, $query) !== false;
                })
            ) {
                $filteredEntries[] = $entry;
            }
        }

        return $filteredEntries; // Return the filtered entries
    }

    /**
     * Add a comment to a blog entry.
     *
     * @param int $postId The ID of the blog entry to comment on.
     * @param string $name The name of the commenter.
     * @param string $email The email of the commenter.
     * @param string $website The website of the commenter.
     * @param string $commentText The content of the comment.
     * @return bool True on success, false on failure.
     */
    public function addComment(int $postId, string $name, string $email, string $website, string $commentText, $captchaAnswer, $hashedAnswer): bool
    {
        // Clear any existing messages before processing
        Message::clear();

        // Create an instance of the Captcha class
        $captcha = new Captcha();

        // Check if the blog post exists
        $post = $this->getEntryById($postId);
        if (!$post) {
            Message::set('error', self::ERROR_MESSAGES['comment_failed']);
            return false;
        }

        // Check if the Captcha protection is enabled
        $captchaSetting = ModuleLoader::getSetting('Captcha', 'captcha') ?? true;

        // Validate the captcha answer
        if ($captchaSetting && !$captcha->validateCaptcha($captchaAnswer, $hashedAnswer)) {
            Message::set('error', self::ERROR_MESSAGES['comment_failed_captcha']);
            return false;
        }

        // Default name if not provided
        if (empty($name)) {
            $name = 'Anon';
        }

        // Sanitize the name and comment text
        $name = sanitizeInput($name);
        $commentText = sanitizeInput($commentText);

        // Validate and sanitize email only if it's not empty
        if (!empty($email)) {
            $email = filter_var($email, FILTER_SANITIZE_EMAIL);
            if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
                Message::set('error', 'Invalid email address.');
                return false;
            }
        }

        // Format and validate the website URL
        if (!empty($website)) {
            // Sanitize the website URL
            $website = sanitizeInput($website);

            // Use formatUrl function to standardize the website URL
            $website = formatUrl($website);
        }

        // Prepare the comment data
        $comments = $this->getComments();
        $comment = [
            'id' => $this->getNextCommentId(),
            'post_id' => $postId,
            'name' => $name,
            'email' => $email,
            'website' => $website,
            'text' => $commentText,
            'created_at' => date('Y-m-d H:i:s'),
        ];

        // Add the new comment to the list of comments
        $comments[] = $comment;
        $this->saveComments($comments);

        // Set success message only after adding the comment successfully
        Message::set('success', self::SUCCESS_MESSAGES['comment_added']);

        // Return true for successful addition
        return true;
    }

    /**
     * Retrieve all comments for a specific blog entry.
     *
     * @param int $postId The ID of the blog entry.
     * @return array The list of comments for the specified entry.
     */
    public function getCommentsByPostId(int $postId): array
    {
        $comments = $this->getComments();
        return array_filter($comments, function ($comment) use ($postId) {
            return $comment['post_id'] === $postId;
        });
    }

    /**
     * Edit an existing comment.
     *
     * @param array $data The updated comment data.
     * @return void
     */
    public function editComment(array $data): void
    {
        $comments = $this->getComments();
        $commentFound = false; // Flag to check if comment is found

        foreach ($comments as &$comment) {
            if ($comment['id'] === $data['id']) {
                // Update the comment fields
                $comment['name'] = $data['name'];
                $comment['email'] = $data['email'];
                $comment['website'] = $data['website'];
                $comment['text'] = $data['text'];

                // Save the updated data
                $this->saveComments($comments);
                Message::set('success', self::SUCCESS_MESSAGES['comment_updated']); // Success message
                $commentFound = true; // Set the flag to true if the comment is found
                break; // Exit the loop after updating
            }
        }

        // If the comment was not found, set an error message
        if (!$commentFound) {
            Message::set('error', self::ERROR_MESSAGES['comment_not_found']);
        }
    }

    /**
     * Delete a comment by its ID.
     *
     * @param int $id The ID of the comment to delete.
     * @return bool True on success, false on failure.
     */
    public function deleteComment(int $id): bool
    {
        $comments = $this->getComments();
        foreach ($comments as $key => $comment) {
            if ($comment['id'] === $id) {
                unset($comments[$key]);
                $this->saveComments(array_values($comments)); // Re-index array after deletion
                Message::set('success', self::SUCCESS_MESSAGES['comment_deleted']);
                return true;
            }
        }

        Message::set('error', self::ERROR_MESSAGES['comment_not_found']);
        return false;
    }

    /**
     * Delete all comments associated with a specific post ID.
     *
     * @param int $postId The ID of the post whose comments should be deleted.
     * @return bool True if comments were deleted, false otherwise.
     */
    public function deleteCommentsByPostId(int $postId): bool
    {
        $comments = $this->getComments(); // Fetch all comments

        // If there are no comments, just return false
        if (empty($comments)) {
            return false; // No comments to delete
        }

        $hasDeleted = false; // Flag to track if any comments were deleted

        foreach ($comments as $key => $comment) {
            if ($comment['post_id'] === $postId) {
                unset($comments[$key]); // Remove the comment associated with the post ID
                $hasDeleted = true; // Mark that a comment was deleted
            }
        }

        if ($hasDeleted) {
            $this->saveComments(array_values($comments)); // Re-index array after deletion
        }

        return $hasDeleted; // Return true if comments were deleted, false otherwise
    }

    /**
     * Get all comments.
     *
     * @return array The list of all comments sorted by created_at in descending order.
     */
    public function getComments(): array
    {
        $filePath = CONTENT_DIR . self::BLOG_COMMENTS_DATA_FILE;

        // Check if the PHP file exists and is readable
        if (file_exists($filePath)) {
            $comments = include($filePath); // Include the PHP file and get the returned array
        } else {
            // If the file doesn't exist, return an empty array
            $comments = [];
        }

        // Sort comments by created_at in descending order
        usort($comments, function ($a, $b) {
            return strtotime($b['created_at']) - strtotime($a['created_at']);
        });

        return $comments;
    }

    /**
     * Count the number of comments for a specific blog entry.
     *
     * @param int $postId The ID of the blog entry.
     * @return int The number of comments for the specified entry.
     */
    public function countCommentsForPost(int $postId): int
    {
        $comments = $this->getCommentsByPostId($postId);
        return count($comments);
    }

    /**
     * Get the appropriate comment count text for display.
     *
     * @param int $count The number of comments.
     * @return string The formatted comment count text.
     */
    public function getCommentCountText(int $count): string
    {
        return $count === 1 ? 'Comment' : 'Comments';
    }

    /**
     * Saves the comments to a PHP file.
     *
     * @param array $comments The comments to save.
     */
    private function saveComments(array $comments): void
    {
        $filePath = CONTENT_DIR . self::BLOG_COMMENTS_DATA_FILE;

        // Prepare the PHP content that should be saved in the file
        $phpContent = "<?php\nreturn " . var_export($comments, true) . ";\n";

        // Write the PHP content to the file
        file_put_contents($filePath, $phpContent);
    }

    /**
     * Generates a unique ID for a new comment.
     *
     * @return int A unique comment ID.
     */
    private function getNextCommentId(): int
    {
        // Get all existing comment IDs.
        $existingCommentIds = array_column($this->getComments(), 'id');

        do {
            $id = random_int(1000, 100000); // Generate a random ID
        } while (in_array($id, $existingCommentIds, true)); // Repeat until a unique ID is found

        return $id;
    }

    /**
     * Get the username by user ID.
     *
     * @param int $userId The user ID.
     * @return string|null The username or null if not found.
     */
    public function getUsernameById(int $userId): ?string
    {
        return SimpleLogin::getUsernameById($userId);
    }

    /**
     * Generate the URL for a blog post image.
     *
     * @param string|null $imagePath The path to the image file (optional).
     * @param int|null $userId The user ID whose profile picture may be used (optional).
     * @return string The full URL to the image.
     */
    public function getPostImage(?string $imagePath, ?int $userId = null): string
    {
        // Check if the 'profilePictureAsPostImage' setting is enabled and if a user ID is provided
        if ($userId !== null && ModuleLoader::getSetting('SimpleBlog', 'profilePictureAsPostImage') === true) {
            // Try to get the user's profile picture
            $userProfilePicture = $this->auth->getUserProfilePicture($userId);

            // If a profile picture exists, use it as the post image
            if (!empty($userProfilePicture)) {
                return $userProfilePicture; // Return the profile picture URL
            }
        }

        // If no image path is provided, check for a default post image setting
        if (empty($imagePath)) {
            // Check if the 'defaultPostImage' setting exists and is not empty
            $defaultImage = ModuleLoader::getSetting('SimpleBlog', 'defaultPostImage');

            // If there's a default image set, check if it's a full URL or a local path
            if (!empty($defaultImage)) {
                // Check if the default image is a full URL
                if (filter_var($defaultImage, FILTER_VALIDATE_URL)) {
                    return $defaultImage; // Return the full URL if it's valid
                } else {
                    // If it's not a full URL, assume it's a local path and construct the URL
                    $imagePath = $defaultImage;
                }
            } else {
                // If no default image is set, return an empty string
                return '';
            }
        }

        // Normalize the URL based on the provided or default image path
        return normalizeUrl(generateBaseUrl() . basename(CONTENT_DIR) . self::IMAGE_UPLOAD_DIR . $imagePath);
    }

    /**
     * Replace shortcodes in a template using data provided.
     *
     * @param string $template The template that contains shortcodes.
     * @param array $data Data to fill in the shortcodes.
     * @return string The template with shortcodes replaced by actual data.
     */
    public function parsePost($template, $data)
    {
        // Replace basic shortcodes like title, username, etc.
        $template = $this->replaceBasicShortcodes($template, $data);

        // Replace shortcodes for hashtags
        $template = $this->processHashtags($template, $data['hashtags'] ?? []);

        // Handle the image if provided
        $template = $this->processImage($template, $data['image'] ?? null, $data['user_id'] ?? null);

        // Handle comments link
        $commentsCount = $this->countCommentsForPost($data['id']);

        // Show comments amount
        $template = preg_replace('/\[comments_amount\]/', $commentsCount, $template);

        // Replace the comments link
        $template = $this->replaceCommentsLinkShortcodes($template, $commentsCount, $data['alias']);

        return $template;
    }

    public function parseComments($template, $data)
    {
        // Handle comments link
        $commentsCount = $this->countCommentsForPost($data['id']);

        // Show comments amount
        $template = preg_replace('/\[comments_amount\]/', $commentsCount, $template);

        // Handle comments if provided
        $template = $this->processComments($template, $data['comments'] ?? [], $data['alias']);

        return $template; // Return the final template with all shortcodes filled in
    }

    public function parseCommentForm($template, $data)
    {
        // Handle messages if present
        $template = $this->processMessages($template, $data['messages'] ?? []);

        // Process smileys (newly added)
        $template = $this->processSmileys($template);

        // Process CAPTCHA (antispam) shortcode
        $template = $this->processCaptcha($template, $data);

        return $template;
    }

    /**
     * Handle hashtags in the template.
     *
     * @param string $template The template containing shortcodes.
     * @param array $hashtags The hashtags to replace in the template.
     * @return string The template with hashtags replaced or removed.
     */
    private function processHashtags($template, array $hashtags)
    {
        // Define the pattern for the hashtags block with an optional separator attribute
        $conditionalPattern = '/\[hashtags(?:\s+separator="([^"]*)")?\](.*?)\[\/hashtags\]/s';

        // Define the pattern for the hashtag shortcode with an optional before attribute
        $hashtagPattern = '/\[hashtag(?:\s+before="([^"]*)")?\]/';

        // Check if there are any hashtags
        if (!empty($hashtags)) {
            // Replace the [hashtags] shortcode with the actual hashtags
            $template = preg_replace_callback($conditionalPattern, function ($matches) use ($hashtags, $hashtagPattern) {
                // Check if a separator is provided in the shortcode
                $separator = isset($matches[1]) && !empty($matches[1]) ? $matches[1] : ' '; // Default separator is space

                $content = $matches[2];

                // Replace the [hashtag] shortcode with the actual hashtags
                $content = preg_replace_callback($hashtagPattern, function ($matches) use ($hashtags, $separator) {
                    $before = isset($matches[1]) ? $matches[1] : '';
                    $formattedHashtags = '';

                    // Iterate over each hashtag and format it with the separator
                    foreach ($hashtags as $tag) {
                        $tag = sanitizeInput($tag);
                        $formattedHashtags .= "<a href=\"" . getFullBaseUrl() . "?hashtag=" . urlencode($tag) . "\">" . $before . "{$tag}</a>" . $separator;
                    }

                    // Remove the trailing separator after all hashtags are added
                    return rtrim($formattedHashtags, $separator);
                }, $content);

                return $content;
            }, $template);
        } else {
            // If no hashtags, remove the entire conditional block
            $template = preg_replace($conditionalPattern, '', $template);
        }

        return $template;
    }

    /**
     * Replace basic shortcodes like title, username, text, permalink, and conditional read more link in the template.
     *
     * @param string $template The template with shortcodes.
     * @param array $data The data to fill in the shortcodes.
     * @return string The updated template.
     */
    private function replaceBasicShortcodes($template, $data)
    {
        // Track if text was shortened
        $textShortened = false;

        // Handle title and author shortcodes
        $template = preg_replace('/\[title\]/', sanitizeInput($data['title'] ?? ''), $template);
        $template = preg_replace('/\[author\]/', sanitizeInput($this->getUsernameById($data['user_id'] ?? 0)), $template);

        // Handle [text] shortcode with optional length attribute
        $template = preg_replace_callback('/\[text(?:\s+length="(\d+)")?\]/', function ($matches) use ($data, &$textShortened) {
            $text = parseSmileys($data['text'] ?? '');

            // Check if length is specified for truncation
            if (!empty($matches[1])) {
                $length = (int) $matches[1];
                $shortenedText = shortenString($text, $length);

                // Check if the text was actually shortened by comparing the length
                if (strlen($shortenedText) < strlen($text)) {
                    $textShortened = true;
                }

                return $shortenedText;
            }

            // No truncation, original text is used
            return $text;
        }, $template);

        // Handle created_at with optional date format
        $template = preg_replace_callback('/\[date(?:\s+"([^"]+)")?\]/', function ($matches) use ($data) {
            $createdAt = $data['created_at'] ?? '';
            $format = !empty($matches[1]) ? $matches[1] : ModuleLoader::getSetting('SimpleBlog', 'dateFormat');

            if (!empty($createdAt)) {
                try {
                    $date = new DateTime($createdAt);
                    return sanitizeInput($date->format($format));
                } catch (Exception $e) {
                    return sanitizeInput($createdAt); // Return unformatted if invalid date
                }
            }
            return ''; // Return empty if no created_at date
        }, $template);

        $permalink = getFullBaseUrl() . "?alias=" . sanitizeInput($data['alias'] ?? '');

        // Handle [permalink] shortcode
        $template = preg_replace('/\[permalink\]/', $permalink, $template);

        // Process [read_more_link]...[/read_more_link] block based on text shortening
        $template = $this->processReadMoreLink($template, $textShortened);

        return $template;
    }

    /**
     * Conditionally includes or excludes the [read_more_link] block.
     *
     * If text was shortened, this function will replace the [read_more_link]...[/read_more_link] 
     * block with the content inside the tags. Otherwise, it removes the block.
     *
     * @param string $template The template containing the [read_more_link] block.
     * @param bool $textShortened Indicates whether the text was shortened.
     * @return string The updated template with the [read_more_link] block processed.
     */
    private function processReadMoreLink($template, $textShortened)
    {
        return preg_replace_callback('/\[read_more_link\](.*?)\[\/read_more_link\]/s', function ($matches) use ($textShortened) {
            if ($textShortened) {
                // Return the content inside [read_more_link] if text was shortened
                return $matches[1];
            }
            // Remove the block if text was not shortened
            return '';
        }, $template);
    }

    /**
     * Process the image URL in the template.
     *
     * @param string $template The template containing shortcodes.
     * @param string|null $image The image to replace in the template.
     * @param int $user_id The user ID to check for profile picture.
     * @return string The template with the image URL replaced or block removed.
     */
    private function processImage($template, $image, $user_id)
    {
        // Default image URL initialization
        $imageUrl = '';

        // Step 1: Check if a post image is provided first
        if (!empty($image)) {
            // If a post image is provided, use it
            $imageUrl = $this->getPostImage($image);
        }

        // Step 2: If no post image is provided, check if profile picture should be used as fallback
        if (empty($imageUrl)) {
            // Get the profile picture setting
            $profilePictureSetting = ModuleLoader::getSetting('SimpleBlog', 'profilePictureAsPostImage');

            // If the profile picture should act as a fallback
            if ($profilePictureSetting) {
                // Check if the user has a profile picture
                $userProfilePicture = $this->auth->getUserProfilePicture($user_id);

                // If the user has a valid profile picture (ends with image extension)
                if (!empty($userProfilePicture) && preg_match('/\.(jpg|jpeg|png|gif|bmp|webp)$/i', $userProfilePicture)) {
                    $imageUrl = $userProfilePicture;
                }
            }
        }

        // Step 3: Process the image block depending on whether an image was found
        if (empty($imageUrl)) {
            // Remove the entire [post_image] block if no image is found
            $template = preg_replace('/\[post_image\](.*?)\[\/post_image\]/s', '', $template);
        } else {
            // Replace the [image_url] shortcode within the [post_image] block
            $template = preg_replace_callback('/\[post_image\](.*?)\[\/post_image\]/s', function ($matches) use ($imageUrl) {
                $imageBlock = $matches[1]; // Extract the original content within the block
                $imageBlock = preg_replace('/\[image_url\]/', $imageUrl, $imageBlock); // Replace the image URL
                return $imageBlock; // Return the modified block
            }, $template);
        }

        return trim($template);
    }

    /**
     * Process comments in the template.
     *
     * @param string $template The template containing shortcodes.
     * @param array $comments The comments to replace in the template.
     * @param string $entryAlias The alias of the entry.
     * @return string The template with comments replaced.
     */
    private function processComments($template, array $comments, $entryAlias)
    {
        // Define the pattern for the [comments]...[/comments] block
        $commentsPattern = '/\[comments\](.*?)\[\/comments\]/s';

        // Check if there are any comments
        if (!empty($comments)) {
            // Process each comment in the block
            $formattedComments = '';

            // Extract the comment template from the [comments] block
            if (preg_match($commentsPattern, $template, $matches)) {
                $commentTemplate = $matches[1]; // The content inside the [comments] block

                // Loop through each comment and replace the placeholders
                foreach ($comments as $comment) {
                    $formattedComment = $commentTemplate;

                    // Replace the basic comment shortcodes
                    $formattedComment = preg_replace('/\[comment_name\]/', $comment['name'] ?? 'Unknown', $formattedComment);
                    $formattedComment = preg_replace('/\[comment_text\]/', parseSmileys($comment['text']) ?? 'No comment text', $formattedComment);

                    // Handle comment_created_at with optional date format
                    $formattedComment = preg_replace_callback('/\[comment_date(?:\s+"([^"]+)")?\]/', function ($matches) use ($comment) {
                        $createdAt = $comment['created_at'] ?? '';
                        $format = isset($matches[1]) ? trim($matches[1], '"') : ModuleLoader::getSetting('SimpleBlog', 'commentDateFormat'); // Default format if none provided

                        // Format the date if it is a valid date string
                        if (!empty($createdAt)) {
                            try {
                                $date = new DateTime($createdAt);
                                return sanitizeInput($date->format($format)); // Format the date with the given or default format
                            } catch (Exception $e) {
                                return sanitizeInput($createdAt); // Return unformatted date if there's an error
                            }
                        }
                        return ''; // Return empty if no date provided
                    }, $formattedComment);

                    // Handle website condition with the [website] wrapper and optional attributes
                    $formattedComment = preg_replace_callback('/\[website(.*?)\](.*?)\[\/website\]/s', function ($matches) use ($comment) {
                        // Parse the optional attributes (text and class)
                        $attributes = $matches[1];
                        preg_match('/icon="(.*?)"/', $attributes, $classMatch);

                        $class = $classMatch[1] ?? ''; // Default class is empty

                        // If the website exists, wrap the content within the [website][/website] tags with an anchor tag and optional attributes
                        if (!empty($comment['website'])) {
                            $websiteLink = sprintf('<a href="%s" target="_blank">', $comment['website']);

                            // Wrap the content within the [website][/website] tags with the anchor tag and a span with the specified class
                            return $websiteLink . '<span class="' . $class . '"></span> ' . $matches[2] . '</a>';
                        } else {
                            // If no website exists, return only the content within the tags
                            return $matches[2];
                        }
                    }, $formattedComment);

                    // Handle the comment author linked shortcode
                    $commentAuthor = sanitizeInput($comment['name'] ?? 'Unknown');

                    // Replace the [comment_author] placeholder with the actual author name
                    $formattedComment = preg_replace('/\[comment_author\]/', $commentAuthor, $formattedComment);

                    // Append the formatted comment to the overall formatted comments
                    $formattedComments .= $formattedComment;
                }

                // Replace the comments block with the formatted comments
                $template = preg_replace($commentsPattern, $formattedComments, $template);
            }
        } else {
            // No comments: replace with a message or remove the block entirely
            $template = preg_replace($commentsPattern, '<p>No comments yet. Be the first to comment!</p>', $template);
        }

        return $template;
    }

    // Generate the comment link shortcodes
    private function replaceCommentsLinkShortcodes($template, $commentsCount, $entryAlias)
    {
        // Define the pattern for the [comments_link]...[/comments_link] block
        $commentsLinkPattern = '/\[comments_link\](.*?)\[\/comments_link\]/s';

        // Define the link URL
        $commentsLinkHref = getFullBaseUrl() . "?alias=" . sanitizeInput($entryAlias) . "#comments";

        // Replace the [comments_link] block if it exists
        if (preg_match($commentsLinkPattern, $template, $matches)) {
            // Extract the inner content of the [comments_link] block
            $commentsLinkContent = $matches[1];

            // Replace [comments_amount] in the link content
            $commentsLinkContent = str_replace('[comments_amount]', $commentsCount, $commentsLinkContent);

            // Check if [comments_link_text] has custom singular/plural text
            $commentsLinkContent = preg_replace_callback('/\[link_text\s+singular="([^"]+)"\s+plural="([^"]+)"\]/', function ($matches) use ($commentsCount) {
                // $matches[1] = singular value
                // $matches[2] = plural value
                $singular = $matches[1] ?? 'Comment'; // Default singular form if not provided
                $plural = $matches[2] ?? 'Comments'; // Default plural form if not provided

                // Return the correct form based on the comment count
                return ($commentsCount === 1) ? $singular : $plural;
            }, $commentsLinkContent);

            // Replace [comments_link_text] with the custom or default text
            $commentsLinkContent = preg_replace_callback('/\[link_text\]/', function ($matches) use ($commentsCount) {
                // Return the default comment count text
                return $this->getCommentCountText($commentsCount);
            }, $commentsLinkContent);

            // Wrap the content in an <a> tag with the appropriate href
            $commentsLinkHtml = "<a href=\"{$commentsLinkHref}\">" . $commentsLinkContent . "</a>";

            // Replace the entire [comments_link]...[/comments_link] block with the HTML
            $template = preg_replace($commentsLinkPattern, $commentsLinkHtml, $template);
        }

        return $template;
    }

    /**
     * Process the [antispam] shortcode and replace it with the CAPTCHA question and hidden answer field in custom HTML.
     *
     * @param string $template The template that contains shortcodes.
     * @param array $data Data to fill in the shortcodes (optional).
     * @return string The template with the [antispam] shortcode replaced by the CAPTCHA HTML.
     */
    public function processCaptcha($template, $data = [])
    {
        // Create an instance of the Captcha class
        $captcha = new Captcha();

        // Check if CAPTCHA is enabled
        if (ModuleLoader::getSetting('Captcha', 'captcha')) {
            // Generate the CAPTCHA question and hashed answer
            $captchaData = $captcha->getQuestion();
            $captchaQuestion = $captchaData['question'];
            $hashedAnswer = $captchaData['hashedAnswer'];

            // Define the regex pattern to find the [antispam] shortcode and extract its content
            $pattern = '/\[antispam\](.*?)\[\/antispam\]/s';

            // Perform the replacement in the template
            $template = preg_replace_callback($pattern, function ($matches) use ($captchaQuestion, $hashedAnswer) {
                // $matches[1] contains the content inside the [antispam] shortcode
                $captchaHtml = $matches[1];

                // Replace [question] with the actual CAPTCHA question
                $captchaHtml = str_replace('[question]', sanitizeInput($captchaQuestion), $captchaHtml);

                // Insert the hidden field with the hashed answer
                $hiddenFieldHtml = '<input type="hidden" name="captcha_hashed_answer" value="' . sanitizeInput($hashedAnswer) . '">';
                $captchaHtml .= $hiddenFieldHtml;

                // Return the processed HTML with the question and hidden hashed answer field
                return $captchaHtml;
            }, $template);
        } else {
            // If CAPTCHA is disabled, remove the [antispam] tags and content entirely
            $template = preg_replace('/\[antispam\].*?\[\/antispam\]/s', '', $template);
        }

        return $template;
    }

    /**
     * Process the [show_smileys] and [smileys] shortcodes.
     *
     * @param string $template The template containing shortcodes.
     * @return string The template with smileys shortcodes replaced.
     */
    private function processSmileys($template)
    {
        // Define the pattern for the [show_smileys]...[/show_smileys] block
        $showSmileysPattern = '/\[show_smileys\](.*?)\[\/show_smileys\]/s';

        // Initialize Smileys object
        $smileys = new Smileys();
        // Retrieve all smileys
        $allSmileys = $smileys->getAllSmileys();

        // Check if there are any smileys available
        if (!empty($allSmileys)) {
            // Process the smileys block
            if (preg_match($showSmileysPattern, $template, $matches)) {
                $smileysTemplate = $matches[1]; // The content inside the [show_smileys] block

                // Replace the [smileys] shortcode with the actual smileys list
                $smileysContent = '';

                // Initialize the smileys content string
                $smileysContent = '<div class="simplify-smileys">'; // Open the parent div for all smileys

                // Loop through each smiley and generate the HTML
                foreach ($allSmileys as $smiley) {
                    $smileysContent .= sprintf(
                        '<img src="%s" alt="%s" class="smiley" data-smiley="%s" />',
                        normalizeUrl(generateBaseUrl() . basename(CONTENT_DIR) . Smileys::IMAGE_UPLOAD_DIR . ltrim($smiley['imageName'], '/')),
                        $smiley['metadata']['smileyCode'],
                        $smiley['metadata']['smileyCode']
                    );
                }

                // Close the parent div
                $smileysContent .= '</div>';


                // Replace the [smileys] tag with the generated smileys HTML
                $smileysTemplate = preg_replace('/\[smileys\]/', $smileysContent, $smileysTemplate);

                // Replace the [show_smileys] block with the processed content
                $template = preg_replace($showSmileysPattern, $smileysTemplate, $template);
            }
        } else {
            // If no smileys are found, remove the [show_smileys] block or replace with a fallback message
            $template = preg_replace($showSmileysPattern, '', $template);
        }

        return $template;
    }

    /**
     * Handle messages in the template.
     *
     * @param string $template The template containing shortcodes.
     * @param array $messages The list of messages to be processed.
     * @return string The processed template with messages formatted.
     */
    private function processMessages($template, array $messages)
    {
        // Define the pattern for the [messages]...[/messages] block
        $messagesPattern = '/\[messages\](.*?)\[\/messages\]/s';

        // Check if there are any messages
        if (!empty($messages)) {
            // Process each message in the block
            $formattedMessages = '';

            // Extract the message template from the [messages] block
            if (preg_match($messagesPattern, $template, $matches)) {
                $messageTemplate = $matches[1]; // The content inside the [messages] block

                // Loop through each message and replace the placeholders
                foreach ($messages as $message) {
                    $formattedMessage = $messageTemplate;

                    // Replace the {type} and {text} placeholders
                    $formattedMessage = preg_replace('/\[message_type\]/', $message['type'] ?? 'info', $formattedMessage);
                    $formattedMessage = preg_replace('/\[message_text\]/', $message['text'] ?? 'No message content', $formattedMessage);

                    // Append the formatted message to the overall formatted messages
                    $formattedMessages .= $formattedMessage;
                }

                // Replace the [messages] block with the formatted messages
                $template = preg_replace($messagesPattern, $formattedMessages, $template);
            }
        } else {
            // No messages: remove the entire block
            $template = preg_replace($messagesPattern, '', $template);
        }

        return $template;
    }

    /**
     * Format messages into a string for template replacement.
     *
     * @param array $messages The messages to format.
     * @return string The formatted messages combined into one string.
     */
    private function formatMessages(array $messages)
    {
        $messagesText = ''; // Start with an empty string
        foreach ($messages as $msg) {
            $messagesText .= sanitizeInput($msg['text'] ?? ''); // Add each message, sanitizing the text
        }
        return $messagesText; // Return the combined messages
    }

    /**
     * Load a template file and fill it with data.
     *
     * @param string $templateFilePath Path to the template file.
     * @param array $data Data to replace in the template.
     * @param callable $parseCallback The function to parse the template (e.g., parsePost, parseComments, parseCommentForm).
     * @return string The content of the parsed template.
     */
    public function loadAndParseTemplate($templateFilePath, array $data, callable $parseCallback)
    {
        // Load the template from a file
        $templateContent = file_get_contents($templateFilePath);

        // Use the provided callback function to parse the template with data
        return $parseCallback($templateContent, $data);
    }

    /**
     * Restore a template file from its backup version.
     *
     * @param string $templateName The name of the template to restore (without .php extension).
     * @return bool True if the restoration was successful, false otherwise.
     */
    public function restoreTemplate($templateName): bool
    {
        $originalFile = MODULES_DIR . '/SimpleBlog/templates/' . $templateName . '.php';
        $backupFile = MODULES_DIR . '/SimpleBlog/templates/' . $templateName . '.backup.php';

        // Check if the backup file exists
        if (file_exists($backupFile)) {
            // Read the content of the backup file
            $backupContent = file_get_contents($backupFile);

            // Write the content back to the original template file
            if (file_put_contents($originalFile, $backupContent) !== false) {
                return true; // Restoration successful
            }
        }

        return false; // Restoration failed
    }

    /**
     * Restore a css file from its backup version.
     *
     * @param string $templateName The name of the css to restore (without .css extension).
     * @return bool True if the restoration was successful, false otherwise.
     */
    public function restoreCSS($cssName)
    {
        $originalFile = MODULES_DIR . '/SimpleBlog/css/' . $cssName . '.css';
        $backupFile = MODULES_DIR . '/SimpleBlog/css/' . $cssName . '.backup.css';

        // Check if the backup file exists
        if (file_exists($backupFile)) {
            // Read the content of the backup file
            $backupContent = file_get_contents($backupFile);

            // Write the content back to the original template file
            if (file_put_contents($originalFile, $backupContent) !== false) {
                return true; // Restoration successful
            }
        }

        return false; // Restoration failed
    }
}
