<?php
// Protects the module files from being accessed directly
if (!defined('ALLOW_DIRECT_ACCESS')) {
    die("Access denied.");
}

// Create an instance of SimpleBlog
$blog = new SimpleBlog();

$module = new ModuleLoader();

// Initialize variables for template and CSS file editing
$templateName = isset($_GET['template']) ? sanitizeInput($_GET['template']) : null;
$cssName = isset($_GET['css']) ? 'blog.css' : null;

// Define paths based on selection
$templatePath = $templateName ? __DIR__ . "/templates/$templateName" : null;
$cssPath = $cssName ? __DIR__ . "/css/$cssName" : null;

// Determine which path is active
$activePath = $templatePath ? $templatePath : ($cssPath ? $cssPath : null);
$fileType = $templatePath ? 'template' : ($cssPath ? 'css' : null);

// Initialize content variable
$templateContent = '';

// Fetch the existing template or CSS content if a valid path is set
if ($activePath && file_exists($activePath)) {
    $templateContent = file_get_contents($activePath);
} else {
    $templateName = $cssName = null; // No valid file selected, so we don't show the edit form
}

// Handle form submission to update the template or CSS file
if ($_SERVER['REQUEST_METHOD'] === 'POST' && $activePath) {
    $templateContent = $_POST['template_content'] ?? ''; // Get the content from the form

    // Save the new content back to the file
    if (file_put_contents($activePath, $templateContent) !== false) {
        // Determine success message based on file type
        $successMessage = ($fileType === 'css') ? 'CSS file updated successfully!' : 'Template updated successfully!';
        Message::set('success', $successMessage);
        // Redirect to avoid form resubmission
        if ($fileType === 'css') {
            safeRedirect("module.php?module=SimpleBlog&file=edit_templates.php&css=$cssName");
        } else {
            safeRedirect("module.php?module=SimpleBlog&file=edit_templates.php&template=$templateName");
        }
    } else {
        // Determine error message based on file type
        $errorMessage = ($fileType === 'css') ? 'Error updating CSS file. Please try again.' : 'Error updating template. Please try again.';
        Message::set('error', $errorMessage);
    }
}

// Handle reset requests
if ($_SERVER['REQUEST_METHOD'] === 'GET') {
    if (isset($_GET['reset'])) {
        if ($fileType === 'template' && $templateName !== null) {
            $templateToRestore = basename($_GET['reset'], '.php');

            if ($blog->restoreTemplate($templateToRestore)) {
                Message::set('success', "The template '{$templateToRestore}.php' has been restored successfully.");
                safeRedirect("module.php?module=SimpleBlog&file=edit_templates.php&template={$templateName}");
            } else {
                Message::set('error', "Failed to restore the template '{$templateToRestore}.php'. Please check if the backup file exists.");
            }
        } elseif ($fileType === 'css' && $cssName !== null) {
            $cssToRestore = basename($_GET['reset'], '.css');

            if ($blog->restoreCSS($cssToRestore)) {
                Message::set('success', "The CSS file '{$cssToRestore}.css' has been restored successfully.");
                safeRedirect("module.php?module=SimpleBlog&file=edit_templates.php&css={$cssName}");
            } else {
                Message::set('error', "Failed to restore the CSS file '{$cssToRestore}.css'. Please check if the backup file exists.");
            }
        }
    }
}

// Retrieve any messages to display to the user
$messages = Message::get();
?>

<h1>Edit Templates and CSS</h1>

<!-- Display any messages -->
<?php if (!empty($messages)): ?>
    <?php foreach ($messages as $msg): ?>
        <p class="<?php echo sanitizeInput($msg['type']); ?>"><?php echo sanitizeInput($msg['text']); ?></p>
    <?php endforeach; ?>
<?php endif; ?>

<p>
    Here, you can effortlessly <em>edit the appearance</em> of your content. Utilize <strong>predefined shortcodes</strong> to simplify the HTML coding process and
    change the style of your entries with ease.
</p>

<br>

<!-- Template and CSS Selection Links -->
<nav class="action-links centered">
    <a href="<?= ModuleLoader::generateUrl('SimpleBlog', 'edit_templates.php', ['template' => 'post.php']); ?>"
        class="<?= ($templateName === 'post.php') ? 'active' : ''; ?>">
        <i class="fas fa-file-alt"></i> Post
    </a>

    <?php if (!ModuleLoader::getSetting('SimpleBlog', 'oneTemplate')): ?>
        <a href="<?= ModuleLoader::generateUrl('SimpleBlog', 'edit_templates.php', ['template' => 'view_post.php']); ?>"
            class="<?= ($templateName === 'view_post.php') ? 'active' : ''; ?>">
            <i class="fas fa-file-alt"></i> View Post
        </a>
    <?php endif; ?>

    <a href="<?= ModuleLoader::generateUrl('SimpleBlog', 'edit_templates.php', ['template' => 'comments.php']); ?>"
        class="<?= ($templateName === 'comments.php') ? 'active' : ''; ?>">
        <i class="fas fa-comments"></i> Comments
    </a>
    <a href="<?= ModuleLoader::generateUrl('SimpleBlog', 'edit_templates.php', ['template' => 'add_comment_form.php']); ?>"
        class="<?= ($templateName === 'add_comment_form.php') ? 'active' : ''; ?>">
        <i class="fas fa-plus"></i> Add Comment Form
    </a>
    <a href="<?= ModuleLoader::generateUrl('SimpleBlog', 'edit_templates.php', ['css' => 'blog.css']); ?>"
        class="<?= ($cssName === 'blog.css') ? 'active' : ''; ?>">
        <i class="fas fa-file-code"></i> CSS
    </a>
</nav>

<br>

<!-- Display template or CSS edit form if selected -->
<?php if ($activePath !== null): ?>
    <h2>Edit <?= ucfirst($fileType); ?>: <?php echo sanitizeInput($fileType === 'css' ? $cssName : $templateName); ?></h2>

    <!-- Form for editing the template or CSS -->
    <form method="post" class="form">
        <div>
            <label for="template_content">
                <?php echo ($fileType === 'css') ? 'CSS Content:' : 'Template Content:'; ?>
            </label>
            <textarea id="template_content" name="template_content" rows="20"
                required><?php echo htmlspecialchars($templateContent); ?></textarea>
        </div>

        <div>
            <div class="horizontal">
                <div>
                    <!-- Update Button -->
                    <button type="submit">
                        <i class="fas fa-save"></i> Update
                    </button>

                    <!-- Reset File Link -->
                    <a class="button"
                        href="<?= ModuleLoader::generateUrl('SimpleBlog', 'edit_templates.php', [$fileType => ($fileType === 'css') ? $cssName : $templateName]); ?>&reset=<?= htmlspecialchars(basename($activePath)); ?>"
                        onclick="return confirm('Are you sure you want to reset the <?= $fileType; ?>? This action cannot be undone.');">
                        <i class="fas fa-redo-alt"></i> Reset <?= ucfirst($fileType); ?>
                    </a>
                </div>

                <?php if ($fileType !== 'css'): ?>
                    <div>
                        <details>
                            <summary style="color: #777; font-size: 9pt; text-align: right;">
                                Basic Shortcodes
                            </summary>

                            <!-- Shortcodes based on the file type -->
                            <p style="font-size: 8pt; opacity: 0.5;">
                                Available shortcodes for
                                <?php echo sanitizeInput($fileType === 'css' ? 'CSS' : $templateName); ?>:<br>
                                <?php
                                // Define shortcodes based on the template name
                                $shortcodes = [];
                                switch ($templateName) {
                                    case 'add_comment_form.php':
                                        $shortcodes = [
                                            '[messages]&#x3C;p class=&#x22;[message_type]&#x22;&#x3E;[message_text]&#x3C;/p&#x3E;[/messages]' => 'Display success/error messages',
                                            '[antispam]&#x3C;div&#x3E;&#x3C;label for=&#x22;captcha&#x22;&#x3E;[question]&#x3C;/label&#x3E;&#x3C;input type=&#x22;text&#x22; id=&#x22;captcha&#x22; name=&#x22;captcha&#x22; required placeholder=&#x22;Enter the answer&#x22;&#x3E;&#x3C;/div&#x3E;[/antispam]' => 'Antispam field',
                                            '[show_smileys][smileys][/show_smileys]' => 'Show smileys',
                                        ];
                                        break;
                                    case 'comments.php':
                                        $shortcodes = [
                                            '[comments]...[/comments]' => 'Comments wrapper (important!)',
                                            '[comments_amount]' => 'Number of comments',
                                            '[comment_author]' => 'Comment author (text only)',
                                            '[website]Link Text[/website]' => 'Website link',
                                            '[website][comment_author][/website]' => 'Comment author (linked if website provided, text if not)',
                                            '[website icon="fas fa-home"][comment_author][/website]' => 'Custom FontAwesome icon for the website',
                                            '[comment_text]' => 'Comment text',
                                            '[comment_date]' => 'Comment date',
                                        ];
                                        break;
                                    case 'post.php':
                                    case 'view_post.php':
                                        $shortcodes = [
                                            '[title]' => 'Post title',
                                            '[permalink]' => 'The full link of the post',
                                            '[alias]' => 'The URL alias of the post',
                                            '[date]' => 'The date of the post',
                                            '[author]' => 'The name of the author',
                                            '[text]' => 'Post text',
                                            '[text length="100"]' => 'Post text shortened, e.g. 100 characters',
                                            '[read_more_link]&lt;a href="[permalink]"&gt;Read more&lt;/a&gt;[/read_more_link]' => 'Conditionally show read more link',
                                            '&lt;a href="[permalink]"&gt;Read more&lt;/a&gt;' => 'Permanent read more link',
                                            '[post_image]&lt;img src=&quot;[image_url]&quot;&gt;[/post_image]' => 'Post image',
                                            '[comments_amount]' => 'Number of comments',
                                            '[comments_link][link_text][/comments_link]' => 'The comments link',
                                            '[link_text singular="Thought" plural="Thoughts"]' => 'Custom comments link text',
                                            '[hashtags][hashtag][/hashtags]' => 'Post hashtags',
                                            '[hashtags separator=", "][hashtag][/hashtags]' => 'Custom separator for hashtags',
                                            '[hashtags][hashtag before="#"][/hashtags]' => 'Custom before symbol for hashtags',
                                        ];
                                        break;
                                    default:
                                        $shortcodes = [
                                            'No shortcodes available for this template.',
                                        ];
                                        break;
                                }

                                // Display the shortcodes
                                foreach ($shortcodes as $code => $description) {
                                    echo "<strong>{$description}:</strong><br><code class='click-to-copy'>{$code}</code><br>";
                                }
                                ?>
                            </p>
                        </details>

                        <?php if ($templateName !== 'add_comment_form.php'): ?>
                            <details>
                                <summary style="color: #777; font-size: 9pt; text-align: right;">
                                    Date Formats
                                </summary>

                                <p style="font-size: 8pt; opacity: 0.5;">
                                    Available date formats for <?php echo sanitizeInput($templateName); ?>:<br>
                                    <?php
                                    // Define date formats based on the template name
                                    $dateFormats = [];
                                    switch ($templateName) {
                                        case 'comments.php':
                                            $dateFormats = [
                                                '[comment_date "Y/m/d H:i"]' => 'Example: ' . date('Y/m/d H:i'),
                                                '[comment_date "l, F j, Y"]' => 'Example: ' . date('l, F j, Y'),
                                                '[comment_date "F j, Y"]' => 'Example: ' . date('F j, Y'),
                                                '[comment_date "Y-m-d"]' => 'Example: ' . date('Y-m-d'),
                                                '[comment_date "d.m.Y"]' => 'Example: ' . date('d.m.Y'),
                                                '[comment_date "M d, Y"]' => 'Example: ' . date('M d, Y'),
                                                '[comment_date "Y/m/d"]' => 'Example: ' . date('Y/m/d'),
                                                '[comment_date "d-m-Y H:i"]' => 'Example: ' . date('d-m-Y H:i'),
                                            ];
                                            break;
                                        case 'post.php':
                                        case 'view_post.php':
                                            $dateFormats = [
                                                '[date "Y/m/d H:i"]' => 'Example: ' . date('Y/m/d H:i'),
                                                '[date "l, F j, Y"]' => 'Example: ' . date('l, F j, Y'),
                                                '[date "F j, Y"]' => 'Example: ' . date('F j, Y'),
                                                '[date "Y-m-d"]' => 'Example: ' . date('Y-m-d'),
                                                '[date "d.m.Y"]' => 'Example: ' . date('d.m.Y'),
                                                '[date "M d, Y"]' => 'Example: ' . date('M d, Y'),
                                                '[date "Y/m/d"]' => 'Example: ' . date('Y/m/d'),
                                                '[date "d-m-Y H:i"]' => 'Example: ' . date('d-m-Y H:i'),
                                            ];
                                            break;
                                        default:
                                            $dateFormats = [
                                                'No date formats available for this template.',
                                            ];
                                            break;
                                    }

                                    // Display the date formats
                                    foreach ($dateFormats as $code => $example) {
                                        echo "<code class='click-to-copy'>{$code}</code> - {$example}<br>";
                                    }
                                    ?>
                                </p>
                            </details>
                        <?php endif; ?>
                    </div>
                <?php endif; ?>
            </div>
        </div>
    </form>
<?php endif; ?>

<script>
document.addEventListener("DOMContentLoaded", () => {
    // Select all elements with the class `click-to-copy`
    document.querySelectorAll(".click-to-copy").forEach((codeBlock) => {
        codeBlock.addEventListener("click", () => {
            // Create a temporary textarea to hold the code
            const tempTextarea = document.createElement("textarea");
            tempTextarea.value = codeBlock.textContent.trim();
            document.body.appendChild(tempTextarea);

            // Select and copy the code
            tempTextarea.select();
            document.execCommand("copy");
            document.body.removeChild(tempTextarea);

            // Provide visual feedback
            codeBlock.classList.add("copied");
            setTimeout(() => codeBlock.classList.remove("copied"), 1000);
        });
    });
});
</script>
