<?php
// Protects the module files from being accessed directly
if (!defined('ALLOW_DIRECT_ACCESS')) {
    die("Access denied.");
}

/**
 * Class Smileys
 *
 * Handles file uploads, deletions, and metadata management for smiley images.
 */
class Smileys
{
    // Current version
    const VERSION = "1.0.3";

    // Shows the current version of the module
    public function getVersion()
    {
        return self::VERSION;
    }

    // The directory where smiley images are stored
    public string $uploadDir;

    // Database file path for smileys metadata
    private string $databaseFilePath;

    // The path for smiley image uploads relative to the content directory
    public const IMAGE_UPLOAD_DIR = '/smileys/';

    // Error messages for various operations
    public const ERROR_MESSAGES = [
        'entry_not_found' => 'The specified smiley could not be found.',
        'delete_failed' => 'Failed to delete the specified smiley.',
        'metadata_not_found' => 'Metadata file not found.',
        'upload_failed' => 'Upload failed: %s',
        'duplicate_code' => 'A smiley with this code already exists. Please choose a different smiley code.',
        'invalid_characters' => 'Smiley code can only contain letters, colons, and underscores.',
    ];

    // Success messages for various operations
    public const SUCCESS_MESSAGES = [
        'entry_created' => 'Smiley uploaded successfully.',
        'entry_deleted' => 'Smiley deleted successfully.',
        'entry_updated' => 'Smiley updated successfully.'
    ];

    /**
     * Constructor to initialize the upload directory and database file path.
     */
    public function __construct()
    {
        $this->uploadDir = CONTENT_DIR . self::IMAGE_UPLOAD_DIR;
        $this->databaseFilePath = $this->uploadDir . 'database.json';

        // Ensure required directories exist
        ensureRequiredDirectories([$this->uploadDir]);

        // Unique Module Icon for Smileys (this will be used in the dashboard)
        ModuleLoader::setDefaultSettings(
            'Smileys',
            'moduleIcon',
            '', // No value here, it's just for the icon
            '', // No label here
            '', // No description
            'string', // Just a placeholder for type
            'fa-newspaper' // FontAwesome icon for the module
        );

        // Setting to turn Smileys on or off
        ModuleLoader::setDefaultSettings(
            'Smileys',
            'smileys',
            true,
            'Smileys',
            'Turn smileys on or off.',
            'bool', // The value is a boolean (checkbox input)
            'fa-smile' // FontAwesome icon for the setting 
        );

        // Perform cleanup of database entries
        $this->cleanUp();
    }

    /**
     * Cleans up the database by removing entries for missing images.
     */
    private function cleanUp(): void
    {
        // Check if the database file exists
        if (!file_exists($this->databaseFilePath)) {
            return; // No database to clean
        }

        // Read and decode the metadata JSON file
        $data = file_get_contents($this->databaseFilePath);
        $metadata = json_decode($data, true);

        if (!is_array($metadata)) {
            return; // Invalid JSON structure
        }

        // Filter out entries for images that do not exist
        $metadata = array_filter($metadata, function ($entry) {
            $filePath = $this->uploadDir . $entry['imageName'];
            return file_exists($filePath); // Keep only if file exists
        });

        // Save the cleaned metadata back to the file
        file_put_contents($this->databaseFilePath, json_encode(array_values($metadata), JSON_PRETTY_PRINT));
    }

    /**
     * Handles the upload of smiley images.
     * 
     * @param array $postData Form data that includes metadata (like smiley code, hashtags, etc.)
     * @return string A message indicating success or failure
     */
    public function createSmiley(array $postData): string
    {
        $fileInputName = 'smileyImage';
        $metadata = [];
    
        // Sanitize metadata like smiley code
        if (isset($postData['smileyCode'])) {
            $smileyCode = trim($postData['smileyCode']);
    
            // Ensure the smiley code starts and ends with a colon
            if ($smileyCode[0] !== ':' && $smileyCode[strlen($smileyCode) - 1] !== ':') {
                $smileyCode = ':' . $smileyCode . ':';
            } elseif ($smileyCode[0] !== ':') {
                $smileyCode = ':' . $smileyCode; // Add the colon at the start
            } elseif ($smileyCode[strlen($smileyCode) - 1] !== ':') {
                $smileyCode = $smileyCode . ':'; // Add the colon at the end
            }
    
            // Check if the smiley code contains only allowed characters (letters, colons, and underscores)
            if (!preg_match('/^[:a-zA-Z0-9_]+$/', $smileyCode)) {
                Message::set('error', self::ERROR_MESSAGES['invalid_characters']);
                return self::ERROR_MESSAGES['duplicate_code'];
            }
    
            // Sanitize the smiley code
            $metadata['smileyCode'] = htmlspecialchars($smileyCode);
    
            // Check for duplicate smiley code
            if ($this->isDuplicateSmileyCode($smileyCode)) {
                Message::set('error', self::ERROR_MESSAGES['duplicate_code']);
                return self::ERROR_MESSAGES['duplicate_code'];
            }
        }
    
        // Set options for file upload
        $prefix = 'smiley_';
        $isMulti = false;
        $isRequired = true;
    
        // Allowed MIME types for smiley images
        $allowedMimeTypes = ['image/jpeg', 'image/png', 'image/gif'];
        $uploader = new SimpleUpload($this->uploadDir, $allowedMimeTypes, $prefix, $isMulti);
    
        // Attempt to upload the smiley image
        $uploadResult = $uploader->upload($fileInputName, $metadata, $isRequired);
    
        return $this->handleUploadResult($uploadResult);
    }    

    /**
     * Checks if a smiley code already exists in the database.
     *
     * @param string $smileyCode The smiley code to check.
     * @return bool True if a duplicate exists, false otherwise.
     */
    private function isDuplicateSmileyCode(string $smileyCode): bool
    {
        // Check if the metadata file exists
        if (!file_exists($this->databaseFilePath)) {
            return false;
        }

        // Read and decode the metadata JSON file
        $data = file_get_contents($this->databaseFilePath);
        $metadata = json_decode($data, true);

        // Check if any entry already has the same smiley code
        foreach ($metadata as $entry) {
            if (isset($entry['metadata']['smileyCode']) && $entry['metadata']['smileyCode'] === $smileyCode) {
                return true; // Duplicate found
            }
        }

        return false; // No duplicate found
    }

    /**
     * Handles the result from the upload process for smileys.
     *
     * @param mixed $result The result of the upload operation.
     * @return string A message indicating success or failure.
     */
    private function handleUploadResult($result): string
    {
        if (is_array($result)) {
            $errors = array_filter($result, fn($msg) => strpos($msg, 'success') === false);
            if (!empty($errors)) {
                Message::set('error', implode('; ', $errors));
                return sprintf(self::ERROR_MESSAGES['upload_failed'], implode('<br>', $errors));
            }

            Message::set('success', implode('; ', $result));
            return implode(', ', $result);
        }

        if (strpos($result, 'success') !== false) {
            Message::set('success', self::SUCCESS_MESSAGES['entry_created']);
            return self::SUCCESS_MESSAGES['entry_created'];
        }

        Message::set('error', sprintf(self::ERROR_MESSAGES['upload_failed'], $result));
        return sprintf(self::ERROR_MESSAGES['upload_failed'], $result);
    }

    /**
     * Deletes a specified smiley image.
     *
     * @param string $imageName The name of the smiley image to delete.
     * @return string A message indicating the result of the deletion.
     */
    public function deleteSmiley(string $imageName): string
    {
        $filePath = $this->uploadDir . $imageName;

        if (!file_exists($filePath)) {
            Message::set('error', self::ERROR_MESSAGES['entry_not_found']);
            return self::ERROR_MESSAGES['entry_not_found'];
        }

        if (!unlink($filePath)) {
            Message::set('error', self::ERROR_MESSAGES['delete_failed']);
            return self::ERROR_MESSAGES['delete_failed'];
        }

        return $this->removeMetadataEntry($imageName);
    }

    /**
     * Removes the metadata entry for a deleted smiley.
     *
     * @param string $imageName The name of the smiley image.
     * @return string A message indicating the result of the metadata removal.
     */
    private function removeMetadataEntry(string $imageName): string
    {
        if (!file_exists($this->databaseFilePath)) {
            Message::set('error', self::ERROR_MESSAGES['metadata_not_found']);
            return self::ERROR_MESSAGES['metadata_not_found'];
        }

        $data = file_get_contents($this->databaseFilePath);
        $metadata = json_decode($data, true);
        $metadata = array_filter($metadata, fn($entry) => $entry['imageName'] !== $imageName);
        file_put_contents($this->databaseFilePath, json_encode(array_values($metadata), JSON_PRETTY_PRINT));

        Message::set('success', self::SUCCESS_MESSAGES['entry_deleted']);
        return self::SUCCESS_MESSAGES['entry_deleted'];
    }

    /**
     * Updates the smiley code for an existing smiley image.
     *
     * @param string $imageName The name of the smiley image to update.
     * @param string $newSmileyCode The new smiley code to be set.
     * @return string A message indicating the result of the update.
     */
    public function updateSmiley(string $imageName, string $newSmileyCode): string
    {
        if (!file_exists($this->databaseFilePath)) {
            Message::set('error', self::ERROR_MESSAGES['metadata_not_found']);
            return self::ERROR_MESSAGES['metadata_not_found'];
        }
    
        // Read the metadata from the file
        $data = file_get_contents($this->databaseFilePath);
        $metadata = json_decode($data, true);
        $found = false;
        $oldSmileyCode = '';
    
        // Iterate through the metadata to find the smiley entry
        foreach ($metadata as &$entry) {
            if ($entry['imageName'] === $imageName) {
                $oldSmileyCode = $entry['metadata']['smileyCode'];
                $found = true;
                break;
            }
        }
    
        if (!$found) {
            Message::set('error', self::ERROR_MESSAGES['entry_not_found']);
            return self::ERROR_MESSAGES['entry_not_found'];
        }
    
        // Ensure the new smiley code starts and ends with a colon
        if ($newSmileyCode[0] !== ':' && $newSmileyCode[strlen($newSmileyCode) - 1] !== ':') {
            $newSmileyCode = ':' . $newSmileyCode . ':'; // Wrap with colons if missing
        } elseif ($newSmileyCode[0] !== ':') {
            $newSmileyCode = ':' . $newSmileyCode; // Add the colon at the start
        } elseif ($newSmileyCode[strlen($newSmileyCode) - 1] !== ':') {
            $newSmileyCode = $newSmileyCode . ':'; // Add the colon at the end
        }    
    
        // Validate the new smiley code: only letters, colons, and underscores are allowed
        if (!preg_match('/^[:a-zA-Z0-9_]+$/', $newSmileyCode)) {
            Message::set('error', self::ERROR_MESSAGES['invalid_characters']);
            return self::ERROR_MESSAGES['invalid_characters'];
        }
    
        // Sanitize the new smiley code
        $newSmileyCode = htmlspecialchars($newSmileyCode);
    
        // If the smiley code has changed, check for duplicate
        if ($newSmileyCode !== $oldSmileyCode && $this->isDuplicateSmileyCode($newSmileyCode)) {
            Message::set('error', self::ERROR_MESSAGES['duplicate_code']);
            return self::ERROR_MESSAGES['duplicate_code'];
        }
    
        // Update the smiley code
        foreach ($metadata as &$entry) {
            if ($entry['imageName'] === $imageName) {
                $entry['metadata']['smileyCode'] = $newSmileyCode;
                break;
            }
        }
    
        // Save the updated metadata back to the file
        file_put_contents($this->databaseFilePath, json_encode($metadata, JSON_PRETTY_PRINT));
    
        Message::set('success', self::SUCCESS_MESSAGES['entry_updated']);
        return self::SUCCESS_MESSAGES['entry_updated'];
    }    
    
    /**
     * Retrieves all smiley images from the database.
     *
     * @return array An array of metadata entries for all uploaded smileys, or an empty array if smileys are disabled.
     */
    public function getAllSmileys(): array
    {
        // Check if the 'smileys' setting is enabled, default to true
        $smileysEnabled = ModuleLoader::getSetting('Smileys', 'smileys') ?? true;

        if (!$smileysEnabled) {
            // Return an empty array if smileys are disabled
            return [];
        }

        // Proceed with retrieving smileys if they are enabled
        if (!file_exists($this->databaseFilePath)) {
            return [];
        }

        $data = file_get_contents($this->databaseFilePath);
        $metadata = json_decode($data, true);

        if (!is_array($metadata)) {
            return [];
        }

        // Ensure each entry has a smiley code
        foreach ($metadata as &$entry) {
            if (!isset($entry['metadata']['smileyCode'])) {
                $entry['metadata']['smileyCode'] = 'N/A';
            }
        }

        return $metadata;
    }
}