<?php
// Protects the module files from being accessed directly
if (!defined('ALLOW_DIRECT_ACCESS')) {
    die("Access denied.");
}

class Updater
{
    // Current version
    const VERSION = "1.0.3";

    // Shows the current version of the module
    public function getVersion()
    {
        return self::VERSION;
    }

    // Base URL for the update zip files
    private $updateBaseUrl;

    private $current_version;
    private $json_url;

    // Constructor with default version and update URL
    public function __construct($json_url = 'https://designfreaks.net/simplify_updater/latest.json', $updateBaseUrl = 'https://designfreaks.net/simplify_updater/')
    {
        // Check if SimpleLogin class exists and has the getVersion method
        if (class_exists('SimpleLogin') && method_exists('SimpleLogin', 'getVersion')) {
            $core = new SimpleLogin();
            $this->current_version = $core->getVersion();
        } else {
            // Fallback for old versions of SimpleLogin
            $this->current_version = '1.0.0';
        }

        $this->json_url = $json_url; // Default URL for checking updates
        $this->updateBaseUrl = $updateBaseUrl; // Base URL for downloading zip files
    }

    /**
     * Fetch the remote version from the JSON file.
     *
     * @return array|null Returns the parsed JSON data or null if there is an error.
     */
    public function fetchRemoteVersion()
    {
        $context = stream_context_create([
            'http' => [
                'method' => 'GET',
                'timeout' => 10, // Timeout in seconds
            ]
        ]);

        $response = @file_get_contents($this->json_url, false, $context);

        if ($response === false) {
            return null;
        }

        $data = json_decode($response, true);

        if (isset($data['version'])) {
            return $data;
        }

        return null;
    }

    /**
     * Compare versions and check for updates.
     *
     * @param string $localVersion  The local version of the script/module
     * @param string $remoteVersion The remote version to compare against
     * @return string A message indicating if an update is available
     */
    public function checkForUpdate($localVersion, $remoteVersion)
    {
        if (version_compare($localVersion, $remoteVersion, '<')) {
            return "An update is available! Current version: {$localVersion}, Update version: {$remoteVersion}";
        }
        return "You are running the latest version.";
    }

    /**
     * Sanitize a string to ensure it's safe for file and URL usage.
     *
     * @param string $input The input to sanitize.
     * @return string The sanitized string.
     */
    private function sanitizeString($input)
    {
        return preg_replace('/[^a-zA-Z0-9_\-]/', '', $input);
    }

    /**
     * Check for updates in all modules located in the given directory.
     *
     * @param string $modulesDir The directory where modules are located
     * @param array|null $manualJsonData Manually provided JSON data, if any
     * @return array A list of update status messages
     */
    public function checkForUpdatesInModules($modulesDir, $manualJsonData = null)
    {
        $updateMessages = [];
        $updatesAvailable = [];
        $upToDate = [];
    
        // Use the manual JSON data if provided, otherwise fetch from the URL
        $remoteData = $manualJsonData ?? $this->fetchRemoteVersion();
    
        // Return if no data was found
        if ($remoteData === null) {
            return [["error" => "Could not fetch the latest update data. Please check the URL."]];
        }
    
        // Check if the remote data is in the correct JSON format
        if (!$this->validateJsonFormat($remoteData)) {
            return [["error" => "Invalid JSON format."]]; 
        }

        // Check for updates for the main script
        $mainScriptUpdateMessage = $this->checkForUpdate($this->current_version, $remoteData['version']);
        if (version_compare($this->current_version, $remoteData['version'], '<')) {
            $updatesAvailable[] = [
                'title' => 'Loginbase', // Main script title
                'currentVersion' => $this->current_version,
                'updateVersion' => $remoteData['version'],
                'message' => $mainScriptUpdateMessage,
                'downloadLink' => "{$this->updateBaseUrl}{$this->sanitizeString('Loginbase')}_{$remoteData['version']}.zip"
            ];
        } else {
            $upToDate[] = [
                'title' => 'Loginbase',
                'currentVersion' => $this->current_version,
                'message' => $mainScriptUpdateMessage
            ];
        }
    
        // Loop through each directory inside the modules directory
        foreach (scandir($modulesDir) as $moduleFolder) {
            if ($moduleFolder === '.' || $moduleFolder === '..' || !is_dir($modulesDir . '/' . $moduleFolder)) {
                continue;
            }
    
            $moduleFile = $modulesDir . '/' . $moduleFolder . '/' . $moduleFolder . '.php';
    
            if (file_exists($moduleFile)) {
                include_once $moduleFile;
    
                if (class_exists($moduleFolder)) {
                    $module = new $moduleFolder();
    
                    // Check if the module has the getVersion() method
                    if (method_exists($module, 'getVersion')) {
                        $currentVersion = $module->getVersion();
                    } else {
                        // If the method doesn't exist, set current version to '1.0.0'
                        $currentVersion = '1.0.0';
                    }
    
                    // Check if the module has a corresponding remote version
                    if (isset($remoteData[$moduleFolder])) {
                        $remoteModuleVersion = $remoteData[$moduleFolder];
                        $updateMessage = $this->checkForUpdate($currentVersion, $remoteModuleVersion);
    
                        if (version_compare($currentVersion, $remoteModuleVersion, '<')) {
                            $updatesAvailable[] = [
                                'title' => $moduleFolder,
                                'currentVersion' => $currentVersion,
                                'updateVersion' => $remoteModuleVersion,
                                'message' => $updateMessage,
                                'downloadLink' => "{$this->updateBaseUrl}{$this->sanitizeString($moduleFolder)}_{$remoteModuleVersion}.zip"
                            ];
                        } else {
                            $upToDate[] = [
                                'title' => $moduleFolder,
                                'currentVersion' => $currentVersion,
                                'message' => $updateMessage
                            ];
                        }
                    } else {
                        $upToDate[] = [
                            'title' => $moduleFolder,
                            'currentVersion' => $currentVersion,
                            'message' => 'No remote version found.'
                        ];
                    }
                } else {
                    $updatesAvailable[] = [
                        'title' => $moduleFolder,
                        'message' => 'Module class not found.'
                    ];
                }
            } else {
                $updatesAvailable[] = [
                    'title' => $moduleFolder,
                    'message' => 'Module file not found.'
                ];
            }
        }
    
        return [
            'updatesAvailable' => $updatesAvailable,
            'upToDate' => $upToDate
        ];
    }

    /**
     * Fix the JSON input to ensure it's properly formatted.
     *
     * @param string $input The raw JSON input.
     * @return string The fixed JSON input.
     */
    public function fixJsonInput($input)
    {
        // Trim the input to remove extra spaces or newlines at the beginning or end
        $input = trim($input);
    
        // Ensure the input starts and ends with curly braces (if missing)
        if (substr($input, 0, 1) !== '{' || substr($input, -1) !== '}') {
            $input = '{' . $input . '}';
        }
    
        // Ensure keys and values are properly quoted
        // This regex will look for "key": "value" style patterns and ensure proper quoting and spacing
        $input = preg_replace_callback('/(\w+)\s*:\s*([^,}]+)/', function($matches) {
            // Remove extra spaces around the value and key, then wrap them in quotes
            return '"' . $matches[1] . '":"' . trim($matches[2]) . '"';
        }, $input);
    
        return $input;
    }    

    /**
     * Validate if the JSON structure matches the required format.
     * It ensures the data is an object where all values are strings.
     *
     * @param mixed $data
     * @return bool
     */
    private function validateJsonFormat($data)
    {
        // Ensure data is an array (or object) and check that all values are strings
        if (!is_array($data) || empty($data)) {
            return false; // Not a valid JSON object
        }

        foreach ($data as $key => $value) {
            // Check that each value is a string
            if (!is_string($value)) {
                return false; // Value is not a string
            }
        }

        return true; // Valid JSON format
    }
}
