<?php
// Load necessary classes and functions
require_once __DIR__ . '/simplify/autoload.php';
require_once __DIR__ . '/simplify/functions.php';
require_once __DIR__ . '/simplify/config.php';

$auth = new SimpleLogin();

// Retrieve user ID from URL parameter
$userId = isset($_GET['id']) ? (int) $_GET['id'] : 0; // Default to 0 if not set

if ($userId > 0) {
    // If an ID is set, fetch the specific user by ID
    $userData = $auth->getUserById($userId);

    // Check if user data was found
    if (!$userData) {
        die($auth::ERROR_MESSAGES['user_not_found']);
    }
} else {
    // If no ID is set, fetch all users
    $userData = $auth->getAllUsers();
}

// Load modules and register hooks
$modules = ModuleLoader::loadModules();

// Retrieve any messages to display to the user
$messages = Message::get();

// Fetch the current avatar URL
$avatarUrl = $auth->getUserProfilePicture($userId);

// Settings for birthday and age visibility
$showBirthday = ModuleLoader::getSetting('Default', 'showBirthdayInProfiles');
$showAge = ModuleLoader::getSetting('Default', 'showAgeInProfiles');

// Initialize variables for birthday and age display
$birthdayDisplay = '';
$ageDisplay = '';

if ($userId > 0 && !empty($userData['birthday'])) {
    // Get the formatted birthdate
    $formattedBirthday = sanitizeInput(formatDate($userData['birthday'], ModuleLoader::getSetting('Default', 'birthdayDateFormat')));

    // Calculate the age using the birthday
    $age = calculateAge($userData['birthday']);

    // Set the birthday and age display based on settings
    if ($showBirthday) {
        $birthdayDisplay = $formattedBirthday;
    }

    if ($showAge) {
        if ($birthdayDisplay) {
            // If birthday is shown, append the age
            $birthdayDisplay .= " (" . $age . " years old)";
        } else {
            // If only age is shown, just display the age
            $ageDisplay = $age . " years old";
        }
    }
}

// Include header file for the page layout
include_once __DIR__ . '/header.php';
?>

<!-- Display any messages -->
<?php if (!empty($messages)): ?>
    <?php foreach ($messages as $msg): ?>
        <p class="<?= sanitizeInput($msg['type']); ?>"><?= sanitizeInput($msg['text']); ?></p>
    <?php endforeach; ?>
<?php endif; ?>

<?php if ($userId > 0): ?>
    <h1><?= sanitizeInput($userData['username']); ?>'s Profile</h1>
    <div id="simplify-profile">
        <!-- Personal Information Section -->
        <div class="profile-personal">
            <div class="profile-avatar">
                <?php if (!empty($userData['avatar'])): ?>
                    <div class="profile-item">
                        <img src="<?= $avatarUrl; ?>" alt="Avatar">
                    </div>
                <?php endif; ?>
            </div>

            <div class="profile-info">
                <?php if (!empty($userData['username'])): ?>
                    <div class="profile-item">
                        <div class="label">Username:</div>
                        <div class="profile-value"><?= sanitizeInput($userData['username']); ?></div>
                    </div>
                <?php endif; ?>

                <?php if (!empty($birthdayDisplay)): ?>
                    <div class="profile-item">
                        <div class="label">Birthday:</div>
                        <div class="profile-value"><?= $birthdayDisplay; ?></div>
                    </div>
                <?php elseif (!empty($ageDisplay)): ?>
                    <!-- If birthday is hidden but age is shown -->
                    <div class="profile-item">
                        <div class="label">Age:</div>
                        <div class="profile-value"><?= $ageDisplay; ?></div>
                    </div>
                <?php endif; ?>

                <?php if (!empty($userData['role'])): ?>
                    <div class="profile-item">
                        <div class="label">Role:</div>
                        <div class="profile-value"><?= sanitizeInput($auth::getRoleName($userData['role'])); ?></div>
                    </div>
                <?php endif; ?>
            </div>

            <!-- Social Media Links Section -->
            <div class="profile-socialmedia">
                <?php if (!empty($userData['facebook'])): ?>
                    <a href="<?= formatUrl($userData['facebook']); ?>" target="_blank" class="social-link">
                        <i class="fa-brands fa-facebook"></i>
                    </a>
                <?php endif; ?>

                <?php if (!empty($userData['bluesky'])): ?>
                    <a href="<?= formatUrl($userData['bluesky']); ?>" target="_blank" class="social-link">
                        <i class="fa-brands fa-bluesky"></i>
                    </a>
                <?php endif; ?>

                <?php if (!empty($userData['x'])): ?>
                    <a href="<?= formatUrl($userData['x']); ?>" target="_blank" class="social-link">
                        <i class="fa-brands fa-x"></i>
                    </a>
                <?php endif; ?>

                <?php if (!empty($userData['tiktok'])): ?>
                    <a href="<?= formatUrl($userData['tiktok']); ?>" target="_blank" class="social-link">
                        <i class="fa-brands fa-tiktok"></i>
                    </a>
                <?php endif; ?>

                <?php if (!empty($userData['reddit'])): ?>
                    <a href="<?= formatUrl($userData['reddit']); ?>" target="_blank" class="social-link">
                        <i class="fa-brands fa-reddit"></i>
                    </a>
                <?php endif; ?>

                <?php if (!empty($userData['instagram'])): ?>
                    <a href="<?= formatUrl($userData['instagram']); ?>" target="_blank" class="social-link">
                        <i class="fa-brands fa-instagram"></i>
                    </a>
                <?php endif; ?>

                <?php if (!empty($userData['deviantart'])): ?>
                    <a href="<?= formatUrl($userData['deviantart']); ?>" target="_blank" class="social-link">
                        <i class="fa-brands fa-deviantart"></i>
                    </a>
                <?php endif; ?>

                <?php if (!empty($userData['discord'])): ?>
                    <div class="social-link">
                        <i class="fa-brands fa-discord"></i>
                        <?= sanitizeInput($userData['discord']); ?>
                    </div>
                <?php endif; ?>
            </div>

            <?php if (!empty($userData['bio'])): ?>
                <div class="profile-item bio">
                    <div class="profile-value"><?= sanitizeHtmlInput($userData['bio']); ?></div>
                </div>
            <?php endif; ?>
        </div>

        <!-- Likes and Dislikes Section -->
        <?php if (!empty($userData['likes']) || !empty($userData['dislikes'])): ?>
            <div class="profile-likesdislikes">
                <?php if (!empty($userData['likes'])): ?>
                    <div class="profile-likes">
                        <h3>
                            <i class="fa fa-thumbs-up"></i> Likes
                        </h3>
                        <div class="profile-value"><?= sanitizeHtmlInput($userData['likes']); ?></div>
                    </div>
                <?php endif; ?>

                <?php if (!empty($userData['dislikes'])): ?>
                    <div class="profile-dislikes">
                        <h3>
                            <i class="fa fa-thumbs-down"></i> Dislikes
                        </h3>
                        <div class="profile-value"><?= sanitizeHtmlInput($userData['dislikes']); ?></div>
                    </div>
                <?php endif; ?>
            </div>
        <?php endif; ?>
    </div>
<?php else: ?>
    <h1>Members</h1>
    <div id="user-list">
        <?php foreach ($userData as $userId => $user): ?>
            <div class="user-item">
                <?php if (!empty($user['avatar'])): ?>
                    <div class="user-image">
                        <img src="<?= $auth->getUserProfilePicture($userId); ?>" alt="Avatar">
                    </div>
                <?php endif; ?>

                <div class="user-name">
                    <?= sanitizeInput($user['username']); ?>
                </div>

                <div class="user-link">
                    <a href="profile.php?id=<?= sanitizeInput($userId); ?>">View Profile</a>
                </div>
            </div>
        <?php endforeach; ?>
    </div>
<?php endif; ?>

<!-- Include footer file for the page layout -->
<?php include_once __DIR__ . '/footer.php'; ?>