<?php
 /**
 * Autoloads class files based on the class name.
 *
 * This function sanitizes the class name to prevent directory traversal attacks
 * and attempts to load the class from predefined class and module directories.
 *
 * @param string $class The fully qualified class name to load.
 * @return void
 */
spl_autoload_register(function ($class) {
    // Sanitize the class name to prevent directory traversal attacks
    $sanitizedClass = preg_replace('/[^a-zA-Z0-9_\\\\]/', '', $class);
    
    // Define the base directories for class and module files
    $baseDirClass = CLASSES_DIR . DIRECTORY_SEPARATOR; // Class directory path
    $baseDirModule = MODULES_DIR . DIRECTORY_SEPARATOR; // Module directory path

    // Attempt to load from the class directory
    $fileClass = $baseDirClass . $sanitizedClass . '.php';
    if (isReadable($fileClass)) {
        require_once $fileClass;
        return;
    }

    // Attempt to load from the modules directory
    $fileModule = $baseDirModule . $sanitizedClass . '/' . $sanitizedClass . '.php';
    if (isReadable($fileModule)) {
        require_once $fileModule;
    }
});

/**
 * Check if the file is readable to ensure it can be included safely.
 *
 * @param string $filePath The path of the file to check.
 * @return bool True if the file is readable, false otherwise.
 */
function isReadable($filePath) {
    return file_exists($filePath) && is_readable($filePath);
}
