<?php
/**
 * AliasGenerator class
 * 
 * Generates URL-friendly aliases from given titles by replacing special characters, 
 * converting text to lowercase, and limiting the length to a maximum of 60 characters. 
 *
 * Main functionality:
 * - Replaces special characters with URL-safe equivalents (e.g., "ä" becomes "ae")
 * - Converts text to lowercase for uniformity
 * - Removes unwanted characters and replaces spaces with dashes
 * - Truncates the alias to a maximum of 60 characters, preserving whole words
 *
 * Usage example:
 * ```
 * $title = "Lorem ipsum dolor sit amet, consectetur adipiscing elit";
 * $alias = AliasGenerator::create($title);
 * echo $alias; // Outputs: "lorem-ipsum-dolor-sit-amet-consectetur-adipiscing"
 * ```
 */

class AliasGenerator {
    /**
     * Generate a URL-friendly alias from the given title.
     *
     * This method processes the title by replacing special characters, converting to lowercase, 
     * and removing unwanted symbols. It also ensures that the final alias is no longer than 
     * 60 characters, preserving whole words to avoid awkward truncations.
     *
     * @param string $title The input title to generate an alias from.
     * @return string The generated URL-friendly alias, limited to 60 characters.
     */
    public static function create($title) {
        // Map special characters to their replacements for URL safety
        $replacements = [
            // German umlauts
            'ä' => 'ae', 'ö' => 'oe', 'ü' => 'ue', 'ß' => 'ss',
            'Ä' => 'Ae', 'Ö' => 'Oe', 'Ü' => 'Ue',
            // French characters
            'é' => 'e', 'è' => 'e', 'ê' => 'e', 'ë' => 'e',
            'ô' => 'o', 'ç' => 'c',
            'É' => 'E', 'È' => 'E', 'Ê' => 'E', 'Ë' => 'E',
            'Ô' => 'O', 'Ç' => 'C',
            // Spanish characters
            'ñ' => 'n', 'Ñ' => 'N',
            // Portuguese characters
            'ã' => 'a', 'â' => 'a', 'à' => 'a', 'á' => 'a',
            'õ' => 'o', 'ó' => 'o',
            // Additional characters
            'ø' => 'o', 'Ø' => 'O', 'ł' => 'l', 'Ł' => 'L', 
            'ś' => 's', 'Ś' => 'S', 'č' => 'c', 'Č' => 'C',
            'ž' => 'z', 'Ž' => 'Z', 'ń' => 'n', 'Ń' => 'N',
            'î' => 'i', 'û' => 'u',
        ];
        
        // Replace special characters with their mapped values
        $title = strtr($title, $replacements);
        
        // Convert the entire title to lowercase to ensure uniformity
        $title = strtolower($title);

        // Remove unwanted characters (non-alphanumeric and non-space/dash) and convert spaces to dashes
        $title = preg_replace('/[^a-z0-9\s-]/', '', $title); // Remove unwanted characters
        $title = preg_replace('/[\s]+/', '-', trim($title)); // Replace one or more spaces with a single dash

        // Limit the length of the alias to 60 characters
        if (strlen($title) > 60) {
            // Truncate to the last full word that fits within the 60 character limit
            $words = explode('-', $title); // Split the title into words
            $new_title = ''; // Initialize the new title

            foreach ($words as $word) {
                // Add words to the new title until it reaches the length limit
                if (strlen($new_title . ($new_title ? '-' : '') . $word) <= 60) {
                    $new_title .= ($new_title ? '-' : '') . $word; // Append word with a dash if necessary
                } else {
                    break; // Stop adding words if the next word would exceed the length limit
                }
            }
            $title = $new_title; // Update the title to the truncated version
        }

        return $title; // Return the final generated alias
    }
}
