<?php
/**
 * AssetsManager class
 *
 * Manages inclusion of CSS and JavaScript. Allows dynamic registration of assets
 * for specific areas (header and footer) while ensuring no duplicate entries are added.
 *
 */
class AssetsManager
{
    // Stores registered assets (CSS, JS) organized by area ('header' or 'footer')
    private static $assets = [
        'header' => [],
        'footer' => [],
    ];

    /**
     * Add a CSS file (local or external) to the specified area (header or footer).
     * 
     * @param string $file The path to the CSS file or external URL.
     * @param string $area The area where to add the asset (header or footer).
     */
    public static function addCss($file, $area = 'header')
    {
        // Check if the file is a URL (external resource)
        if (filter_var($file, FILTER_VALIDATE_URL)) {
            // For external resources (URL), ensure the file is not already added
            if (!in_array($file, self::$assets[$area])) {
                self::$assets[$area][] = $file;
            }
        } else {
            // For local resources, normalize the path and check for duplicates
            $relativePath = trim($file, '/');
            if (!in_array($relativePath, self::$assets[$area])) {
                self::$assets[$area][] = $relativePath;
            }
        }
    }

    /**
     * Add a JavaScript file to the specified area (header or footer).
     *
     * Registers the provided JavaScript file path in the designated area.
     * Ensures that duplicate file paths are not added to the list of assets.
     *
     * @param string $file The relative path to the JavaScript file.
     * @param string $area The area where to add the asset (header or footer).
     */
    public static function addJs($file, $area = 'footer')
    {
        $relativePath = trim($file, '/');
        if (!in_array($relativePath, self::$assets[$area])) {
            self::$assets[$area][] = $relativePath;
        }
    }

    /**
     * Render the registered assets (CSS, JS) into HTML format for the specified area.
     *
     * @param string $area The area for which to render assets (header or footer).
     * @return string The HTML string of rendered assets tags.
     */
    public static function renderAssets($area = 'header')
    {
        $output = '';
        $modulesDirName = basename(MODULES_DIR);

        // Render CSS and JS assets
        foreach (self::$assets[$area] as $asset) {
            // If the asset is a URL, output as <link> tag
            if (filter_var($asset, FILTER_VALIDATE_URL)) {
                $output .= "<link rel='stylesheet' href='" . htmlspecialchars($asset, ENT_QUOTES) . "'>\n";
            } else {
                // If the asset is a local file, construct the URL based on BASE_URL and MODULES_DIR
                $assetUrl = rtrim(BASE_URL, '/') . '/' . $modulesDirName . '/' . ltrim($asset, '/');
                $output .= "<link rel='stylesheet' href='" . htmlspecialchars($assetUrl, ENT_QUOTES) . "'>\n";
            }
        }

        // Render JS in the footer
        foreach (self::$assets['footer'] as $asset) {
            $assetUrl = rtrim(BASE_URL, '/') . '/' . $modulesDirName . '/' . ltrim($asset, '/');
            $output .= "<script src='" . htmlspecialchars($assetUrl, ENT_QUOTES) . "'></script>\n";
        }

        return $output;
    }
}
