<?php
/**
 * HookManager class
 *
 * This file defines the HookManager class, which is responsible for managing 
 * and executing hooks. Hooks allow other parts of the application to register 
 * callbacks for specific events, enabling modularity and extensibility.
 *
 * Key functionality:
 * - Registering callbacks to specific hooks using `addHook`.
 * - Executing registered callbacks for a given hook using `executeHook`.
 *
 * Usage example:
 * ```
 * // Define a callback function to generate a "Manage Users" link for the dashboard
 * function addManageUsersLink() {
 *     return "<a href='/admin/manage-users' class='dashboard-link'>Manage Users</a>";
 * }
 * 
 * // Register the callback to the "dashboard_links" hook
 * HookManager::addHook('dashboard_links', 'addManageUsersLink');
 *
 * // Execute all registered callbacks for the "dashboard_links" hook
 * $dashboardLinks = HookManager::executeHook('dashboard_links');
 * 
 * // Display the generated links in the dashboard
 * foreach ($dashboardLinks as $link) {
 *     echo $link;
 * }
 *
 * // Output:
 * // <a href='/admin/manage-users' class='dashboard-link'>Manage Users</a>
 * ```
 */

class HookManager
{
    /**
     * An associative array to store hooks and their associated callbacks and orders.
     *
     * @var array
     */
    private static array $hooks = [];

    /**
     * Register a callback function to a specific hook with an optional order.
     *
     * @param string $hook The name of the hook to which the callback is being registered.
     * @param callable $callback The callback function to execute when the hook is triggered.
     * @param int $order The order to display the link, default is 0.
     * @throws InvalidArgumentException If the hook name is not valid.
     */
    public static function addHook(string $hook, callable $callback, int $order = 0): void
    {
        if (empty($hook) || !is_string($hook)) {
            throw new InvalidArgumentException("Hook name must be a non-empty string.");
        }

        if (!is_callable($callback)) {
            throw new InvalidArgumentException("The provided callback is not callable.");
        }

        // Initialize hook array if it does not exist
        if (!isset(self::$hooks[$hook])) {
            self::$hooks[$hook] = [];
        }

        // Register the callback with its order
        self::$hooks[$hook][] = ['callback' => $callback, 'order' => $order];
    }

    /**
     * Execute all callbacks registered to a specific hook, sorted by order.
     *
     * @param string $hook The name of the hook to execute.
     * @return array The output from each callback executed.
     * @throws InvalidArgumentException If the hook name is not valid.
     */
    public static function executeHook(string $hook): array
    {
        if (empty($hook) || !is_string($hook)) {
            throw new InvalidArgumentException("Hook name must be a non-empty string.");
        }

        if (!isset(self::$hooks[$hook])) {
            return []; // Return an empty array if no hooks are found
        }

        // Sort the hooks by their order before executing
        usort(self::$hooks[$hook], function ($a, $b) {
            return $a['order'] <=> $b['order'];
        });

        $output = [];
        foreach (self::$hooks[$hook] as $entry) {
            if (is_callable($entry['callback'])) {
                $output[] = $entry['callback'](); // Collect the output
            }
        }
        return $output; // Return the collected output
    }
}