<?php
/**
 * Message class
 *
 * Provides a simple interface for managing and displaying session-based messages, 
 * allowing messages to be set, retrieved, and cleared across different pages or components.
 *
 * Key functionality:
 * - Setting messages with a specified type and text using `set`.
 * - Retrieving all messages or filtering by type with `get`.
 * - Clearing all messages using `clear`.
 *
 * Session management is handled internally to ensure messages are stored across user interactions.
 * 
 * Usage example:
 * ```
 * // Set a success message
 * Message::set('success', 'Your profile was updated successfully.');
 * 
 * // Set an error message
 * Message::set('error', 'Unable to update profile. Please try again.');
 *
 * // Retrieve and display all messages
 * foreach (Message::get() as $message) {
 *     echo "<div class='message {$message['type']}'>{$message['text']}</div>";
 * }
 * ```
 */

class Message
{
    /**
     * Starts the session if it hasn't been started yet.
     *
     * @return void
     */
    private static function startSession()
    {
        if (session_status() == PHP_SESSION_NONE) {
            session_start();
        }
    }

    /**
     * Sets a message in the session.
     *
     * @param string $type The type of message (e.g., 'success', 'error').
     * @param string $text The message content.
     * @return void
     */
    public static function set($type, $text)
    {
        self::startSession();

        // Initialize the messages array if not already set
        if (!isset($_SESSION['messages'])) {
            $_SESSION['messages'] = [];
        }

        // Check if the message already exists in the session
        foreach ($_SESSION['messages'] as $message) {
            if ($message['type'] === $type && $message['text'] === $text) {
                // If the same message exists, do nothing (no duplicate)
                return;
            }
        }

        // If no duplicate, add the new message
        $_SESSION['messages'][] = ['type' => $type, 'text' => $text];
    }

    /**
     * Retrieves messages from the session.
     * If a type is specified, only messages of that type are returned.
     * Clears messages from the session after retrieval.
     *
     * @param string|null $type Optional. The type of messages to retrieve.
     *                           If null, retrieves all messages.
     * @return array The retrieved messages, filtered by type if specified.
     */
    public static function get($type = null)
    {
        self::startSession();

        // Retrieve messages from the session
        $messages = $_SESSION['messages'] ?? [];

        // Filter messages by type if specified
        if ($type) {
            $messages = array_filter($messages, function ($msg) use ($type) {
                return $msg['type'] === $type;
            });
        }

        // Clear messages after they have been retrieved
        unset($_SESSION['messages']);

        return $messages; // Return all messages if no type is specified
    }

    /**
     * Clears all messages from the session.
     *
     * @return void
     */
    public static function clear()
    {
        self::startSession();
        unset($_SESSION['messages']);
    }
}
