<?php
/**
 * ModuleLoader class
 *
 * Handles loading, sorting, and permission management of application modules. 
 * Loads configuration files for modules, verifies access permissions, 
 * and registers module links to hooks based on user roles.
 *
 * Primary functionality:
 * - Loads and organizes modules based on defined order and permissions (`loadModules`).
 * - Registers module links and validates user access roles.
 * - Loads specific module files dynamically, enforcing access permissions (`loadModuleFile`).
 *
 * Usage example:
 * ```
 * // Load and sort modules
 * $modules = ModuleLoader::loadModules();
 * 
 * // Load a specific module file with permission check
 * try {
 *     ModuleLoader::loadModuleFile("blog", "posts.php");
 * } catch (Exception $e) {
 *     echo $e->getMessage(); // Handle access denied or file not found
 * }
 * 
 * // Generate a URL for a module with parameters
 * $url = ModuleLoader::generateUrl("blog", "posts.php", ["id" => 123]);
 * echo $url; // Outputs: "module.php?module=blog&file=posts.php&id=123"
 * 
 * // Ensure required modules are loaded
 * ModuleLoader::requiresModules(["SimpleBlog", "Help"]);
 * ```
 */

class ModuleLoader
{
    private static $settings = []; // Static settings array to store module settings

    /**
     * Load modules and their configurations, sorting by `module_order` and ensuring link order within each module.
     * Ensures the function is not executed more than once per request.
     *
     * @return array Loaded and sorted modules
     */
    public static function loadModules(): array
    {
        static $modules = null;

        if ($modules !== null) {
            return $modules;
        }

        $modulesDir = MODULES_DIR;
        $userRole = $_SESSION['role'] ?? SimpleLogin::getDefaultRoleId();

        foreach (glob($modulesDir . '/*', GLOB_ONLYDIR) as $moduleFolder) {
            $moduleName = basename($moduleFolder);
            $configFile = $moduleFolder . '/' . $moduleName . '.config.php';
            $classFile = $moduleFolder . '/' . $moduleName . '.php';

            // Check if the config file exists
            if (file_exists($configFile)) {
                $config = require $configFile;
            } else {
                // Fallback: Empty configuration or default values
                $config = [
                    'module_order' => 0,
                    'module_name' => $moduleName,
                    'links' => [] // No links if config is absent
                ];
            }

            // Ensure module order and process links
            $config['module_order'] = $config['module_order'] ?? 0;
            $config['module_name'] = $moduleName;

            if (!empty($config['links']) && is_array($config['links'])) {
                usort($config['links'], function ($a, $b) {
                    return ($a['order'] ?? 0) <=> ($b['order'] ?? 0);
                });
            }

            $modules[] = $config;

            // Register assets if possible
            if (method_exists($moduleName, 'registerAssets')) {
                $moduleName::registerAssets();
            }
        }

        // Sort modules by `module_order` and alphabetically by `module_name`
        usort($modules, function ($a, $b) {
            $orderComparison = ($b['module_order'] ?? 0) <=> ($a['module_order'] ?? 0);
            return $orderComparison === 0 ? strcmp($a['module_name'], $b['module_name']) : $orderComparison;
        });

        foreach ($modules as $config) {
            $userRole = $_SESSION['role'] ?? SimpleLogin::getDefaultRoleId();

            if (!empty($config['links']) && is_array($config['links'])) {
                foreach ($config['links'] as $link) {
                    if ($userRole === SimpleLogin::getAdminRoleId()) {
                        if (isset($link['visible']) && $link['visible']) {
                            if (isset($link['hook'])) {
                                HookManager::addHook($link['hook'], function () use ($link) {
                                    $link['url'] = 'module.php?module=' . urlencode($link['module']) . '&file=' . urlencode($link['file']);
                                    $iconHtml = isset($link['icon_class']) ? "<i class='" . htmlspecialchars($link['icon_class'], ENT_QUOTES) . "'></i> " : '';
                                    return "<a href='" . htmlspecialchars($link['url'], ENT_QUOTES) . "' class='" . htmlspecialchars($link['class'] ?? '', ENT_QUOTES) . "'>{$iconHtml}" . htmlspecialchars($link['title'], ENT_QUOTES) . "</a>";
                                });
                            }
                        }
                        continue; // Skip further checks for admin
                    }

                    $allowedRoles = $link['access'] ?? [SimpleLogin::getDefaultRoleId()];
                    if (in_array($userRole, $allowedRoles, true) && isset($link['hook'])) {
                        if (!isset($link['visible']) || $link['visible']) {
                            HookManager::addHook($link['hook'], function () use ($link) {
                                $link['url'] = 'module.php?module=' . urlencode($link['module']) . '&file=' . urlencode($link['file']);
                                $iconHtml = isset($link['icon_class']) ? "<i class='" . htmlspecialchars($link['icon_class'], ENT_QUOTES) . "'></i> " : '';
                                return "<a href='" . htmlspecialchars($link['url'], ENT_QUOTES) . "' class='" . htmlspecialchars($link['class'] ?? '', ENT_QUOTES) . "'>{$iconHtml}" . htmlspecialchars($link['title'], ENT_QUOTES) . "</a>";
                            });
                        }
                    }
                }
            }

            if (isset($config['class_file'])) {
                $classFilePath = $moduleFolder . '/' . $config['class_file'];
                if (file_exists($classFilePath)) {
                    require_once $classFilePath;
                }
            }
        }

        return $modules;
    }

    /**
     * Load persistent settings from a file.
     *
     * @return void
     */
    public static function loadSettings(): void
    {
        $settingsFile = MODULES_DIR . '/settings.json'; // Specify path to settings file

        if (file_exists($settingsFile)) {
            self::$settings = json_decode(file_get_contents($settingsFile), true) ?? [];
        }
    }

    /**
     * Save persistent settings to a file.
     *
     * @return void
     */
    public static function saveSettings(): void
    {
        $settingsFile = MODULES_DIR . '/settings.json'; // Specify path to settings file
        file_put_contents($settingsFile, json_encode(self::$settings, JSON_PRETTY_PRINT));
    }

    /**
     * Get all settings for all modules.
     *
     * @return array The settings for all modules.
     */
    public static function getAllSettings(): array
    {
        // Ensure settings are loaded
        self::loadSettings();
        return self::$settings;
    }

    /**
     * Get a module setting by its key.
     *
     * @param string $module The module name
     * @param string $key The setting key
     * @return mixed|null The setting value or null if not found
     */
    public static function getSetting(string $module, string $key)
    {
        // Ensure settings are loaded
        self::loadSettings();

        // Check if the setting exists and return the value
        return self::$settings[$module][$key]['value'] ?? null;  // Access 'value' inside the setting
    }

    /**
     * Set a module setting by its key.
     *
     * Updates an existing setting or creates a new one for a specified module. Optionally accepts label, description, type, and icon.
     *
     * @param string $module The module name.
     * @param string $key The setting key.
     * @param mixed $value The setting value.
     * @param string|null $label The setting label (optional).
     * @param string|null $description The setting description (optional).
     * @param string $type The input field type (optional, default 'string').
     * @param string|null $icon The FontAwesome icon class (optional).
     * @param string|null $moduleIcon The module icon class (optional).
     */
    public static function setSetting(string $module, string $key, $value, $label = null, $description = null, string $type = 'string', string $icon = '', string $moduleIcon = ''): void
    {
        // Ensure settings are loaded
        self::loadSettings();

        // Initialize the module settings if not already present
        if (!isset(self::$settings[$module])) {
            self::$settings[$module] = [];
        }

        // Check if the key exists and update its value, otherwise create a new entry
        if (isset(self::$settings[$module][$key])) {
            // Update the 'value' of the existing setting
            self::$settings[$module][$key]['value'] = $value;
        } else {
            // Create a new setting entry with 'value', 'label', 'description', 'type', and 'icon'
            self::$settings[$module][$key] = [
                'value' => $value,
                'label' => $label ?? $key,  // Use the key as label by default
                'description' => $description ?? '',  // Use empty string as description by default
                'type' => $type,  // Store the type (e.g., 'int', 'string', 'bool', 'select', 'text')
                'icon' => $icon  // Store the icon for individual settings
            ];
        }

        // If a module icon is provided, store it separately for module-wide use
        if ($moduleIcon) {
            self::$settings[$module]['moduleIcon'] = $moduleIcon;
        }

        // Save the settings after updating
        self::saveSettings();
    }

    /**
     * Set default settings for a module if they don't already exist.
     *
     * Adds default settings if they aren't already defined, without overwriting the existing value,
     * but updating the label, description, and icon if they differ.
     *
     * @param string $module The module name.
     * @param string $key The setting key.
     * @param mixed $value The default value.
     * @param string|null $label The setting label (optional).
     * @param string|null $description The setting description (optional).
     * @param string $type The input field type (optional, default 'string').
     * @param string|null $icon The FontAwesome icon class (optional).
     * @param string|null $moduleIcon The module icon class (optional).
     */
    public static function setDefaultSettings(string $module, string $key, $value, $label = null, $description = null, string $type = 'string', string $icon = '', string $moduleIcon = ''): void
    {
        // Ensure settings are loaded
        self::loadSettings();

        // Initialize the module settings if not already present
        if (!isset(self::$settings[$module])) {
            self::$settings[$module] = [];
        }

        // Check if the setting already exists
        if (isset(self::$settings[$module][$key])) {
            // Update label, description, and icon if they differ from the current ones
            if (($label !== null) && (self::$settings[$module][$key]['label'] !== $label)) {
                self::$settings[$module][$key]['label'] = $label;
            }
            if (($description !== null) && (self::$settings[$module][$key]['description'] !== $description)) {
                self::$settings[$module][$key]['description'] = $description;
            }
            if (($icon !== '') && (self::$settings[$module][$key]['icon'] !== $icon)) {
                self::$settings[$module][$key]['icon'] = $icon;
            }
        } else {
            // If the setting does not exist, create a new entry
            self::$settings[$module][$key] = [
                'value' => $value,
                'label' => $label ?? $key, // Use the key as label by default
                'description' => $description ?? '', // Default to an empty description
                'type' => $type, // Store the type (e.g., 'int', 'string', 'bool', 'select', 'text')
                'icon' => $icon // Store the icon for individual settings
            ];
        }

        // If a module icon is provided, store it separately for module-wide use
        if ($moduleIcon) {
            self::$settings[$module]['moduleIcon'] = $moduleIcon;
        }

        // Save the settings after updating
        self::saveSettings();
    }

    /**
     * Clean up settings by removing settings for non-existent modules.
     *
     * This function checks each module in the settings file and removes the settings
     * for modules that do not have a corresponding folder in the modules directory.
     * It skips the "Default" module, which is a special key that should not be deleted.
     *
     * @return void
     */
    public static function cleanUpSettings(): void
    {
        // Ensure settings are loaded
        self::loadSettings();

        // Define the path to the modules directory
        $modulesDir = MODULES_DIR;

        // Get a list of all module folders in the modules directory
        $existingModules = array_map('basename', glob($modulesDir . '/*', GLOB_ONLYDIR));

        // Iterate through settings to find modules without matching folders
        foreach (self::$settings as $module => $settings) {
            // Skip the "Default" module to avoid deletion
            if ($module === "Default") {
                continue;
            }

            // If the module folder does not exist, remove its settings
            if (!in_array($module, $existingModules, true)) {
                unset(self::$settings[$module]);
            }
        }

        // Save the updated settings to the file
        self::saveSettings();
    }

    /**
     * Load a specific module file, checking permissions based on user role.
     *
     * @param string $moduleName The name of the module
     * @param string $fileName The name of the file to load
     * @throws Exception if access is denied or the file does not exist
     */
    public static function loadModuleFile(string $moduleName, string $fileName): void
    {
        $sanitizedModule = preg_replace('/[^a-zA-Z0-9_]/', '', $moduleName);
        $sanitizedFile = preg_replace('/[^a-zA-Z0-9_.]/', '', $fileName);

        $modulesDir = MODULES_DIR;
        $configFilePath = $modulesDir . '/' . $sanitizedModule . '/' . $sanitizedModule . '.config.php';

        if (file_exists($configFilePath)) {
            $config = require $configFilePath;

            // Verify if the requested file exists in the config links and user has access
            $fileFound = false;
            foreach ($config['links'] as $link) {
                if ($link['file'] === $sanitizedFile) {
                    $fileFound = true;
                    $allowedRoles = $link['access'] ?? [SimpleLogin::getDefaultRoleId()];
                    $userRole = $_SESSION['role'] ?? SimpleLogin::getDefaultRoleId();

                    // Admin bypass check
                    if ($userRole === SimpleLogin::getAdminRoleId()) {
                        // Admin can access all files
                        continue; // Skip role checks for admin
                    }

                    if (!in_array($userRole, $allowedRoles, true)) {
                        throw new Exception("Access denied. You do not have permission to view this page.");
                    }
                }
            }
            if (!$fileFound) {
                throw new Exception("Requested module file not found in module configuration.");
            }
        }

        $filePath = $modulesDir . '/' . $sanitizedModule . '/' . $sanitizedFile;
        if (file_exists($filePath)) {
            require_once $filePath;
        } else {
            throw new Exception("Requested module file does not exist.");
        }
    }

    /**
     * Generate a URL for a module file.
     *
     * @param string $module The name of the module
     * @param string $file The name of the file
     * @param array $params Optional parameters to include in the query string
     * @return string The generated URL
     */
    public static function generateUrl(string $module, string $file, array $params = []): string
    {
        $url = 'module.php?module=' . urlencode($module) . '&file=' . urlencode($file);
        if (!empty($params)) {
            $url .= '&' . http_build_query($params);
        }
        return $url;
    }

    /**
     * Checks if all required modules are loaded. Returns true if all modules are loaded, false if not.
     * Optionally, it can display an error and exit if any module is missing.
     *
     * @param array $moduleNames The names of the required modules
     * @param string|null $fileName (optional) The name of the file making the check
     * @param bool $throwError Whether to throw an error if modules are missing (default: true)
     * @param bool $isStrict Whether the check should stop execution if the module is not found (default: false)
     * @return bool Returns true if all required modules are loaded, false otherwise
     */
    public static function requiresModules(array $moduleNames, string $fileName = null, bool $throwError = true, bool $isStrict = true): bool
    {
        try {
            $modules = self::loadModules(); // Load all modules
            $loadedModuleNames = array_column($modules, 'module_name'); // Get only the names of loaded modules

            // Find missing modules
            $missingModules = array_diff($moduleNames, $loadedModuleNames);

            if (!empty($missingModules)) {
                $fileMessage = $fileName ? " for this file ($fileName)" : "";
                $missingModulesCount = count($missingModules);
                $missingModulesList = implode(', ', $missingModules);

                // Use singular or plural form based on the count of missing modules
                $moduleVerb = $missingModulesCount === 1 ? "module" : "modules";
                $areOrIs = $missingModulesCount === 1 ? "is" : "are";

                // If the error flag is set to true, throw the exception or just return false based on $isStrict flag
                if ($throwError) {
                    if ($isStrict) {
                        throw new Exception("The required $moduleVerb ($missingModulesList) $areOrIs not loaded$fileMessage.");
                    } else {
                        return false;
                    }
                }

                return false; // Return false if modules are missing and throwError is false
            }
        } catch (Exception $e) {
            // Display the error message and stop script execution if throwError is true and isStrict is true
            if ($throwError && $isStrict) {
                echo "<strong>Critical Error:</strong> " . htmlspecialchars($e->getMessage(), ENT_QUOTES);
                exit;
            }

            return false; // Return false if the exception occurs and throwError is false
        }

        return true; // Return true if all required modules are loaded
    }
}
