<?php
/**
 * Pagination class
 *
 * This class handles pagination for displaying items across multiple pages.
 * It provides methods for setting the total number of items, calculating the
 * current page, and rendering pagination links for navigation.
 *
 * Key functionality:
 * - Setting the total number of items using `setTotalItems`.
 * - Retrieving total pages with `getTotalPages`.
 * - Getting current page number with `getCurrentPage`.
 * - Rendering pagination links with `render`.
 *
 * Usage example:
 * $usersPerPage = 10; // Define how many users to display per page
 * 
 * // Fetch all users and total count
 * $allUsers = $auth->getAllUsers(); // Retrieve all users
 * $totalUsers = count($allUsers); // Count total users for pagination
 * 
 * // Initialize pagination
 * $pagination = new Pagination($usersPerPage);
 * $pagination->setTotalItems($totalUsers);
 *
 * // Get current page entries for display
 * $current_page = $pagination->getCurrentPage();
 * $users = array_slice($allUsers, ($current_page - 1) * $pagination->getLimit(), $pagination->getLimit(), true); 
 * 
 * // Render pagination links
 * echo $pagination->render('manage_users.php');
 * ```
 */
class Pagination
{
    private $limit; // Items per page
    private $current_page; // Current page number
    private $total_items = 0; // Total number of items
    private $display_limit; // Number of pagination links to display

    /**
     * Pagination constructor.
     *
     * @param int $limit          Number of items per page (default is 5)
     * @param int $display_limit  Number of pagination links to display (default is 3)
     */
    public function __construct($limit = 5, $display_limit = 3)
    {
        // Validate and set the limit
        $this->limit = max(1, $limit);

        // Set current page based on the 'page' parameter in the URL, default is 1
        $this->current_page = isset($_GET['page']) ? (int) $_GET['page'] : 1;
        $this->current_page = max(1, $this->current_page); // Ensure current page is positive

        // Set the display limit for pagination links
        $this->display_limit = max(1, $display_limit);
    }

    /**
     * Set the total number of items for pagination.
     *
     * @param int $total_items The total number of items
     */
    public function setTotalItems($total_items)
    {
        $this->total_items = max(0, $total_items);
        // Adjust current page if it exceeds total pages
        if ($this->current_page > $this->getTotalPages()) {
            $this->current_page = $this->getTotalPages();
        }
    }

    /**
     * Get the total number of pages.
     *
     * @return int Total pages calculated based on items per page.
     */
    public function getTotalPages()
    {
        return ceil($this->total_items / $this->limit);
    }

    /**
     * Get the current page number.
     *
     * @return int The current page number.
     */
    public function getCurrentPage()
    {
        return $this->current_page;
    }

    /**
     * Get the limit of items per page.
     *
     * @return int The limit of items per page.
     */
    public function getLimit()
    {
        return $this->limit;
    }

    /**
     * Get the offset for the current page, useful for slicing the items array.
     *
     * @return int The offset for the current page.
     */
    public function getOffset()
    {
        return ($this->current_page - 1) * $this->limit;
    }

    /**
     * Render pagination links dynamically.
     *
     * @param string $base_url The base URL for the pagination links.
     * @param array $extra_params Additional parameters as key-value pairs for the URL.
     * @return string HTML for pagination links.
     */
    public function render($base_url, array $extra_params = [])
    {
        $total_pages = $this->getTotalPages();

        // Determine if customization is enabled (default to false)
        $customized = $extra_params['customized'] ?? false;

        // Fetch settings only if customization is enabled
        $wrapper_before = $customized
            ? ModuleLoader::getSetting('Default', 'paginationWrapperBefore')
            : '<div class="pagination">';
        $wrapper_after = $customized
            ? ModuleLoader::getSetting('Default', 'paginationWrapperAfter')
            : '</div>';
        $show_first_last = $customized
            ? ModuleLoader::getSetting('Default', 'paginationShowFirstAndLast')
            : true;
        $show_numbers = $customized
            ? ModuleLoader::getSetting('Default', 'paginationShowNumbers')
            : true;
        $first_symbol = $customized
            ? ModuleLoader::getSetting('Default', 'paginationFirstLinkSymbol')
            : '&laquo;';
        $last_symbol = $customized
            ? ModuleLoader::getSetting('Default', 'paginationLastLinkSymbol')
            : '&raquo;';
        $next_symbol = $customized
            ? ModuleLoader::getSetting('Default', 'paginationNextLinkSymbol')
            : '&rsaquo;';
        $prev_symbol = $customized
            ? ModuleLoader::getSetting('Default', 'paginationPreviousLinkSymbol')
            : '&lsaquo;';

        // Start building the output
        $output = $wrapper_before;

        // Render "First" and "Previous" links if needed
        if ($this->current_page > 1) {
            if ($show_first_last) {
                $output .= $this->renderPageLink(1, $base_url, $first_symbol, $extra_params); // First
            }
            $output .= $this->renderPageLink($this->current_page - 1, $base_url, $prev_symbol, $extra_params); // Previous
        }

        // Calculate the page range to display
        $start_page = max(1, $this->current_page - floor($this->display_limit / 2));
        $end_page = min($total_pages, $start_page + $this->display_limit - 1);
        if ($end_page - $start_page < $this->display_limit - 1) {
            $start_page = max(1, $end_page - $this->display_limit + 1);
        }

        // Render page links
        if ($show_numbers) {
            for ($i = $start_page; $i <= $end_page; $i++) {
                $output .= $this->renderPageLink($i, $base_url, (string) $i, $extra_params);
            }
        }

        // Render "Next" and "Last" links if needed
        if ($this->current_page < $total_pages) {
            $output .= $this->renderPageLink($this->current_page + 1, $base_url, $next_symbol, $extra_params); // Next
            if ($show_first_last) {
                $output .= $this->renderPageLink($total_pages, $base_url, $last_symbol, $extra_params); // Last
            }
        }

        // Close the wrapper
        $output .= $wrapper_after;

        return $output;
    }

    /**
     * Helper method to render an individual page link with the current page parameter.
     *
     * @param int $page Page number.
     * @param string $base_url Base URL for pagination.
     * @param string $text Text to display for the link.
     * @param array $extra_params Additional parameters as key-value pairs.
     * @return string HTML for a single page link.
     */
    private function renderPageLink($page, $base_url, $text, array $extra_params = [])
    {
        $is_current_page = ($page == $this->current_page);
        $class = $is_current_page ? 'current-page' : '';

        // Parse the URL and add the 'page' parameter
        $url_parts = parse_url($base_url);
        parse_str($url_parts['query'] ?? '', $query_params);

        // Include the page parameter
        $query_params['page'] = $page;

        // Merge with extra parameters
        $query_params = array_merge($query_params, $extra_params);

        // Remove 'customized' parameter since it is only for internal use
        unset($query_params['customized']);

        $query_string = http_build_query($query_params);
        $url = "{$url_parts['path']}?$query_string";

        return "<a href=\"" . htmlspecialchars($url) . "\" class=\"$class\">$text</a>";
    }
}