<?php
/**
 * SimpleLogin Class
 *
 * This class handles user registration, login, and management for a simple login system.
 * It provides functionalities to register users, log them in and out, manage their data,
 * and enforce role-based access control. User data is stored in PHP files within a specified 
 * directory.
 *
 * PHP Version: 8.0.0 or higher
 * 
 */

if (version_compare(PHP_VERSION, '8.0.0', '<')) {
    die('This script requires PHP version 8.0.0 or higher. Your current version is ' . PHP_VERSION . ".\n\n"
        . "To resolve this issue, please check your web hosting settings to update your PHP version. "
        . "If you are unsure how to do this, we recommend contacting your hosting provider for assistance.");
}

// Check for required extensions
$missingExtensions = [];

if (!class_exists('DOMDocument')) {
    $missingExtensions[] = 'DOM extension';
}

if (!extension_loaded('mbstring')) {
    $missingExtensions[] = 'mbstring extension';
}

if (!extension_loaded('gd')) {
    $missingExtensions[] = 'GD extension';
}

if (!extension_loaded('fileinfo')) {
    $missingExtensions[] = 'Fileinfo extension';
}

if (!empty($missingExtensions)) {
    die("This script requires the following PHP extensions to be enabled: " . implode(', ', $missingExtensions) . ".\n\n"
        . "To resolve this issue, please check your PHP settings to ensure these extensions are enabled. "
        . "You may need to edit your `php.ini` file or contact your hosting provider for assistance.");
}

if (session_status() === PHP_SESSION_NONE) {
    session_start();
}

class SimpleLogin
{
    // Current version
    const VERSION = "1.0.4";

    // Shows the current version of the module
    public function getVersion()
    {
        return self::VERSION;
    }

    /**
     * Role constants used throughout the core application.
     * 
     * Important: Do not rename the following constants (ROLE_ADMIN, ROLE_MEMBER)
     * as they are integral to the core functionality. Instead, update the 'name'
     * field in the ROLES array to modify how these roles are displayed in the application.
     */
    public static $roles = [];
    public const ROLE_ADMIN = 1; // Constant for the Admin role
    public const ROLE_MEMBER = 2; // Constant for the Member role
    public const ROLE_CUSTOM = 3; // Constant for a Custom Role

    // Data directory
    private const DATA_DIR = USER_DATA_DIR;

    // The directory where smiley images are stored
    private string $uploadDir;

    // The path for smiley image uploads relative to the content directory
    public const IMAGE_UPLOAD_DIR = '/profile_pictures/';

    // File names
    private const USER_FILE_NAME = 'users.php';
    private const USER_DATA_FILE_NAME = 'users_data.php';

    private $users = [];
    private $userData = [];

    private $userFile; // Stores the user id and email address
    private $userDataFile; // Contains profile data linked to the user IDs

    private $isRegistrationOpen = REGISTRATION_OPEN; // Determines if registration is open or closed
    private $sessionDataRefreshed = false;

    public const ERROR_MESSAGES = [
        'username_format' => "Invalid username format. Only alphanumeric characters and underscores are allowed (3-20 characters).",
        'email_format' => "Invalid email format. Please provide a valid email address.",
        'email_exists' => "An account with that email address already exists.",
        'login_failed' => "Login failed. Please check your credentials and try again.",
        'login_required' => "Please log in to access this page.",
        'forced_logout' => "You have been logged out. If you believe this is an error, please contact the admin.",
        'password_mismatch' => "The provided passwords do not match.",
        'no_permission' => "You do not have permission to access this page.",
        'user_not_found' => "The specified user does not exist.",
        'self_deletion' => "You cannot delete your own account.",
        'update_failed' => "Failed to update user data.",
        'deletion_failed' => "Failed to save changes after deletion.",
        'password_update_failed' => "Password update failed. Please try again.",
        'new_password_has_to_be_different' => 'The new password must be different from the current password.',
    ];

    public const SUCCESS_MESSAGES = [
        'registration_success' => "Registration successful! You can now log in.",
        'logged_out' => "You have been logged out successfully.",
        'user_deleted' => "User deleted successfully.",
        'profile_updated' => "Profile updated successfully!",
        'password_updated' => 'Your password has been updated successfully. Please log in again with your new password. If you encounter issues, try clearing your browser cache or refreshing the page using Ctrl + F5 (Strg + F5).',
    ];

    private $defaultUserFields = ['username', 'email', 'role', 'session_token'];

    public function __construct()
    {
        // Make sure the Settings module is installed properly
        ModuleLoader::requiresModules(["Settings"]);

        // Ensure that the roles are initialized only once, not every time an instance is created
        if (empty(self::$roles)) {
            self::$roles = [
                self::ROLE_ADMIN => [
                    'name' => ModuleLoader::getSetting('Default', 'roleNameAdmin'), // Display name for the Admin role
                    'description' => 'Administrator with full access', // Description of Admin role
                    'visible' => true, // Should this role be visible in the UI?
                ],
                self::ROLE_MEMBER => [
                    'name' => ModuleLoader::getSetting('Default', 'roleNameMember'), // Display name for the Member role
                    'description' => 'Regular member access', // Description of Member role
                    'visible' => true, // Should this role be visible in the UI?
                ],
                self::ROLE_CUSTOM => [
                    'name' => 'Custom Role', // Display name for Custom Role
                    'description' => 'You can create a custom role like this', // Description for Custom Role
                    'visible' => false, // Should this role be visible in the UI?
                ],
            ];
        }

        // Ensure header.php and footer.php exist
        $this->ensureHeaderFooterFilesExist();

        $this->uploadDir = CONTENT_DIR . self::IMAGE_UPLOAD_DIR;
        $this->userFile = self::DATA_DIR . '/' . self::USER_FILE_NAME;
        $this->userDataFile = self::DATA_DIR . '/' . self::USER_DATA_FILE_NAME;

        $this->loadUsers();
        $this->loadUserData();

        // Unique Module Icon for SimpleLogin (this will be used in the dashboard)
        ModuleLoader::setDefaultSettings(
            'Default',
            'moduleIcon',
            '', // No value here, it's just for the icon
            '', // No label here
            '', // No description
            'string', // Just a placeholder for type
            'fa-wand-magic-sparkles' // FontAwesome icon for the module
        );

        // Setting to enable or disable the TinyMCE editor
        ModuleLoader::setDefaultSettings(
            'Default',
            'tinyMCE',
            true,
            'Enable TinyMCE Editor',
            'When enabled, the WYSIWYG editor TinyMCE will be used instead of standard textareas, wherever applicable.',
            'bool', // The value is a boolean (checkbox input)
            'fa-code' // FontAwesome icon 
        );

        // Setting for Dark Mode
        ModuleLoader::setDefaultSettings(
            'Default',
            'darkMode',
            false,
            'Enable Dark Mode',
            'When enabled, Simplify will use a dark theme. Disable to revert to the default light theme.',
            'bool', // The value is a boolean (checkbox input)
            'fa-moon' // FontAwesome icon for dark mode
        );

        // Setting for accent color
        ModuleLoader::setDefaultSettings(
            'Default',
            'simplifyAccentColor',
            '#7480e9',
            'Accent Color',
            'Select a color to customize the accent throughout the Simplify admin interface. Default: <code>#7480e9</code>',
            'color', // The value is a color picker (input type="color")
            'fa-palette' // FontAwesome icon
        );

        // Setting for birthday date format
        ModuleLoader::setDefaultSettings(
            'Default',
            'birthdayDateFormat',
            'd.m.Y', // Default birthday date format
            'Birthday Date Format',
            'Date format for the birthday in the profile page.',
            'string', // The value is a string (text input)
            'fa-cake' // FontAwesome icon
        );

        // Setting for the registration status
        ModuleLoader::setDefaultSettings(
            'Default',
            'registrationStatus',
            true,
            'Allow Registrations',
            'When enabled, users will be able to register for an account on the site. Disable this option to prevent new user registrations.',
            'bool', // The value is a boolean (checkbox input)
            'fa-users' // FontAwesome icon
        );

        // Setting for showing age in profiles
        ModuleLoader::setDefaultSettings(
            'Default',
            'showAgeInProfiles',
            true,
            'Show Age in Profiles',
            'When enabled, the age will be displayed on the profile.',
            'bool', // The value is a boolean (checkbox input)
            'fa-calendar' // FontAwesome icon for the setting
        );

        // Setting for showing birthday in profiles
        ModuleLoader::setDefaultSettings(
            'Default',
            'showBirthdayInProfiles',
            true,
            'Show Birthday in Profiles',
            'When enabled, the birthday will be displayed on the profile.',
            'bool', // The value is a boolean (checkbox input)
            'fa-birthday-cake' // FontAwesome icon for the setting
        );

        // Setting for the timezone
        ModuleLoader::setDefaultSettings(
            'Default',
            'timezone',
            'Europe/Berlin',
            'Timezone',
            'The timezone used for all date and time functions. Check <a href="https://www.php.net/manual/en/timezones.php" target="_blank">this</a> page for more details on valid timezone formats.',
            'string', // The value is a boolean (checkbox input)
            'fa-clock' // FontAwesome icon
        );

        // Get the timezone setting from the ModuleLoader
        $timezone = ModuleLoader::getSetting('Default', 'timezone') ?? 'Europe/Berlin'; // Default to Europe/Berlin if not set

        // Set the timezone for the application
        date_default_timezone_set($timezone);

        // Site title tag
        ModuleLoader::setDefaultSettings(
            'Default',
            'titleTag',
            'Your Site',
            'Site Title',
            'The title shown in the title tags of your head section.',
            'string', // The value is a string (text input)
            'fa-heading' // FontAwesome icon for the setting
        );

        // Security setting to control error visibility
        ModuleLoader::setDefaultSettings(
            'Default',
            'showErrors',
            false,
            'Display PHP Errors',
            'When enabled, PHP errors will be visible on the site, which can help identify problems but may reveal sensitive server information or file paths.',
            'bool', // Boolean value (checkbox input)
            'fa-shield' // FontAwesome icon indicating security
        );

        // Public name for the role Admin
        ModuleLoader::setDefaultSettings(
            'Default',
            'roleNameAdmin',
            'Admin',
            'Public name for the role Admin',
            'Public label for the Admin role, used across the site wherever the role is displayed.',
            'string', // The value is a string (text input)
            'fa-user-tag' // FontAwesome icon for the setting
        );

        // Public name for the role Member
        ModuleLoader::setDefaultSettings(
            'Default',
            'roleNameMember',
            'Member',
            'Public name for the role Member',
            'Public label for the Member role, used across the site wherever the role is displayed.',
            'string', // The value is a string (text input)
            'fa-user-tag' // FontAwesome icon for the setting
        );

        // Security setting to control error visibility
        ModuleLoader::setDefaultSettings(
            'Default',
            'addFontAwesome',
            true,
            'FontAwesome Support',
            'Adds support for FontAwesome.',
            'bool', // Boolean value (checkbox input)
            'fa-icons' // FontAwesome icon indicating security
        );

        // Pagination settings
        ModuleLoader::setDefaultSettings(
            'Default',
            'paginationWrapperBefore',
            '<div class="pagination">',
            'Pagination Wrapper Before',
            'HTML wrapper',
            'string', // The value is a string (text input)
            'fa-code' // FontAwesome icon for the setting
        );

        ModuleLoader::setDefaultSettings(
            'Default',
            'paginationWrapperAfter',
            '</div>',
            'Pagination Wrapper After',
            'HTML wrapper',
            'string', // The value is a string (text input)
            'fa-code' // FontAwesome icon for the setting
        );

        ModuleLoader::setDefaultSettings(
            'Default',
            'paginationFirstLinkSymbol',
            '&laquo;',
            'Pagination First',
            'The symbol/text shown for the first page in the pagination. Default: <code>&amp;laquo;</code> (&laquo;)',
            'string', // The value is a string (text input)
            'fa-code' // FontAwesome icon for the setting
        );

        ModuleLoader::setDefaultSettings(
            'Default',
            'paginationLastLinkSymbol',
            '&raquo;',
            'Pagination Last',
            'The symbol/text shown for the last page in the pagination. Default: <code>&amp;raquo;</code> (&raquo;)',
            'string', // The value is a string (text input)
            'fa-code' // FontAwesome icon for the setting
        );

        ModuleLoader::setDefaultSettings(
            'Default',
            'paginationNextLinkSymbol',
            '&rsaquo;',
            'Pagination Next',
            'The symbol/text shown for the next page in the pagination. Default: <code>&amp;rsaquo;</code> (&rsaquo;)',
            'string', // The value is a string (text input)
            'fa-code' // FontAwesome icon for the setting
        );

        ModuleLoader::setDefaultSettings(
            'Default',
            'paginationPreviousLinkSymbol',
            '&lsaquo;',
            'Pagination Previous',
            'The symbol/text shown for the previous page in the pagination. Default: <code>&amp;lsaquo;</code> (&lsaquo;)',
            'string', // The value is a string (text input)
            'fa-code' // FontAwesome icon for the setting
        );

        ModuleLoader::setDefaultSettings(
            'Default',
            'paginationShowNumbers',
            true,
            'Show Pagination Numbers',
            'When enabled, pagination numbers are visible. When disabled, only the Previous and Next links will be visible.',
            'bool', // The value is a boolean (checkbox input)
            'fa-ellipsis' // FontAwesome icon
        );

        ModuleLoader::setDefaultSettings(
            'Default',
            'paginationShowFirstAndLast',
            true,
            'Show First and Last Pagination Link',
            'When enabled, it will show a link for the first and last page in the pagination.',
            'bool', // The value is a boolean (checkbox input)
            'fa-ellipsis' // FontAwesome icon
        );
    }

    /**
     * Ensures that header.php and footer.php exist. If not, creates them with proper content.
     */
    private function ensureHeaderFooterFilesExist()
    {
        $headerPath = __DIR__ . '/../../header.php';
        $footerPath = __DIR__ . '/../../footer.php';

        // Check if header.php exists
        if (!file_exists($headerPath)) {
            // Create header.php with the required HTML content
            $headerContent = <<<HTML
<!DOCTYPE html>
<html>

<head>
    <title><?= function_exists('getSiteTitle') ? getSiteTitle(\$pageTitleSuffix ?? '') : ''; ?></title>

    <meta charset="UTF-8">

    <!-- Stylesheets -->
    <link rel="stylesheet" type="text/css" href="style.css">
    <link rel="stylesheet" type="text/css" href="profile.css">

    <?php
    // Render assets
    if (method_exists('AssetsManager', 'renderAssets')) echo AssetsManager::renderAssets('header');

    // Render SEO meta tags
    if (isset(\$blog) && is_object(\$blog) && method_exists(\$blog, 'renderMetaTags')) \$blog->renderMetaTags(\$metaTags);

    // Add FontAwesome stylesheet
    if (class_exists('ModuleLoader') && method_exists('ModuleLoader', 'getSetting') && ModuleLoader::getSetting('Default', 'addFontAwesome')): ?>
        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.6.0/css/all.min.css">
    <?php endif; ?>
</head>

<body>

    <div id="container">
HTML;

            // Write the content to the header.php file
            file_put_contents($headerPath, $headerContent);
            chmod($headerPath, 0644); // Set appropriate permissions
        }

        // Check if footer.php exists
        if (!file_exists($footerPath)) {
            // Create footer.php with the required HTML content
            $footerContent = <<<HTML
</div> <!-- End of container -->

<?php if (method_exists('AssetsManager', 'renderAssets')) echo AssetsManager::renderAssets('footer'); ?>

</body>
</html>
HTML;

            // Write the content to the footer.php file
            file_put_contents($footerPath, $footerContent);
            chmod($footerPath, 0644); // Set appropriate permissions
        }
    }

    // Load users from the Users file if it exists
    private function loadUsers()
    {
        if (file_exists($this->userFile)) {
            $this->users = include($this->userFile);
        } else {
            file_put_contents($this->userFile, "<?php\n return [];"); // Create file if it doesn't exist
            chmod($this->userFile, 0600);
        }
    }

    // Load user data from the UserData file if it exists
    private function loadUserData()
    {
        if (file_exists($this->userDataFile)) {
            $this->userData = include($this->userDataFile);
        } else {
            file_put_contents($this->userDataFile, "<?php\n return [];"); // Create file if it doesn't exist
            chmod($this->userDataFile, 0600);
        }
    }

    /**
     * Saves user information to the user file with restricted file permissions.
     *
     * @return bool True if the file write operation is successful, otherwise false.
     */
    private function saveUsers()
    {
        // Attempt to write user data to the file
        if (file_put_contents($this->userFile, "<?php\n return " . var_export($this->users, true) . ";") === false) {
            return false; // Return false if file write fails
        }

        chmod($this->userFile, 0600); // Change file permissions
        return true; // Return true if file write is successful
    }

    /**
     * Saves user data to the user data file with restricted file permissions.
     *
     * @return bool True if the file write operation is successful, otherwise false.
     */
    private function saveUserData()
    {
        // Attempt to write user data to the file
        if (file_put_contents($this->userDataFile, "<?php\n return " . var_export($this->userData, true) . ";") === false) {
            return false; // Return false if file write fails
        }

        chmod($this->userDataFile, 0600); // Change file permissions
        return true; // Return true if file write is successful
    }

    /**
     * Generates a unique user ID based on a random number within a defined range.
     *
     * @return int A unique user ID.
     */
    private function generateUserId(): int
    {
        // Get all existing user IDs.
        $existingUserIds = array_column($this->userData, 'id');

        do {
            $id = random_int(1000, 100000); // Generate a random ID
        } while (in_array($id, $existingUserIds, true)); // Repeat until a unique ID is found

        return $id;
    }

    /**
     * Validates the format of a username.
     *
     * @param string $username The username to validate.
     * @return bool True if the username format is valid, otherwise false.
     */
    private function validateUsername($username)
    {
        return preg_match('/^[a-zA-Z0-9_]{3,20}$/', $username);
    }

    /**
     * Validates the format of an email address.
     *
     * @param string $email The email address to validate.
     * @return bool True if the email address format is valid, otherwise false.
     */
    private function validateEmail($email)
    {
        return filter_var($email, FILTER_VALIDATE_EMAIL);
    }

    /**
     * Registers a new user after validation.
     *
     * @param array $userInfo User data (username, email, password).
     * @return bool True on success, false on failure.
     */
    public function register($userInfo)
    {
        // Sanitize input
        $userInfo['username'] = sanitizeInput($userInfo['username']);
        $userInfo['email'] = sanitizeInput($userInfo['email']);

        // Check if confirm_password is set and matches the password
        if (!isset($_POST['confirm_password']) || $userInfo['password'] !== trim($_POST['confirm_password'])) {
            Message::set('error', self::ERROR_MESSAGES['password_mismatch']);
            return false;
        }

        // Validate username and email
        if (!$this->validateUsername($userInfo['username'])) {
            Message::set('error', self::ERROR_MESSAGES['username_format']);
            return false;
        }

        if (!$this->validateEmail($userInfo['email'])) {
            Message::set('error', self::ERROR_MESSAGES['email_format']);
            return false;
        }

        // Ensure the email is unique
        if (isset($this->users[$userInfo['email']])) {
            Message::set('error', self::ERROR_MESSAGES['email_exists']);
            return false;
        }

        // Generate a unique user ID and hash the password
        $userId = $this->generateUserId();
        $this->users[$userInfo['email']] = password_hash($userInfo['password'], PASSWORD_DEFAULT);

        // Assign role and store user data without password
        $role = count($this->userData) === 0 ? self::getAdminRoleId() : self::getDefaultRoleId();
        $userInfo['role'] = $role;

        // Generate a unique session token for this user
        $userInfo['session_token'] = bin2hex(random_bytes(32));

        unset($userInfo['password']);
        $this->userData[$userId] = array_merge(array_fill_keys($this->defaultUserFields, ''), $userInfo);

        // Save users and user data
        $this->saveUsers();
        $this->saveUserData();

        // After the first user registers, close registration
        if (count($this->userData) === 1) {  // Check if this is the first user
            ModuleLoader::setSetting('Default', 'registrationStatus', false);  // Close registration after the first user
        }

        // Set success message and redirect
        Message::set('success', self::SUCCESS_MESSAGES['registration_success']);
        safeRedirect("login.php");

        return true;
    }

    /**
     * Logs in a user by verifying credentials.
     *
     * @param string $email The user's email.
     * @param string $password The user's password.
     * @return bool True if login is successful, false otherwise.
     */
    public function login($email, $password)
    {
        // Sanitize input
        $email = sanitizeInput($email);

        if (isset($this->users[$email]) && password_verify($password, $this->users[$email])) {
            session_regenerate_id(true);
            foreach ($this->userData as $id => $data) {
                if ($data['email'] === $email) {
                    // Set session variables, including a session token
                    $_SESSION['user_id'] = $id;
                    $_SESSION['username'] = $data['username'];
                    $_SESSION['role'] = $data['role'];
                    $_SESSION['session_token'] = $data['session_token'];

                    break;
                }
            }
            return true;
        }

        Message::set('error', self::ERROR_MESSAGES['login_failed']);
        return false;
    }

    // Log the user out and clear the session with an optional message
    public function logout($messageKey = 'logged_out')
    {
        // Clear session data including the session token
        session_unset();
        session_destroy();

        // Check if the provided messageKey exists in SUCCESS_MESSAGES
        if (array_key_exists($messageKey, self::SUCCESS_MESSAGES)) {
            Message::set('success', self::SUCCESS_MESSAGES[$messageKey]);
        } elseif (array_key_exists($messageKey, self::ERROR_MESSAGES)) {
            Message::set('error', self::ERROR_MESSAGES[$messageKey]);
        } else {
            Message::set('success', self::SUCCESS_MESSAGES['logged_out']);
        }

        // Redirect to the login page after logging out
        safeRedirect("login.php");
    }

    /**
     * Checks if the user is currently logged in by verifying session data.
     *
     * @return bool True if the user is logged in, otherwise false.
     */
    public function isLoggedIn()
    {
        if (isset($_SESSION['user_id'])) {
            $userId = $_SESSION['user_id'];
            $userData = $this->getUserById($userId);

            // Verify the session token to prevent access from old sessions
            if ($userData && $userData['session_token'] === $_SESSION['session_token']) {
                return true;
            } else {
                // If token doesn't match, force logout
                $this->logout('forced_logout');
            }
        }
        return false;
    }

    // Refresh session data if the user is logged in
    public function refreshSessionData()
    {
        if ($this->isLoggedIn() && !$this->sessionDataRefreshed) {
            $userId = $_SESSION['user_id'];
            $userData = $this->getUserById($userId);

            // Check if the session token matches
            if ($userData && $userData['session_token'] === $_SESSION['session_token']) {
                $_SESSION['username'] = $userData['username'];
                $_SESSION['role'] = $userData['role'];
            } else {
                $this->logout('forced_logout'); // Force logout if the token is invalid
            }

            $this->sessionDataRefreshed = true;
        }
    }

    // Ensure the user is logged in before accessing protected pages
    public function requireLogin()
    {
        $this->refreshSessionData();

        if (!$this->isLoggedIn()) {
            Message::set('error', self::ERROR_MESSAGES['login_required']);
            safeRedirect("login.php");
        }
    }

    /**
     * Gets all role names.
     *
     * @return array List of role names.
     */
    public static function getRoleNames(): array
    {
        // Using self::$roles to retrieve role data and returning only the 'name' field
        return array_column(self::$roles, 'name');
    }

    /**
     * Gets role information by ID.
     *
     * @param int $roleId The role ID.
     * @return array|null The role data (name, description, visibility), or null if not found.
     */
    public static function getRoleById(int $roleId): ?array
    {
        // Checking if the role exists in self::$roles and returning its data
        return self::$roles[$roleId] ?? null;
    }

    /**
     * Gets the role name by ID.
     *
     * @param int $roleId The role ID.
     * @return string|null The role name, or null if not found.
     */
    public static function getRoleName(int $roleId): ?string
    {
        // Retrieving the role name by its ID
        return self::$roles[$roleId]['name'] ?? null;
    }

    /**
     * Gets the role description by ID.
     *
     * @param int $roleId The role ID.
     * @return string|null The role description, or null if not found.
     */
    public static function getRoleDescription(int $roleId): ?string
    {
        // Retrieving the role description by its ID
        return self::$roles[$roleId]['description'] ?? null;
    }

    /**
     * Gets the role ID by name.
     *
     * @param string $roleName The role name.
     * @return int|null The role ID, or null if not found.
     */
    public static function getRoleId(string $roleName): ?int
    {
        // Iterating over the roles to find the matching name and return the role ID
        foreach (self::$roles as $id => $role) {
            if ($role['name'] === $roleName) {
                return $id;
            }
        }
        return null;
    }

    // Methods to get role IDs without hardcoding the names
    public static function getAdminRoleId(): int
    {
        return self::ROLE_ADMIN; // Directly return the constant
    }

    public static function getDefaultRoleId(): int
    {
        return self::ROLE_MEMBER; // Directly return the constant
    }

    /**
     * Gets all visible roles with their IDs.
     *
     * @return array List of role IDs for visible roles.
     */
    public static function getAllRoles(): array
    {
        return array_filter(array_keys(self::$roles), function ($roleId) {
            return self::$roles[$roleId]['visible'] ?? true; // Default visibility to true if not set
        });
    }

    // Check if the logged-in user has admin privileges
    public function isAdmin()
    {
        return isset($_SESSION['role']) && $_SESSION['role'] === self::getAdminRoleId();
    }

    // Ensure the user has admin privileges for accessing protected admin pages
    public function requireAdmin()
    {
        if (!$this->isAdmin()) {
            die(self::ERROR_MESSAGES['no_permission']);
        }
    }

    // Check if user registration is currently open
    public function isRegistrationOpen()
    {
        return $this->isRegistrationOpen;
    }

    // Retrieve user data based on user ID
    public function getUserById($userId)
    {
        return $this->userData[$userId] ?? null;
    }

    /**
     * Retrieves the username based on a user ID.
     *
     * @param int $userId The user ID.
     * @return string The username if found, or "Deleted User" if not found.
     */
    public static function getUsernameById(int $userId): string
    {
        // Load the user data file directly in the static method
        $userDataFile = self::DATA_DIR . '/' . self::USER_DATA_FILE_NAME;

        // Check if the file exists
        if (file_exists($userDataFile)) {
            // Include the file to get the user data
            $userData = include($userDataFile);

            // Check if the userId exists in the data
            if (isset($userData[$userId]) && isset($userData[$userId]['username'])) {
                return $userData[$userId]['username']; // Return the username if found
            }
        }

        // Return "Deleted User" if the user doesn't exist or file isn't found
        return "Deleted User";
    }

    /**
     * Updates user data for a specified user ID.
     *
     * @param int $userId The user ID to update.
     * @param array $data The new user data (e.g., username, email, role).
     * @return bool True if the update is successful, otherwise false.
     */
    public function updateUserData($userId, $data)
    {
        // Sanitize input
        $data['username'] = sanitizeInput($data['username']);
        $data['email'] = sanitizeInput($data['email']);

        // Check if the user exists in the data
        if (isset($this->userData[$userId])) {
            // Validate username
            if (isset($data['username']) && !$this->validateUsername($data['username'])) {
                Message::set('error', self::ERROR_MESSAGES['username_format']);
                return false;
            }

            // Validate email and check if it's unique (only if the email has changed)
            if (isset($data['email']) && $data['email'] !== $this->userData[$userId]['email']) {
                if (!$this->validateEmail($data['email'])) {
                    Message::set('error', self::ERROR_MESSAGES['email_format']);
                    return false;
                }

                // Ensure the email is unique, excluding the current user's email
                foreach ($this->userData as $existingUserId => $userInfo) {
                    // Only check if the new email is different from the current user's email
                    if ($existingUserId !== $userId && $userInfo['email'] === $data['email']) {
                        Message::set('error', self::ERROR_MESSAGES['email_exists']);
                        return false;
                    }
                }
            }

            // Cast role to integer if it's set
            if (isset($data['role'])) {
                $data['role'] = (int) $data['role']; // Ensure role is an integer
            }

            // Update user data, including all fields
            $this->userData[$userId] = array_merge($this->userData[$userId], $data);

            // Save user data and check if it was successful
            if ($this->saveUserData()) {
                Message::set('success', self::SUCCESS_MESSAGES['profile_updated']); // Success message
                return true; // Indicate that the update was successful
            } else {
                Message::set('error', self::ERROR_MESSAGES['update_failed']); // Error message if saving fails
                return false; // Indicate failure to save user data
            }
        }

        // If user not found
        Message::set('error', self::ERROR_MESSAGES['user_not_found']);
        return false; // Indicate failure to find user
    }

    /**
     * Updates the password for a specified user.
     *
     * @param int $userId The ID of the user whose password is being updated.
     * @param string $newPassword The new password to be set.
     * @param string $confirmPassword The confirmation of the new password.
     *
     * @return bool True if the password update was successful, false otherwise.
     */
    public function updatePassword($userId, $newPassword, $confirmPassword)
    {
        // Only proceed if both password fields are filled
        if (empty($newPassword) && empty($confirmPassword)) {
            return true; // No update needed if no password fields are provided
        }

        // Get the user data by user ID to fetch the email address
        $user = $this->getUserById($userId);

        // Check if the user exists
        if (!$user) {
            Message::set('error', self::ERROR_MESSAGES['user_not_found']);
            return false;
        }

        $email = $user['email']; // Retrieve the user's email from the user data

        // Check if new password and confirm password match
        if ($newPassword !== $confirmPassword) {
            Message::set('error', self::ERROR_MESSAGES['password_mismatch']);
            return false;
        }

        // Prevent rehashing the old password by checking if the new password matches the old one
        if (password_verify($newPassword, $this->users[$email])) {
            Message::set('error', self::ERROR_MESSAGES['new_password_has_to_be_different']);
            return false;
        }

        // Hash the new password
        $hashedPassword = password_hash($newPassword, PASSWORD_BCRYPT);

        // Update password
        if (isset($this->users[$email])) {
            $this->users[$email] = $hashedPassword;
        }

        // Save user data and check if it was successful
        if ($this->saveUsers()) {
            $this->logout('password_updated'); // This will show the success message for password update

            return true;
        } else {
            Message::set('error', self::ERROR_MESSAGES['password_update_failed']);
            return false;
        }
    }

    /**
     * Updates the user's profile picture (avatar).
     *
     * @param int $userId The user ID to update the profile picture for.
     * @return bool True if the update is successful, otherwise false.
     */
    public function updateProfilePicture($userId)
    {
        // Check if the user exists
        if (isset($this->userData[$userId])) {
            // Directory and allowed file types for avatar
            $uploadDir = CONTENT_DIR . self::IMAGE_UPLOAD_DIR;
            $allowedMimeTypes = ['image/jpeg', 'image/png', 'image/gif'];
            $prefix = 'avatar_';
            $isMulti = false;
            $isRequired = false;

            // Initialize file upload class
            $uploader = new SimpleUpload($uploadDir, $allowedMimeTypes, $prefix, $isMulti);

            // Check if the avatar file is uploaded without errors
            if (isset($_FILES['avatar']) && $_FILES['avatar']['error'] === UPLOAD_ERR_OK) {
                // Attempt to upload the avatar
                $uploadResult = $uploader->upload('avatar', [], $isRequired);

                // Check if the upload was successful
                if (strpos($uploadResult, 'success') !== false) {
                    // Get the uploaded file's name
                    $avatarFilename = $uploader->getUploadedFilesAsString();

                    // Check if the user already has an existing avatar and delete the old one if needed
                    if (!empty($this->userData[$userId]['avatar'])) {
                        $oldAvatar = $this->userData[$userId]['avatar'];
                        $oldAvatarPath = $uploadDir . '/' . $oldAvatar;
                        if (file_exists($oldAvatarPath)) {
                            unlink($oldAvatarPath); // Remove the old avatar
                        }
                    }

                    // Update the user data with the new avatar filename
                    $this->userData[$userId]['avatar'] = $avatarFilename;

                    // Save the user data and check if it was successful
                    if ($this->saveUserData()) {
                        Message::set('success', self::SUCCESS_MESSAGES['profile_updated']);
                        return true; // Avatar update successful
                    } else {
                        Message::set('error', self::ERROR_MESSAGES['update_failed']);
                        return false; // Failed to save user data
                    }
                } else {
                    // If upload failed, set error message
                    Message::set('error', self::ERROR_MESSAGES['update_failed']);
                    return false; // Avatar upload failed
                }
            } else {
                // No file was uploaded or an error occurred during upload
                Message::set('error', self::ERROR_MESSAGES['update_failed']);
                return false; // No file or upload error
            }
        }

        // If user is not found, return an error message
        Message::set('error', self::ERROR_MESSAGES['user_not_found']);
        return false; // User not found
    }


    /**
     * Returns the URL of the user's avatar or an empty string if not set.
     *
     * @param int $userId The user ID to fetch the avatar for.
     * @return string The avatar image URL.
     */
    public function getUserProfilePicture($userId)
    {
        // Fetch user data to get the avatar field
        $userData = $this->getUserById($userId);

        // Check if the user has an avatar
        if (!empty($userData['avatar'])) {
            // If avatar exists, return the full URL
            return normalizeUrl(generateBaseUrl() . basename(CONTENT_DIR) . self::IMAGE_UPLOAD_DIR . $userData['avatar']);
        }

        // Return an empty string if no avatar is set
        return '';
    }

    /**
     * Clears the user's profile picture (avatar).
     *
     * @param int $userId The user ID to clear the profile picture for.
     * @return bool True if the profile picture is cleared successfully, otherwise false.
     */
    public function clearProfilePicture($userId)
    {
        // Check if the user exists
        if (isset($this->userData[$userId])) {
            // Check if the user already has an avatar
            if (!empty($this->userData[$userId]['avatar'])) {
                // Get the avatar filename
                $avatarFilename = $this->userData[$userId]['avatar'];

                // Define the upload directory
                $uploadDir = CONTENT_DIR . self::IMAGE_UPLOAD_DIR;

                // Remove the avatar file from the filesystem
                $avatarFilePath = $uploadDir . '/' . $avatarFilename;
                if (file_exists($avatarFilePath)) {
                    unlink($avatarFilePath); // Delete the avatar file
                }

                // Remove the metadata entry associated with the avatar
                if (!$this->removeMetadataEntry($avatarFilename)) {
                    Message::set('error', 'Failed to remove metadata for the profile picture.');
                    return false;
                }

                // Clear the avatar field in the user data
                $this->userData[$userId]['avatar'] = '';

                // Save the updated user data
                if ($this->saveUserData()) {
                    Message::set('success', 'Profile picture cleared successfully.');
                    return true;
                } else {
                    Message::set('error', 'Failed to save user data after clearing profile picture.');
                    return false;
                }
            } else {
                // If there is no avatar set, return an error
                Message::set('error', 'No profile picture found to clear.');
                return false;
            }
        }

        // If user is not found, return an error message
        Message::set('error', 'User not found.');
        return false;
    }

    /**
     * Helper function to remove the metadata entry associated with a deleted image.
     *
     * @param string $imageName The name of the image file whose metadata is to be removed.
     * @return bool True if metadata removal was successful, false otherwise.
     */
    private function removeMetadataEntry(string $imageName): bool
    {
        $databaseFilePath = $this->uploadDir . 'database.json';

        // Check if the metadata file exists
        if (!file_exists($databaseFilePath)) {
            Message::set('error', self::ERROR_MESSAGES['metadata_not_found']);
            return false; // Return false if metadata not found
        }

        // Read and decode the JSON file
        $data = file_get_contents($databaseFilePath);
        $metadata = json_decode($data, true);

        // If no metadata is found, return false
        if (!is_array($metadata)) {
            Message::set('error', self::ERROR_MESSAGES['no_metadata_found']);
            return false; // Return false if there is no valid metadata
        }

        // Filter out the metadata entry corresponding to the image name
        $updatedMetadata = array_filter($metadata, fn($entry) => $entry['imageName'] !== $imageName);

        // Write the updated metadata back to the database.json
        if (file_put_contents($databaseFilePath, json_encode(array_values($updatedMetadata), JSON_PRETTY_PRINT)) === false) {
            Message::set('error', self::ERROR_MESSAGES['metadata_update_failed']);
            return false; // Return false if writing fails
        }

        return true; // Return true if everything was successful
    }

    /**
     * Retrieves all users, sorted alphabetically by username.
     *
     * @return array Sorted user data.
     */
    public function getAllUsers()
    {
        // Sort users by username while preserving keys
        $sortedUsers = $this->userData;
        uasort($sortedUsers, function ($a, $b) {
            return strcmp($a['username'], $b['username']);
        });

        // Return the sorted user data
        return $sortedUsers;
    }

    /**
     * Deletes a user by their ID.
     *
     * @param int $userId The ID of the user to delete.
     * @return bool True if deletion is successful, false otherwise.
     */
    public function deleteById($userId)
    {
        // Check if the user ID exists
        if (!isset($this->userData[$userId])) {
            Message::set('error', self::ERROR_MESSAGES['user_not_found']);
            return false;
        }

        // Get the email of the user being deleted
        $email = $this->userData[$userId]['email'] ?? null;

        // Sanitize input
        $email = sanitizeInput($email);

        // Check if the current user is trying to delete themselves
        if ($this->isCurrentUser($userId)) {
            Message::set('error', self::ERROR_MESSAGES['self_deletion']);
            return false;
        }

        // Remove user data
        unset($this->userData[$userId]);

        // Also check if the user email exists in users array
        if ($email && isset($this->users[$email])) {
            unset($this->users[$email]);
        }

        // Save changes to both user data and users
        $saveUserDataResult = $this->saveUserData();
        $saveUsersResult = $this->saveUsers();

        if ($saveUserDataResult && $saveUsersResult) {
            Message::set('success', self::SUCCESS_MESSAGES['user_deleted']);
            return true;
        } else {
            Message::set('error', self::ERROR_MESSAGES['deletion_failed']);
            return false;
        }
    }

    /**
     * Check if the specified user ID is the same as the currently logged-in user's ID.
     *
     * @param int $userId The user ID to check against the current user.
     * @return bool True if the user is the current user, false otherwise.
     */
    public function isCurrentUser($userId)
    {
        // Sanitize input
        $userId = sanitizeInput($userId);

        // Read user ID from session and cast to int to ensure proper comparison
        $currentUserId = (int) $_SESSION['user_id'];

        // Compare IDs (both converted to int)
        return $currentUserId === (int) $userId;
    }
}

// Generates the "Manage Users" link for the dashboard.
function addManageUsersLink()
{
    global $auth; // Access the global auth instance to check user roles

    // Check if the user is an admin
    if ($auth->isAdmin()) {
        // Return an HTML link to the 'Manage Users' page, styled for the dashboard
        return '<a href="manage_users.php" class="orange">
                    <i class="fas fa-users-cog"></i> Manage Users
                </a>';
    }

    // Return an empty string if the user is not an admin (no link will be shown)
    return '';
}

// Register the function to the 'dashboard_links' hook to include it in the dashboard display
HookManager::addHook('dashboard_links', 'addManageUsersLink');
