<?php
/**
 * SimpleUpload class
 * Handles secure file uploads, including single and multiple file uploads,
 * as well as ZIP file uploads with extraction capabilities.
 */
class SimpleUpload
{
    public const ERROR_MESSAGES = [
        'no_file_uploaded' => 'No file uploaded, but it is required.',
        'invalid_file_upload' => 'Invalid file upload.',
        'temporary_file_missing' => 'Temporary file does not exist.',
        'invalid_file_type' => 'Invalid file type ',
        'move_failed' => 'Failed to move uploaded file.',
        'zip_extraction_failed' => 'Failed to extract ZIP file.',
    ];

    public const SUCCESS_MESSAGES = [
        'file_uploaded' => 'File uploaded successfully.',
        'multiple_files_uploaded' => 'All files uploaded successfully.',
        'file_metadata_saved' => 'File metadata saved successfully.',
        'zip_uploaded_and_extracted' => 'ZIP file uploaded and extracted successfully.',
    ];

    private string $uploadDir; // Directory where uploaded files will be stored
    private array $allowedTypes; // Array of allowed MIME types for uploads
    private string $prefix; // Prefix to prepend to uploaded file names
    private bool $isMulti; // Indicates if multiple file uploads are allowed
    private array $uploadedFiles = []; // Array to hold uploaded filenames

    /**
     * Constructor for the SimpleUpload class.
     *
     * @param string $uploadDir Directory where files will be uploaded.
     * @param array $allowedTypes Array of allowed MIME types for uploads.
     * @param string $prefix Optional prefix for uploaded file names.
     * @param bool $isMulti Indicates if multiple files can be uploaded.
     * @throws InvalidArgumentException if no allowed types are specified.
     */
    public function __construct(string $uploadDir, array $allowedTypes, string $prefix = '', bool $isMulti = false)
    {
        if (empty($allowedTypes)) {
            throw new InvalidArgumentException('At least one allowed file type must be specified.');
        }

        $this->prefix = $prefix;
        $this->uploadDir = rtrim($uploadDir, '/') . '/';
        $this->allowedTypes = $allowedTypes;
        $this->isMulti = $isMulti;

        ensureRequiredDirectories($this->uploadDir);
    }

    /**
     * Uploads a file or multiple files.
     *
     * @param string $fileField The name of the file input field.
     * @param array $metadata Optional metadata to associate with the uploaded file(s).
     * @param bool $isRequired Indicates if the file is required.
     * @return string|false A success message or an error message, or false if not required and no file is uploaded.
     */
    public function upload(string $fileField, array $metadata = [], bool $isRequired = false, bool $generateThumbnails = true)
    {
        if (!isset($_FILES[$fileField])) {
            return $isRequired ? self::ERROR_MESSAGES['no_file_uploaded'] : false;
        }

        return $this->isMulti ?
            $this->uploadMultipleFiles($fileField, $metadata, $isRequired, $generateThumbnails) :
            $this->uploadSingleFile($fileField, $metadata, $isRequired, $generateThumbnails);
    }

    /**
     * Generates a secure, unique filename for each uploaded file.
     *
     * @param string $fileExtension The file extension (e.g., "jpg", "png").
     * @return string A unique filename with the prefix.
     */
    protected function generateUniqueFileName(string $extension): string
    {
        // Shorten the unique identifier by using a hash
        $uniqueId = hash('sha256', uniqid('', true));
        $shortenedId = substr($uniqueId, 0, 10);
        return $this->prefix . $shortenedId . '.' . $extension;
    }

    /**
     * Uploads a single file.
     *
     * @param string $fileField The name of the file input field.
     * @param array $metadata Optional metadata to associate with the uploaded file.
     * @param bool $isRequired Indicates if the file is required.
     * @return string A success or error message.
     */
    private function uploadSingleFile(string $fileField, array $metadata, bool $isRequired, bool $generateThumbnails)
    {
        $file = $_FILES[$fileField];

        if ($file['error'] === UPLOAD_ERR_NO_FILE) {
            return $isRequired ? self::ERROR_MESSAGES['no_file_uploaded'] : false;
        }

        if ($this->isFileUploadInvalid($file)) {
            return self::ERROR_MESSAGES['invalid_file_upload'] . ': ' . $this->getUploadErrorMessage($file['error']);
        }

        if (!file_exists($file['tmp_name'])) {
            return self::ERROR_MESSAGES['temporary_file_missing'];
        }

        $fileType = mime_content_type($file['tmp_name']);
        if (!in_array($fileType, $this->allowedTypes, true)) {
            return self::ERROR_MESSAGES['invalid_file_type'] . implode(', ', $this->allowedTypes);
        }

        // Generate a unique filename
        $fileExtension = pathinfo($file['name'], PATHINFO_EXTENSION);
        $fileName = $this->generateUniqueFileName($fileExtension);
        $targetFilePath = $this->uploadDir . $fileName;

        // Move the uploaded file
        if (move_uploaded_file($file['tmp_name'], $targetFilePath)) {
            // Generate a thumbnail if requested
            if ($generateThumbnails) {
                $this->createThumbnail($targetFilePath, $fileExtension);
            }

            // Save metadata and return success message
            $this->uploadedFiles[] = $fileName;
            $this->saveMetadata($fileName, $metadata);
            return self::SUCCESS_MESSAGES['file_uploaded'];
        }

        return self::ERROR_MESSAGES['move_failed'];
    }

    /**
     * Uploads multiple files.
     *
     * @param string $fileField The name of the file input field.
     * @param array $metadata Optional metadata to associate with the uploaded files.
     * @param bool $isRequired Indicates if files are required.
     * @return string A success or error message.
     */
    private function uploadMultipleFiles(string $fileField, array $metadata, bool $isRequired, bool $generateThumbnails)
    {
        $files = $_FILES[$fileField];
        $errorMessages = [];

        // Process each file one by one
        foreach ($files['name'] as $index => $fileName) {
            $file = [
                'name' => $fileName,
                'type' => $files['type'][$index],
                'tmp_name' => $files['tmp_name'][$index],
                'error' => $files['error'][$index],
                'size' => $files['size'][$index],
            ];

            // Check if file exists or is required
            if ($file['error'] === UPLOAD_ERR_NO_FILE) {
                if ($isRequired) {
                    $errorMessages[] = self::ERROR_MESSAGES['no_file_uploaded'];
                }
                continue;
            }

            // Validate file upload
            if ($this->isFileUploadInvalid($file)) {
                $errorMessages[] = self::ERROR_MESSAGES['invalid_file_upload'] . ': ' . $this->getUploadErrorMessage($file['error']);
                continue;
            }

            if (!file_exists($file['tmp_name'])) {
                $errorMessages[] = self::ERROR_MESSAGES['temporary_file_missing'] . ' for ' . $file['name'];
                continue;
            }

            // Validate MIME type
            $fileType = mime_content_type($file['tmp_name']);
            if (!in_array($fileType, $this->allowedTypes, true)) {
                $errorMessages[] = self::ERROR_MESSAGES['invalid_file_type'] . ' for ' . $file['name'] . '. Allowed types are: ' . implode(', ', $this->allowedTypes);
                continue;
            }

            // Generate unique filename
            $fileExtension = pathinfo($file['name'], PATHINFO_EXTENSION);
            $uniqueFileName = $this->generateUniqueFileName($fileExtension);
            $targetFilePath = $this->uploadDir . $uniqueFileName;

            // Move the uploaded file
            if (move_uploaded_file($file['tmp_name'], $targetFilePath)) {
                // Generate a thumbnail if requested
                if ($generateThumbnails) {
                    $this->createThumbnail($targetFilePath, $fileExtension);
                }

                // Save metadata and add to uploaded files
                $this->uploadedFiles[] = $uniqueFileName;
                $this->saveMetadata($uniqueFileName, $metadata);
            } else {
                $errorMessages[] = self::ERROR_MESSAGES['move_failed'] . ': ' . $file['name'];
            }

            // Free up memory after each file is processed
            unset($file);
            gc_collect_cycles(); // Force garbage collection for large files
        }

        if (!empty($errorMessages)) {
            return implode('; ', $errorMessages);
        }

        return self::SUCCESS_MESSAGES['multiple_files_uploaded'] . ' Uploaded files: ' . implode(', ', $this->uploadedFiles);
    }

    /**
     * Creates a thumbnail from the given image file.
     * 
     * @param string $filePath The path to the image file.
     * @param string $fileExtension The file extension (e.g., 'jpg', 'png').
     */
    private function createThumbnail(string $filePath, string $fileExtension)
    {
        // Only process image files
        if (!in_array(strtolower($fileExtension), ['jpg', 'jpeg', 'png', 'gif'])) {
            return;
        }
    
        // Get image dimensions
        list($width, $height) = getimagesize($filePath);
    
        // Set thumbnail dimensions (500x500px)
        $thumbWidth = 500;
        $thumbHeight = 500;
    
        // Check if the image is smaller than the target thumbnail size
        if ($width < $thumbWidth && $height < $thumbHeight) {
            // For smaller images, just copy them without resizing
            $thumbImage = imagecreatetruecolor($width, $height);
            
            // Ensure transparency for PNG and GIF
            if ($fileExtension === 'png' || $fileExtension === 'gif') {
                imagealphablending($thumbImage, false);
                imagesavealpha($thumbImage, true); // Save alpha channel (transparency)
            }
            
            // Create the source image based on the file extension
            switch (strtolower($fileExtension)) {
                case 'jpg':
                case 'jpeg':
                    $sourceImage = imagecreatefromjpeg($filePath);
                    break;
                case 'png':
                    $sourceImage = imagecreatefrompng($filePath);
                    break;
                case 'gif':
                    $sourceImage = imagecreatefromgif($filePath);
                    break;
                default:
                    return;
            }
    
            // Copy the image without resizing (just keep it original size)
            imagecopy($thumbImage, $sourceImage, 0, 0, 0, 0, $width, $height);
    
            // Set the resized image to null since it's not needed for small images
            $resizedImage = null;
    
        } else {
            // For images larger than 500x500, scale them to fit within the 500px boundary
    
            // Calculate the scaling ratio for the image
            $srcAspect = $width / $height;
            $thumbAspect = $thumbWidth / $thumbHeight;
    
            // Calculate new width and height for resizing based on the aspect ratio
            if ($srcAspect > $thumbAspect) {
                // Image is wider than the thumbnail
                $newHeight = $thumbHeight;
                $newWidth = (int) ($newHeight * $srcAspect);
            } else {
                // Image is taller than the thumbnail
                $newWidth = $thumbWidth;
                $newHeight = (int) ($newWidth / $srcAspect);
            }
    
            // Create the new resized image with a transparent background for PNG or GIF
            $thumbImage = imagecreatetruecolor($thumbWidth, $thumbHeight);
            if ($fileExtension === 'png' || $fileExtension === 'gif') {
                imagealphablending($thumbImage, false);
                imagesavealpha($thumbImage, true); // Save transparency
            }
    
            // Create the source image based on the file extension
            switch (strtolower($fileExtension)) {
                case 'jpg':
                case 'jpeg':
                    $sourceImage = imagecreatefromjpeg($filePath);
                    break;
                case 'png':
                    $sourceImage = imagecreatefrompng($filePath);
                    break;
                case 'gif':
                    $sourceImage = imagecreatefromgif($filePath);
                    break;
                default:
                    return;
            }
    
            // Create a resized version of the source image
            $resizedImage = imagecreatetruecolor($newWidth, $newHeight);
            if ($fileExtension === 'png' || $fileExtension === 'gif') {
                imagealphablending($resizedImage, false);
                imagesavealpha($resizedImage, true); // Preserve transparency in resized image
            }
    
            // Copy and resize the image while preserving the aspect ratio
            imagecopyresampled($resizedImage, $sourceImage, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
    
            // Calculate position to center the image within the 500x500 thumbnail
            $xOffset = (int) (($newWidth - $thumbWidth) / 2);
            $yOffset = (int) (($newHeight - $thumbHeight) / 2);
    
            // Copy the resized image onto the final thumbnail image, cropping it if necessary
            imagecopy($thumbImage, $resizedImage, 0, 0, $xOffset, $yOffset, $thumbWidth, $thumbHeight);
        }
    
        // Save the thumbnail with a "thumb_" prefix
        $thumbFilePath = $this->uploadDir . 'thumb_' . basename($filePath);
    
        switch (strtolower($fileExtension)) {
            case 'jpg':
            case 'jpeg':
                imagejpeg($thumbImage, $thumbFilePath);
                break;
            case 'png':
                imagepng($thumbImage, $thumbFilePath);
                break;
            case 'gif':
                imagegif($thumbImage, $thumbFilePath);
                break;
        }
    
        // Free up memory
        imagedestroy($thumbImage);
        imagedestroy($sourceImage);
    
        // Only destroy $resizedImage if it's created
        if ($resizedImage !== null) {
            imagedestroy($resizedImage);
        }
    }    

    /**
     * Checks if the file upload is invalid based on the error code.
     *
     * @param array $file The uploaded file information.
     * @return bool True if the upload is invalid, false otherwise.
     */
    private function isFileUploadInvalid(array $file): bool
    {
        return $file['error'] !== UPLOAD_ERR_OK;
    }

    /**
     * Returns a descriptive error message based on the error code.
     *
     * @param int $errorCode The error code from the file upload.
     * @return string A human-readable error message.
     */
    private function getUploadErrorMessage(int $errorCode): string
    {
        switch ($errorCode) {
            case UPLOAD_ERR_INI_SIZE:
                return 'The uploaded file exceeds the upload_max_filesize directive in php.ini.';
            case UPLOAD_ERR_FORM_SIZE:
                return 'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form.';
            case UPLOAD_ERR_PARTIAL:
                return 'The uploaded file was only partially uploaded.';
            case UPLOAD_ERR_NO_FILE:
                return 'No file was uploaded.';
            case UPLOAD_ERR_NO_TMP_DIR:
                return 'Missing a temporary folder.';
            case UPLOAD_ERR_CANT_WRITE:
                return 'Failed to write file to disk.';
            case UPLOAD_ERR_EXTENSION:
                return 'A PHP extension stopped the file upload.';
            default:
                return 'Unknown upload error.';
        }
    }

    /**
     * Saves metadata for the uploaded file to a JSON database.
     *
     * @param string $fileName The name of the uploaded file.
     * @param array $metadata Optional metadata to save with the file.
     */
    private function saveMetadata(string $fileName, array $metadata): void
    {
        // Ensure session is started
        if (session_status() === PHP_SESSION_NONE) {
            session_start(); // Start the session if not already started
        }

        $databaseFilePath = $this->uploadDir . 'database.json';

        // Load existing data if the database file already exists
        $existingData = [];
        if (file_exists($databaseFilePath)) {
            $existingData = json_decode(file_get_contents($databaseFilePath), true) ?? []; // Handle JSON decode errors
        }

        // Add file extension to metadata if it exists
        $fileExtension = pathinfo($fileName, PATHINFO_EXTENSION);

        // Create a new entry with the file name, current timestamp, and user ID
        $newEntry = [
            'imageName' => basename($fileName), // Save only the filename without path
            'created_at' => date('Y-m-d H:i:s'), // Add created_at timestamp
            'user_id' => $_SESSION['user_id'] ?? null, // Add user ID from session,
            'file_extension' => $fileExtension, // Add user ID from session
        ];

        // Include additional metadata if provided
        if (!empty($metadata)) {
            $newEntry['metadata'] = $metadata;
        }

        // Append the new entry to the existing data
        $existingData[] = $newEntry;

        // Write the updated data back to the database file
        file_put_contents($databaseFilePath, json_encode($existingData, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
    }

    /**
     * Handles uploading and extracting a ZIP file.
     *
     * @param string $fileField The name of the file input field for the ZIP file.
     * @param array $metadata Optional metadata to associate with the uploaded ZIP file.
     * @param bool $isRequired Indicates if the ZIP file is required.
     * @return string|false A success message or an error message, or false if not required and no file is uploaded.
     */
    public function uploadAndExtractZip(string $fileField, array $metadata, bool $isRequired)
    {
        $file = $_FILES[$fileField];

        if ($file['error'] === UPLOAD_ERR_NO_FILE) {
            return $isRequired ? self::ERROR_MESSAGES['no_file_uploaded'] : false;
        }

        if ($this->isFileUploadInvalid($file)) {
            return self::ERROR_MESSAGES['invalid_file_upload'] . ': ' . $this->getUploadErrorMessage($file['error']);
        }

        if (!file_exists($file['tmp_name'])) {
            return self::ERROR_MESSAGES['temporary_file_missing'];
        }

        // Check MIME type against allowed types
        $fileType = mime_content_type($file['tmp_name']);
        if (!in_array($fileType, $this->allowedTypes, true)) {
            return self::ERROR_MESSAGES['invalid_file_type'] . implode(', ', $this->allowedTypes);
        }

        // Generate a unique filename for the zip file
        $fileExtension = pathinfo($file['name'], PATHINFO_EXTENSION);
        $timestamp = time(); // Get the current timestamp
        $fileName = $this->prefix . $timestamp . ($this->prefix ? '_' : '') . '.' . $fileExtension;
        $targetFilePath = $this->uploadDir . $fileName;

        // Move the uploaded zip file to the target directory
        if (!move_uploaded_file($file['tmp_name'], $targetFilePath)) {
            return self::ERROR_MESSAGES['move_failed'];
        }

        // Create a directory for extraction based on the file name (without extension)
        $extractedDirName = pathinfo($fileName, PATHINFO_FILENAME); // Get the filename without the extension
        $extractedDirPath = $this->uploadDir . $extractedDirName . '/';

        // Ensure the directory exists
        if (!mkdir($extractedDirPath, 0777, true) && !is_dir($extractedDirPath)) {
            return self::ERROR_MESSAGES['zip_extraction_failed'];
        }

        // Proceed to extract the zip file into the created directory
        return $this->extractZipFile($targetFilePath, $extractedDirPath, $metadata);
    }

    /**
     * Extracts a ZIP file to a specified directory.
     *
     * @param string $filePath The path to the ZIP file to extract.
     * @param string $extractedDirPath The directory where the files will be extracted.
     * @param array $metadata Optional metadata to save with the ZIP file.
     * @return string A success or error message.
     */
    private function extractZipFile(string $filePath, string $extractedDirPath, array $metadata)
    {
        $zip = new ZipArchive;
        if ($zip->open($filePath) === true) {
            $zip->extractTo($extractedDirPath); // Extract to the specified directory
            $zip->close();
            // Clean up the ZIP file after extraction
            unlink($filePath);

            // Update metadata for the ZIP file before saving it
            $zipFileName = basename($filePath); // Ensure we are saving just the zip file name
            $this->saveMetadata($zipFileName, $metadata); // Save metadata for the zip file

            return self::SUCCESS_MESSAGES['zip_uploaded_and_extracted'];
        } else {
            unlink($filePath); // Clean up the uploaded ZIP file
            return self::ERROR_MESSAGES['zip_extraction_failed'];
        }
    }

    /**
     * Get the filenames of the uploaded files.
     *
     * @return array An array of uploaded filenames.
     */
    public function getUploadedFiles(): array
    {
        return $this->uploadedFiles;
    }

    /**
     * Get the filenames of the uploaded files as a string.
     *
     * @return string A string of uploaded filenames, separated by commas.
     */
    public function getUploadedFilesAsString(): string
    {
        // Check if there are any uploaded files
        if (empty($this->uploadedFiles)) {
            return ''; // Return an empty string if no files were uploaded
        }

        // Join the array of filenames into a single string, separated by commas
        return implode(', ', $this->uploadedFiles);
    }
}
