<?php
// Load necessary classes and functions
require_once __DIR__ . '/autoload.php';
require_once __DIR__ . '/functions.php';
require_once __DIR__ . '/config.php';

// Initialize the login class
$auth = new SimpleLogin();

// Ensure user is logged in
$auth->requireLogin();

// Load modules and register hooks
$modules = ModuleLoader::loadModules();

// Retrieve the user ID to edit from the URL or default to the logged-in user's ID
$editUserId = $_GET['id'] ?? $_SESSION['user_id'];

// Fetch current user data for the specified edit user ID
$userData = $auth->getUserById($editUserId);

// Check if user data was found
if (!$userData) {
    die($auth::ERROR_MESSAGES['user_not_found']);
}

// Define the fields for user data with default values
$userFields = [
    'username' => $userData['username'] ?? '',
    'email' => $userData['email'] ?? '',
    'birthday' => $userData['birthday'] ?? '',
    'nickname' => $userData['nickname'] ?? '',
    'bio' => $userData['bio'] ?? '',
    'role' => $userData['role'] ?? SimpleLogin::getDefaultRoleId(), // Default to member
    'avatar' => $userData['avatar'] ?? '',

    // Likes and Dislikes
    'likes' => $userData['likes'] ?? '',
    'dislikes' => $userData['dislikes'] ?? '',

    // Social media fields
    'facebook' => $userData['facebook'] ?? '',
    'bluesky' => $userData['bluesky'] ?? '',
    'x' => $userData['x'] ?? '',
    'tiktok' => $userData['tiktok'] ?? '',
    'discord' => $userData['discord'] ?? '',
    'reddit' => $userData['reddit'] ?? '',
    'instagram' => $userData['instagram'] ?? '',
    'deviantart' => $userData['deviantart'] ?? '',

    // Add any other fields you may need here
];

// Check if the current user is an admin
$isAdmin = $auth->isAdmin();

// Ensure users can only edit their own profile unless they are an admin
if (!$isAdmin && $editUserId != $_SESSION['user_id']) {
    die($auth::ERROR_MESSAGES['no_permission']); // Permission denied
}

// Handle form submission for profile update
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Gather updated fields
    $updatedData = [];
    foreach (array_keys($userFields) as $field) {
        if (isset($_POST[$field])) {
            $updatedData[$field] = trim($_POST[$field]); // Trim whitespace from input
        }
    }

    // Prevent self-downgrade by skipping role changes if editing own profile
    if ($editUserId == $_SESSION['user_id'] && isset($updatedData['role'])) {
        // Only allow role to change if it's not going to downgrade the user
        if ($userFields['role'] === SimpleLogin::getAdminRoleId() && $updatedData['role'] !== SimpleLogin::getAdminRoleId()) {
            // If user is an admin and is trying to change to a non-admin role, remove the role change
            unset($updatedData['role']);
        }
    }

    // Validate and update user data
    if ($auth->updateUserData($editUserId, $updatedData)) {
        foreach ($updatedData as $field => $value) {
            if (array_key_exists($field, $userFields)) {
                $userFields[$field] = $value; // Reflect updated values
            }
        }
    }

    // Check for profile picture upload (avatar)
    if (isset($_FILES['avatar']) && $_FILES['avatar']['error'] === UPLOAD_ERR_OK) {
        if ($auth->updateProfilePicture($editUserId)) {
            // If avatar is updated successfully, reflect the change in the user fields
            $userFields['avatar'] = $auth->getUserById($editUserId)['avatar'];  // Make sure we get the updated avatar info
        }
    }
}

// Handle password update if the form for password change is submitted
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $newPassword = isset($_POST['newpassword']) ? trim($_POST['newpassword']) : '';
    $confirmNewPassword = isset($_POST['confirmnewpassword']) ? trim($_POST['confirmnewpassword']) : '';

    $auth->updatePassword($editUserId, $newPassword, $confirmNewPassword);
}

// Check if the delete_profile_picture parameter is set in the GET request
if (isset($_GET['delete_profile_picture']) && $_GET['delete_profile_picture'] == 1 && isset($_GET['id'])) {
    // Get the user ID from the GET request
    $userIdToDeleteAvatar = $_GET['id'];

    // Call the deleteProfileImage function to delete the user's avatar
    if ($auth->clearProfilePicture($userIdToDeleteAvatar)) {
        // Redirect back to the profile page with a success message
        safeRedirect('edit_profile.php', ['id' => $userIdToDeleteAvatar]);
    } else {
        // Redirect back with an error message
        safeRedirect('edit_profile.php', ['id' => $userIdToDeleteAvatar]);
    }
}

// Fetch the current avatar URL using the new function
$avatarUrl = $auth->getUserProfilePicture($editUserId);

// Retrieve any error or success messages
$messages = Message::get();

// Include header file for the page layout
include_once __DIR__ . '/header.php';
?>

<div class="content">

    <h1>Edit Profile</h1>

    <!-- Display any messages -->
    <?php if (!empty($messages)): ?>
        <?php foreach ($messages as $msg): ?>
            <p class="<?php echo sanitizeInput($msg['type']); ?>"><?php echo sanitizeInput($msg['text']); ?></p>
        <?php endforeach; ?>
    <?php endif; ?>

    <form method="POST" class="form" enctype="multipart/form-data">
        <!-- Tabs Navigation -->
        <div class="tabs action-links">
            <a href="javascript:void(0);" class="tab-link" onclick="openTab(event, 'personalInfo')">
                <i class="fa fa-user"></i> Personal Info
            </a>
            <a href="javascript:void(0);" class="tab-link" onclick="openTab(event, 'password')">
                <i class="fa fa-shield"></i> Change Password
            </a>
            <a href="javascript:void(0);" class="tab-link" onclick="openTab(event, 'socialMedia')">
                <i class="fa fa-share-alt"></i> Social Media
            </a>
            <a href="javascript:void(0);" class="tab-link" onclick="openTab(event, 'likesDislikes')">
                <i class="fa fa-thumbs-up"></i> Likes & Dislikes
            </a>
        </div>

        <!-- Tab Content -->
        <div id="personalInfo" class="tab-content">
            <div class="form-profile-picture size-small">
                <div class="form-current-profile-picture">
                    <?php if (!empty($userFields['avatar'])): ?>
                        <img src="<?php echo $avatarUrl; ?>" alt="Avatar">
                        <a class="action-icon delete-icon"
                            href="edit_profile.php?id=<?php echo htmlspecialchars($editUserId); ?>&delete_profile_picture=1"
                            onclick="return confirm('Are you sure you want to delete your profile image?')">
                            <i class="fa-solid fa-circle-xmark"></i>
                        </a>
                    <?php endif; ?>
                </div>

                <label for="avatar"><i class="fa fa-camera"></i> Profile Picture (Avatar):</label>
                <input type="file" id="avatar" name="avatar" accept="image/*">
            </div>

            <div class="horizontal">
                <div>
                    <label for="username"><i class="fa fa-user"></i> Username:</label>
                    <input type="text" id="username" name="username"
                        value="<?php echo sanitizeInput($userFields['username']); ?>">
                </div>

                <div>
                    <label for="email"><i class="fa fa-envelope"></i> Email:</label>
                    <input type="email" id="email" name="email"
                        value="<?php echo sanitizeInput($userFields['email']); ?>">
                </div>
            </div>

            <div class="horizontal">
                <div>
                    <label for="birthday"><i class="fa fa-calendar"></i> Birthday:</label>
                    <input type="date" id="birthday" name="birthday"
                        value="<?php echo sanitizeInput($userFields['birthday']); ?>">
                </div>

                <?php if ($isAdmin): ?>
                    <div>
                        <label for="role"><i class="fa fa-user-lock"></i> Role:</label>
                        <select id="role" name="role">
                            <?php foreach (SimpleLogin::getAllRoles() as $roleId): ?>
                                <option value="<?php echo htmlspecialchars($roleId, ENT_QUOTES); ?>" <?php echo (int) $userFields['role'] === (int) $roleId ? 'selected' : ''; ?>>
                                    <?php
                                    // Display the role name
                                    $roleName = SimpleLogin::getRoleName($roleId);
                                    echo htmlspecialchars(ucfirst($roleName), ENT_QUOTES);
                                    ?>
                                </option>
                            <?php endforeach; ?>
                        </select>
                    </div>
                <?php endif; ?>
            </div>

            <div>
                <label for="bio"><i class="fa fa-pencil-alt"></i> Bio:</label>
                <textarea rows="15" id="bio" name="bio"
                    class="tinymce"><?php echo sanitizeInput($userFields['bio']); ?></textarea>
            </div>
        </div>

        <div id="password" class="tab-content">
            <div class="horizontal">
                <div>
                    <label for="newpassword"><i class="fa fa-shield-halved"></i> New Password:</label>
                    <input type="password" id="newpassword" name="newpassword">
                </div>

                <div>
                    <label for="confirmnewpassword"><i class="fa fa-shield-halved"></i> Confirm New Password:</label>
                    <input type="password" id="confirmnewpassword" name="confirmnewpassword">
                </div>
            </div>
        </div>

        <div id="socialMedia" class="tab-content">
            <div class="horizontal">
                <div>
                    <label for="facebook"><i class="fa-brands fa-facebook"></i> Facebook:</label>
                    <input type="text" id="facebook" name="facebook"
                        value="<?php echo sanitizeInput($userFields['facebook']); ?>"
                        placeholder="https://facebook.com/yourprofile">
                </div>

                <div>
                    <label for="bluesky"><i class="fa-brands fa-bluesky"></i> Bluesky:</label>
                    <input type="text" id="bluesky" name="bluesky"
                        value="<?php echo sanitizeInput($userFields['bluesky']); ?>"
                        placeholder="https://bsky.app/yourprofile">
                </div>
            </div>

            <div class="horizontal">
                <div>
                    <label for="x"><i class="fa-brands fa-x-twitter"></i> X (formerly Twitter):</label>
                    <input type="text" id="x" name="x" value="<?php echo sanitizeInput($userFields['x']); ?>"
                        placeholder="https://x.com/yourprofile">
                </div>

                <div>
                    <label for="tiktok"><i class="fa-brands fa-tiktok"></i> TikTok:</label>
                    <input type="text" id="tiktok" name="tiktok"
                        value="<?php echo sanitizeInput($userFields['tiktok']); ?>"
                        placeholder="https://tiktok.com/@yourprofile">
                </div>
            </div>

            <div class="horizontal">
                <div>
                    <label for="discord"><i class="fa-brands fa-discord"></i> Discord:</label>
                    <input type="text" id="discord" name="discord"
                        value="<?php echo sanitizeInput($userFields['discord']); ?>" placeholder="yourusername#1234">
                </div>

                <div>
                    <label for="reddit"><i class="fa-brands fa-reddit"></i> Reddit:</label>
                    <input type="text" id="reddit" name="reddit"
                        value="<?php echo sanitizeInput($userFields['reddit']); ?>"
                        placeholder="https://reddit.com/u/yourprofile">
                </div>
            </div>

            <div class="horizontal">
                <div>
                    <label for="instagram"><i class="fa-brands fa-instagram"></i> Instagram:</label>
                    <input type="text" id="instagram" name="instagram"
                        value="<?php echo sanitizeInput($userFields['instagram']); ?>"
                        placeholder="https://instagram.com/yourprofile">
                </div>

                <div>
                    <label for="deviantart"><i class="fa-brands fa-deviantart"></i> DeviantArt:</label>
                    <input type="text" id="deviantart" name="deviantart"
                        value="<?php echo sanitizeInput($userFields['deviantart']); ?>"
                        placeholder="https://deviantart.com/yourprofile">
                </div>
            </div>
        </div>

        <div id="likesDislikes" class="tab-content">
            <div>
                <label for="likes"><i class="fa fa-thumbs-up"></i> Likes:</label>
                <textarea rows="10" id="likes" name="likes"
                    class="tinymce"><?php echo sanitizeInput($userFields['likes']); ?></textarea>
            </div>

            <br>

            <div>
                <label for="dislikes"><i class="fa fa-thumbs-down"></i> Dislikes:</label>
                <textarea rows="10" id="dislikes" name="dislikes"
                    class="tinymce"><?php echo sanitizeInput($userFields['dislikes']); ?></textarea>
            </div>
        </div>

        <button type="submit">Update Profile</button>
    </form>

</div>

<script>
    function openTab(evt, tabName) {
        // Get all tab content elements
        var tabContents = document.querySelectorAll('.tab-content');
        // Remove active class from all tab contents
        tabContents.forEach(function (content) {
            content.classList.remove('active');
        });

        // Get all tab links (a tags)
        var tabLinks = document.querySelectorAll('.tab-link');
        // Remove active class from all tab links
        tabLinks.forEach(function (link) {
            link.classList.remove('active');
        });

        // Show the clicked tab's content and add active class to the link
        document.getElementById(tabName).classList.add('active');
        evt.currentTarget.classList.add('active');
    }

    // Set the default tab to be the first one
    document.addEventListener('DOMContentLoaded', function () {
        // By default, open the first tab (personalInfo)
        document.querySelector('.tab-link').click();
    });
</script>

<!-- Include footer file for the page layout -->
<?php include_once __DIR__ . '/footer.php'; ?>