<?php
/**
 * Shortens an HTML string to a specified maximum character length, preserving HTML structure.
 * The function ensures that HTML tags are properly opened and closed, even in truncated content.
 * 
 * @param string $html The HTML string to shorten.
 * @param int $maxLength The maximum number of characters to retain in the shortened string.
 *                        This count only applies to text content, not including HTML tags.
 * 
 * @return string The truncated HTML string with valid HTML structure. An ellipsis (...) is
 *                added at the end if the content is truncated.
 */
function shortenString($html, $maxLength = 30)
{
    // Handle empty or non-string input gracefully
    if (empty($html) || !is_string($html)) {
        return ''; // Return an empty string if the input is empty
    }

    // Load the HTML into a DOMDocument object
    $dom = new DOMDocument();
    @$dom->loadHTML(mb_convert_encoding($html, 'HTML-ENTITIES', 'UTF-8'));

    // Initialize a variable to keep track of the current length and result
    $currentLength = 0;
    $result = '';

    // Recursive function to iterate through nodes and add them to the result
    $traverseNodes = function ($node) use (&$traverseNodes, &$currentLength, $maxLength, &$result) {
        if ($currentLength >= $maxLength) {
            return;
        }

        // Handle text nodes
        if ($node->nodeType === XML_TEXT_NODE) {
            $text = $node->nodeValue;
            $remainingLength = $maxLength - $currentLength;

            // Check if this text segment fits within the remaining length
            if (mb_strlen($text) <= $remainingLength) {
                $result .= htmlspecialchars($text);
                $currentLength += mb_strlen($text);
            } else {
                $result .= htmlspecialchars(mb_substr($text, 0, $remainingLength)) . '...';
                $currentLength = $maxLength;
            }
        }
        // Handle element nodes
        elseif ($node->nodeType === XML_ELEMENT_NODE) {
            // Open the tag
            $result .= '<' . $node->nodeName;
            foreach ($node->attributes as $attr) {
                $result .= ' ' . $attr->nodeName . '="' . htmlspecialchars($attr->nodeValue) . '"';
            }
            $result .= '>';

            // Recursively traverse child nodes
            foreach ($node->childNodes as $childNode) {
                $traverseNodes($childNode);
                if ($currentLength >= $maxLength) {
                    break;
                }
            }

            // Close the tag
            $result .= '</' . $node->nodeName . '>';
        }
    };

    // Start traversing from the body node to get the inner content
    $body = $dom->getElementsByTagName('body')->item(0);
    if ($body) {
        foreach ($body->childNodes as $node) {
            $traverseNodes($node);
            if ($currentLength >= $maxLength) {
                break;
            }
        }
    }

    return $result;
}

/**
 * Formats the given URL to ensure it has a valid scheme.
 *
 * @param string $url The URL to format.
 * @param string $scheme The scheme to prepend if the URL does not already have one. Default is 'http://'.
 * @return string The formatted URL with a valid scheme.
 */
function formatUrl($url, $scheme = 'http://')
{
    $url = strip_tags($url);
    // Check if the URL starts with http:// or https://
    if (stripos($url, 'http://') === 0 || stripos($url, 'https://') === 0) {
        return $url; // Return the URL as is if it already has a scheme
    }
    return $scheme . $url; // Otherwise, prepend the default scheme
}

/**
 * Sanitizes input by stripping all HTML tags and special characters.
 * It ensures that input is sanitized once, avoiding double sanitization.
 *
 * @param mixed $input The input string or array to sanitize.
 * @return mixed The sanitized string or array with HTML entities escaped.
 */
function sanitizeInput($input)
{
    if (is_array($input)) {
        return array_map('sanitizeInput', $input); // Recursively sanitize each item in the array
    }

    // If input is not a string, convert it to string and then sanitize
    if (!is_string($input)) {
        $input = (string) $input; // Convert to string
    }

    // First check: If input already has HTML entities (indicating prior sanitization)
    // Check for HTML entities, but exclude common cases like `&lt;`, `&gt;`, `&amp;`.
    $encoded = htmlspecialchars($input, ENT_QUOTES, 'UTF-8');

    // If it's already been encoded (it should be identical to `htmlspecialchars`)
    if ($input === $encoded) {
        // It wasn't encoded before, so we sanitize it
        return $encoded;
    }

    // If it is double-encoded (i.e., it's already been sanitized once)
    // Decode first, then sanitize it again to ensure correct encoding
    $decodedInput = html_entity_decode($input, ENT_QUOTES, 'UTF-8');

    // Now sanitize again
    return htmlspecialchars(trim($decodedInput), ENT_QUOTES, 'UTF-8');
}

/**
 * Sanitizes HTML input, allowing specific HTML tags and attributes.
 *
 * @param mixed $input The HTML input string or array to sanitize.
 * @return mixed The sanitized HTML string or array, allowing only specified tags and attributes.
 */
function sanitizeHtmlInput($input)
{
    // Define allowed tags, including structural and semantic HTML5 elements
    $allowedTags = '<p><strong><b><em><i><u><s><a><ul><ol><li><img><br><blockquote><code><hr><span>'
        . '<small><mark><sub><sup><time><cite><abbr><del><ins>'
        . '<div><section><article><aside><header><footer>';

    if (is_array($input)) {
        return array_map('sanitizeHtmlInput', $input); // Recursively sanitize each item in the array
    }

    // Sanitize the input
    $sanitized = strip_tags($input, $allowedTags);

    // Further sanitize links and images
    $sanitized = preg_replace(
        '/<a[^>]*href=["\']?([^"\'>]*)["\']?[^>]*>(.*?)<\/a>/i',
        '<a href="$1" target="_blank" rel="noopener noreferrer">$2</a>',
        $sanitized
    );
    $sanitized = preg_replace(
        '/<img[^>]*src=["\']?([^"\'>]*)["\']?[^>]*>/i',
        '<img src="$1" alt="User Image" loading="lazy">',
        $sanitized
    );

    return $sanitized;
}

/**
 * Generates the base URL for the application.
 *
 * @param string $scriptFolder The name of the script folder (default: SCRIPT_FOLDER constant).
 * @return string The base URL constructed from the current script's folder.
 */
function generateBaseUrl($scriptFolder = SCRIPT_FOLDER)
{
    $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') ? 'https://' : 'http://';
    $host = $_SERVER['HTTP_HOST'];

    // Get the request URI and remove the current script folder (simplify)
    $requestUri = rtrim(dirname($_SERVER['SCRIPT_NAME']), '/');

    // If you're inside the "simplify" folder, remove it from the base URL
    if (strpos($requestUri, SCRIPT_FOLDER) !== false) {
        $requestUri = dirname($requestUri);  // Go up one level to remove "/simplify"
    }

    // Construct the base URL and normalize it to prevent double slashes
    return normalizeUrl($protocol . $host . $requestUri . '/');
}

/**
 * Generates the full base URL for the application.
 *
 * This function constructs the base URL by retrieving the current
 * protocol, host, and script directory structure, ensuring it
 * works correctly whether the script is run in a root or subfolder.
 *
 * @return string The full base URL constructed from the current script's location.
 */
function getFullBaseUrl()
{
    // Determine the protocol (HTTP or HTTPS)
    $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') ? 'https://' : 'http://';

    // Get the host name
    $host = $_SERVER['HTTP_HOST'];

    // Get the current script's directory path
    $scriptPath = dirname($_SERVER['SCRIPT_NAME']); // Get the directory of the current script

    // Construct the full base URL, normalize it, and strip any unnecessary slashes
    return stripslashes(normalizeUrl($protocol . $host . rtrim($scriptPath, '/') . '/'));
}

/**
 * Constructs and returns the full URL of the current script, including any query parameters.
 * It handles various server configurations such as HTTPS, non-standard ports, and subfolders.
 * 
 * @return string The complete URL of the current executing script, including the protocol (http/https), host, path, and query parameters.
 */
function getScriptUrl()
{
    // Determine the protocol (HTTP/HTTPS)
    $protocol = 'http://';
    if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') {
        $protocol = 'https://';
    } elseif (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] === 'https') {
        $protocol = 'https://';
    }

    // Get the host (e.g., localhost or domain.com)
    $host = $_SERVER['HTTP_HOST'] ?? 'localhost';
    // Validate host (should not contain suspicious characters like newline or null byte)
    $host = filter_var($host, FILTER_SANITIZE_URL);

    // Handle non-standard ports if present
    $port = $_SERVER['SERVER_PORT'] ?? null;
    $port = ($port && !in_array($port, [80, 443])) ? ':' . $port : '';

    // Get the request URI (path without query string)
    $uri = $_SERVER['REQUEST_URI'] ?? $_SERVER['PHP_SELF'];
    // Sanitize the URI to prevent unwanted input (e.g., script injections)
    $uri = filter_var($uri, FILTER_SANITIZE_URL);

    // Parse the URI to extract the path and existing query parameters
    $parsedUrl = parse_url($uri);
    $path = $parsedUrl['path'] ?? '';
    $existingQuery = $parsedUrl['query'] ?? '';

    // Normalize the URI (remove any duplicate slashes and prevent directory traversal)
    $path = preg_replace('#/+#', '/', $path);
    $path = str_replace('..', '', $path);  // Prevent directory traversal

    // Ensure that the path includes the subfolder (if present), but remove the script directory part
    $scriptName = $_SERVER['SCRIPT_NAME'] ?? '';
    $scriptDir = dirname($scriptName); // Get script directory
    $uri = ltrim(str_replace($scriptDir, '', $path), '/');  // Clean up the URI to avoid redundancy

    // Rebuild the base URL without the query string
    $fullUrl = $protocol . rtrim($host, '/') . $port . rtrim($scriptDir, '/') . '/' . ltrim($uri, '/');

    // Parse the query string from the current request (e.g., ?alias=test)
    $query = $_SERVER['QUERY_STRING'] ?? '';
    if (!empty($query)) {
        parse_str($query, $queryParams); // Parse query string into array
        if (!empty($existingQuery)) {
            parse_str($existingQuery, $existingParams);
            $queryParams = array_merge($existingParams, $queryParams);
        }
        // Sanitize query parameters
        foreach ($queryParams as $key => $value) {
            $queryParams[$key] = filter_var($value, FILTER_SANITIZE_SPECIAL_CHARS);
        }
        // Build the cleaned-up query string without duplicates
        $query = http_build_query($queryParams);
        if (!empty($query)) {
            $fullUrl .= '?' . $query;
        }
    }

    return $fullUrl;
}

/**
 * Normalizes the URL by removing any double slashes, except in the protocol.
 *
 * @param string $url The URL to normalize.
 * @return string The normalized URL.
 */
function normalizeUrl($url)
{
    return preg_replace('#(?<!:)//+#', '/', $url);
}

/**
 * Ensures that required directories exist, creating them if necessary
 * and optionally adding an index.php file in each to prevent directory listing.
 *
 * @param array|string $dirs An array of directory paths or a single directory path to check and create if necessary.
 * @param array $options Optional parameters to customize behavior:
 * - 'createIndex' (bool): If true, creates an index.php file in each directory. Defaults to true.
 *
 * @return void
 *
 * @example
 * // Create multiple directories and add index.php by default
 * ensureRequiredDirectories(['/path/to/dir1', '/path/to/dir2']);
 *
 * @example
 * // Create a single directory without adding index.php
 * ensureRequiredDirectories('/path/to/dir3', ['createIndex' => false]);
 */
function ensureRequiredDirectories($dirs, array $options = [])
{
    // Normalize $dirs to an array if it's a string
    if (is_string($dirs)) {
        $dirs = [$dirs];
    }

    // Set default options
    $createIndex = isset($options['createIndex']) ? $options['createIndex'] : true; // Default to true

    foreach ($dirs as $dir) {
        // Check if the directory exists
        if (!is_dir($dir)) {
            // Attempt to create the directory
            if (!mkdir($dir, 0755, true)) {
                die("Failed to create directory: $dir\n");
            }
            // Create an empty index.php file to prevent directory listing if requested
            if ($createIndex) {
                file_put_contents($dir . '/index.php', "<?php // Silence is golden?\n");
            }
        }
    }
}

/**
 * Redirects the user to a specified URL, optionally with parameters and a delay.
 * - Uses PHP header redirect if possible (preferred method for speed and reliability).
 * - Falls back to JavaScript redirect if headers have already been sent.
 * - Uses a meta tag redirect as a last resort for environments where JavaScript is disabled.
 *
 * @param string $url The base URL to redirect to. Must be a valid URL.
 * @param array  $params Optional associative array of query parameters.
 * @param int    $delay Optional delay in seconds before redirecting. Default is 0 (immediate redirect).
 * @return void
 */
function safeRedirect($url, array $params = [], $delay = 0)
{
    // Build query string from params
    if (!empty($params)) {
        $queryString = http_build_query($params);
        $url .= '?' . $queryString; // Append query string to the URL
    }

    // Determine the method of redirection
    if ($delay > 0) {
        // Use Refresh header for PHP header redirect
        if (!headers_sent()) {
            header("Refresh: $delay; url=" . $url);
            exit;
        } else {
            outputRedirectJs($url, $delay);
        }
    } else {
        // Immediate redirection
        if (!headers_sent()) {
            header("Location: " . $url);
            exit;
        } else {
            outputRedirectJs($url);
        }
    }
}

/**
 * Outputs a JavaScript redirect to the specified URL.
 *
 * @param string $url The URL to redirect to.
 * @param int|null $delay Optional delay in seconds before redirecting. Default is null for immediate redirect.
 * @return void
 */
function outputRedirectJs($url, $delay = null)
{
    if ($delay) {
        // Delayed JavaScript redirect
        echo '<script type="text/javascript">';
        echo 'setTimeout(function() { window.location.href = ' . json_encode($url) . '; }, ' . ($delay * 1000) . ');';
        echo '</script>';
    } else {
        // Immediate JavaScript redirect
        echo '<script type="text/javascript">';
        echo 'window.location.href = ' . json_encode($url) . ';';
        echo '</script>';
    }

    // Meta tag redirect for users without JavaScript enabled
    echo '<noscript><meta http-equiv="refresh" content="' . (int) ($delay ?? 0) . ';url=' . htmlspecialchars($url, ENT_QUOTES, 'UTF-8') . '"></noscript>';
    exit; // Exit after outputting redirect
}

/**
 * Formats a date string into a specified format.
 *
 * @param string $date The date string to format. Expected in a format that DateTime can parse.
 * @param string $format The desired date format, using PHP date() format syntax. Default is 'Y-m-d H:i:s'.
 * @return string|null The formatted date string, or null if the input date is invalid.
 *
 * @example
 * echo formatDate('2023-10-15', 'F j, Y'); // Outputs: October 15, 2023
 * echo formatDate('2023-10-15 14:30:00', 'l, F j, Y \a\t g:i A'); // Outputs: Sunday, October 15, 2023 at 2:30 PM
 */
function formatDate($date, $format = 'Y-m-d H:i:s')
{
    try {
        // Create a DateTime object from the input date
        $dateTime = new DateTime($date);

        // Return the date formatted according to the specified format
        return $dateTime->format($format);
    } catch (Exception $e) {
        // If the date is invalid, return null
        return null;
    }
}

/**
 * Replaces smiley codes in content with corresponding image tags.
 *
 * @param string $content Text content to parse for smiley codes.
 * @return string Content with smiley codes replaced by image tags, or unmodified if no smileys are available.
 */
function parseSmileys($content)
{
    // Check if Smileys module is loaded
    if (!ModuleLoader::requiresModules(["Smileys"], null, false, false)) {
        return $content; // Return unmodified content if module is not available
    }

    // Continue with the rest of the logic to replace smileys
    $smileys = new Smileys();
    $allSmileys = $smileys->getAllSmileys();

    if (empty($allSmileys)) {
        return $content;
    }

    // Replace each smiley code in content with its image tag
    foreach ($allSmileys as $smiley) {
        $smileyCode = preg_quote($smiley['metadata']['smileyCode'], '/');
        $smileyImageUrl = normalizeUrl(generateBaseUrl() . basename(CONTENT_DIR) . Smileys::IMAGE_UPLOAD_DIR . ltrim($smiley['imageName'], '/'));

        $imgTag = '<img src="' . htmlspecialchars($smileyImageUrl) . '" alt="' . htmlspecialchars($smiley['metadata']['smileyCode']) . '" class="smiley-icon" />';
        $pattern = '/(?<=\s|^|>)' . $smileyCode . '(?=\s|$|[.!?,;]|<)/u';
        $content = preg_replace($pattern, $imgTag, $content);
    }

    return $content;
}

/**
 * Get the site title, with an optional dynamic page title suffix.
 *
 * @param string $pageTitleSuffix (Optional) The dynamic page title to append to the site title.
 * @return string The final page title.
 */
function getSiteTitle($pageTitleSuffix = '')
{
    $baseTitle = '';

    // Get the base title from the settings
    $baseTitle = ModuleLoader::getSetting('Default', 'titleTag');

    // Fallback if no setting is available
    $baseTitle = $baseTitle ? htmlspecialchars($baseTitle) : '';

    // Check if there's a dynamic page title suffix (e.g., blog post title)
    if (!empty($pageTitleSuffix)) {
        // Add a hyphen and the dynamic page title
        return $baseTitle . ' - ' . htmlspecialchars($pageTitleSuffix);
    }

    // Return the base title if no dynamic title is provided
    return $baseTitle;
}

/**
 * Calculates the age from a given birthdate.
 *
 * This function takes a birthdate as a string and calculates the age in years.
 *
 * @param string $birthdate The birthdate as a string (e.g., '2000-01-01').
 * 
 * @return int The calculated age in years.
 * 
 * @throws InvalidArgumentException If the birthdate is not a valid date or is in an incorrect format.
 */
function calculateAge(string $birthdate): int
{
    // Try to create a DateTime object using the raw birthdate (assumed to be 'Y-m-d' format)
    $birthDate = DateTime::createFromFormat('Y-m-d', $birthdate);

    // Check if the birthdate is valid
    if (!$birthDate || $birthDate->format('Y-m-d') !== $birthdate) {
        throw new InvalidArgumentException("Invalid birthdate format. Expected format: 'Y-m-d'.");
    }

    // Get the current date
    $today = new DateTime();

    // Calculate the difference between the birthdate and the current date in years
    $age = $today->diff($birthDate)->y;

    return $age;
}
