<?php
/**
 * Initializes and loads the necessary modules and classes for the application.
 * 
 * This script performs the following tasks:
 * - Verifies user authentication.
 * - Dynamically loads requested modules and files.
 * - Enforces access control based on user roles.
 */

// Load necessary classes and functions
require_once __DIR__ . '/autoload.php';
require_once __DIR__ . '/functions.php';
require_once __DIR__ . '/config.php';

// Initialize the login class
$auth = new SimpleLogin();

// Ensure user is logged in
$auth->requireLogin();

// Load modules and register hooks
$modules = ModuleLoader::loadModules();

// Include header file for the page layout
include_once __DIR__ . '/header.php';

// Sanitize and validate the module and file parameters from the URL
$moduleName = filter_input(INPUT_GET, 'module', FILTER_SANITIZE_FULL_SPECIAL_CHARS) ?: '';
$fileName = filter_input(INPUT_GET, 'file', FILTER_SANITIZE_FULL_SPECIAL_CHARS) ?: '';

echo '<div class="content">'; // Start the content wrapper
$currentModuleConfig = null;

// Find the current module configuration based on the loaded module name
foreach ($modules as $config) {
    if ($config['module_name'] === $moduleName) {
        $currentModuleConfig = $config;
        break;
    }
}

// Initialize action links array
$actionLinks = [];

// Get the user's role
$userRole = $_SESSION['role'] ?? null;

// Check if we found the module configuration and if it has links
if ($currentModuleConfig && !empty($currentModuleConfig['links'])) {
    // Loop through links and collect action links for the current module
    foreach ($currentModuleConfig['links'] as $link) {
        // Only include action links that are not file-specific
        if (!empty($link['hook']) && $link['hook'] === 'action_links') {
            // Ensure user has access to the link based on their role, or if they are an admin
            if (in_array($userRole, $link['access']) || $auth->isAdmin()) {
                // Determine if the current link matches the file parameter
                $activeClass = ($fileName === $link['file']) ? 'active' : '';

                // Construct the action link HTML
                $actionLinks[] = sprintf(
                    '<a href="?module=%s&file=%s" class="%s %s"><i class="%s"></i> %s</a>',
                    htmlspecialchars($moduleName, ENT_QUOTES),
                    htmlspecialchars($link['file'], ENT_QUOTES),
                    htmlspecialchars($link['class'], ENT_QUOTES),
                    $activeClass, // Add the 'active' class if it matches the file
                    htmlspecialchars($link['icon_class'], ENT_QUOTES),
                    htmlspecialchars($link['title'], ENT_QUOTES)
                );
            }
        }
    }

    // Render action links if they exist
    if (!empty($actionLinks)) {
        echo '<div class="action-links">';
        echo implode("\n", array_unique($actionLinks)); // Render each unique action link
        echo '</div>'; // End action links
    }
}

// Load the specific module file using the ModuleLoader class
try {
    if (!empty($moduleName) && !empty($fileName)) {
        // Validate module name and file name using regex
        if (!preg_match('/^[a-zA-Z0-9_]+$/', $moduleName) || !preg_match('/^[a-zA-Z0-9_.]+$/', $fileName)) {
            throw new InvalidArgumentException("Invalid module or file name.");
        }

        ModuleLoader::loadModuleFile($moduleName, $fileName);
    } else {
        throw new InvalidArgumentException("Module and file names cannot be empty.");
    }
} catch (Exception $e) {
    // Handle the exception: log it and show an error message
    echo '<div class="error">';
    echo '<strong>Critical Error:</strong> ' . htmlspecialchars($e->getMessage(), ENT_QUOTES, 'UTF-8');
    echo '</div>';
}

echo '</div>'; // End the content wrapper

// Include footer file for the page layout
include_once __DIR__ . '/footer.php';
?>
